@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/clients/ra.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/clients/ra.c#77 $
+ * $DateTime: 2014/06/16 21:25:12 $
+ * $Change: 2869 $
  */
 
 /*
@@ -107,9 +107,49 @@ ArgusClientInit (struct ArgusParserStruct *parser)
 
       if (parser->Pctflag) {
          parser->ArgusPassNum = 2;
-         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, "srcid")) == NULL)
+         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, "srcid", ARGUS_RECORD_AGGREGATOR)) == NULL)
             ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
       }
+
+
+      if (parser->dflag) {
+         int pid;
+
+         if (parser->Sflag)
+            parser->ArgusReliableConnection++;
+
+         ArgusLog(LOG_WARNING, "started");
+         if (chdir ("/") < 0)
+            ArgusLog (LOG_ERR, "Can't chdir to / %s", strerror(errno));
+
+         if ((pid = fork ()) < 0) {
+            ArgusLog (LOG_ERR, "Can't fork daemon %s", strerror(errno));
+         } else {
+            if (pid) {
+               struct timespec ts = {0, 20000000};
+               int status;
+               nanosleep(&ts, NULL);   
+               waitpid(pid, &status, WNOHANG);
+               if (kill(pid, 0) < 0) {
+                  exit (1);
+               } else
+                  exit (0);
+            } else {
+               FILE *tmpfile;
+
+               parser->ArgusSessionId = setsid();
+               if ((tmpfile = freopen ("/dev/null", "r", stdin)) == NULL)
+                  ArgusLog (LOG_ERR, "Cannot map stdout to /dev/null");
+
+               if ((tmpfile = freopen ("/dev/null", "a+", stdout)) == NULL)
+                  ArgusLog (LOG_ERR, "Cannot map stdout to /dev/null");
+
+               if ((tmpfile = freopen ("/dev/null", "a+", stderr)) == NULL)
+                  ArgusLog (LOG_ERR, "Cannot map stderr to /dev/null");
+            }
+         }
+      }
+
       parser->RaInitialized++;
    }
 }
@@ -123,7 +163,7 @@ RaParseComplete (int sig)
       if (!ArgusParser->RaParseCompleting++) {
 
          if (ArgusParser->Aflag) {
-#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(HAVE_SOLARIS)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(ARGUS_SOLARIS)
             printf (" Totalrecords %-8lld  TotalMarRecords %-8lld  TotalFarRecords %-8lld TotalPkts %-8lld TotalBytes %-8lld\n",
                           ArgusParser->ArgusTotalRecords, ArgusParser->ArgusTotalMarRecords, ArgusParser->ArgusTotalFarRecords,
                           ArgusParser->ArgusTotalPkts, ArgusParser->ArgusTotalBytes);
@@ -196,7 +236,7 @@ ArgusClientTimeout ()
    }
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (2, "ArgusClientTimeout()\n");
+   ArgusDebug (6, "ArgusClientTimeout()\n");
 #endif
 }
 
@@ -249,7 +289,7 @@ usage ()
    fprintf (stdout, "            [io]<start-end>     process this inclusive range of matching records.\n");
    fprintf (stdout, "            [io]<start+num>     process this number of matching records, starting at start.\n");
    fprintf (stdout, "         -p <digits>            print fractional time with <digits> precision.\n");
-   fprintf (stdout, "         -q                              quiet mode. don't print record outputs.\n");
+   fprintf (stdout, "         -q                     quiet mode. don't print record outputs.\n");
    fprintf (stdout, "         -r <[type:]file[::ostart[:ostop]] ...>\n");
    fprintf (stdout, "                                read <type> data from <file>. '-' denotes stdin.\n");
    fprintf (stdout, "                                types supported are argus (default), 'cisco' and 'ft' (flow-tools)\n");
@@ -266,8 +306,8 @@ usage ()
    fprintf (stdout, "                                smac, dmac, dir, [s|d]intpkt, [s|d]jit, state, suser, duser,\n");
    fprintf (stdout, "                                swin, dwin, trans, srng, erng, stcpb, dtcpb, tcprtt, inode,\n");
    fprintf (stdout, "                                offset, smaxsz, dmaxsz, sminsz, dminsz\n");
-   fprintf (stdout, "         -S <host[:port]>       specify remote argus and optional port number\n");
-   fprintf (stdout, "         -S <URI://host[:port]> \n");
+   fprintf (stdout, "         -S <[user[:pass]@]host[:port]>       specify remote argus and optional port number\n");
+   fprintf (stdout, "         -S <URI://[user[:pass]@]host[:port]> \n");
    fprintf (stdout, "             URI : argus-udp    \n");
    fprintf (stdout, "                   argus-tcp    \n");
    fprintf (stdout, "                   argus        \n");
@@ -307,12 +347,15 @@ RaProcessRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *arg
 
       case ARGUS_NETFLOW:
       case ARGUS_FAR: {
-         struct ArgusMetricStruct *metric = (void *)argus->dsrs[ARGUS_METRIC_INDEX];
-         if (metric != NULL) {
-            parser->ArgusTotalPkts  += metric->src.pkts;
-            parser->ArgusTotalPkts  += metric->dst.pkts;
-            parser->ArgusTotalBytes += metric->src.bytes;
-            parser->ArgusTotalBytes += metric->dst.bytes;
+         if (ArgusParser->Aflag) {
+            struct ArgusMetricStruct *metric = (void *)argus->dsrs[ARGUS_METRIC_INDEX];
+
+            if (metric != NULL) {
+               parser->ArgusTotalPkts  += metric->src.pkts;
+               parser->ArgusTotalPkts  += metric->dst.pkts;
+               parser->ArgusTotalBytes += metric->src.bytes;
+               parser->ArgusTotalBytes += metric->dst.bytes;
+            }
          }
 
          if ((parser->RaMonMode) || (parser->RaUniMode)) {
@@ -350,8 +393,13 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
 {
    static char buf[MAXSTRLEN];
             
-   if (parser->RaUniMode)
+   if (parser->RaUniMode) {
+      struct ArgusMetricStruct *metric = (void *)argus->dsrs[ARGUS_METRIC_INDEX];
+
       ArgusUniDirectionalRecord (argus);
+      if (metric->src.pkts == 0)
+         return;
+   }
 
    switch (parser->ArgusPassNum)  {
       case 2: {
