@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -35,9 +35,9 @@
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/clients/rabins.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/clients/rabins.c#89 $
+ * $DateTime: 2014/06/16 21:25:12 $
+ * $Change: 2869 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -59,6 +59,7 @@
 #include <argus_client.h>
 #include <argus_main.h>
 #include <argus_filter.h>
+#include <argus_output.h>
 
 #include <rabins.h>
 #include <rasplit.h>
@@ -171,17 +172,31 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                else
                if (!(strncasecmp (mode->mode, "oui", 3)))
                   parser->ArgusPrintEthernetVendors++;
+               else
+               if (!(strncasecmp (mode->mode, "poll", 4)))
+                  parser->RaPollMode++;
+               else
+               if (!(strncasecmp (mode->mode, "uni", 3)))
+                  parser->RaUniMode++;
+               else
+               if (!(strncasecmp (mode->mode, "man", 3)))
+                  parser->ArgusPrintMan = 1;
+               else
+               if (!(strncasecmp (mode->mode, "noman", 5)))
+                  parser->ArgusPrintMan = 0;
                else {
-                  for (i = 0, ind = -1; i < ARGUSSPLITMODENUM; i++) {
+                  int done = 0;
+                  for (i = 0, ind = -1; !(done) && (i < ARGUSSPLITMODENUM); i++) {
                      if (!(strncasecmp (mode->mode, RaSplitModes[i], 3))) {
                         ind = i;
                         switch (ind) {
                            case ARGUSSPLITTIME:
                            case ARGUSSPLITSIZE:
-                           case ARGUSSPLITCOUNT:
+                           case ARGUSSPLITCOUNT: {
                               if ((mode = mode->nxt) == NULL)
                                  usage();
-                              break;
+                              done++;
+                           }
                         }
                      }
                   }
@@ -213,7 +228,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                               nadp->RaStartTmStruct.tm_hour = 0;
                               nadp->RaStartTmStruct.tm_mday = 1;
                               nadp->RaStartTmStruct.tm_mon = 0;
-//                            nadp->start.tv_sec = mktime(&nadp->RaStartTmStruct);
+                              tsec= mktime(&nadp->RaStartTmStruct);
                               nadp->size = nadp->value*3600.0*24.0*7.0*52.0*1000000LL;
                               break;
 
@@ -225,7 +240,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                               nadp->RaStartTmStruct.tm_hour = 0;
                               nadp->RaStartTmStruct.tm_mday = 1;
                               nadp->RaStartTmStruct.tm_mon = 0;
-//                            nadp->start.tv_sec = mktime(&nadp->RaStartTmStruct);
+                              tsec = mktime(&nadp->RaStartTmStruct);
                               nadp->size = nadp->value*3600.0*24.0*7.0*4.0*1000000LL;
                               break;
 
@@ -237,7 +252,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                               nadp->RaStartTmStruct.tm_hour = 0;
                               nadp->RaStartTmStruct.tm_mday = 1;
                               nadp->RaStartTmStruct.tm_mon = 0;
-//                            nadp->start.tv_sec = mktime(&nadp->RaStartTmStruct);
+                              tsec = mktime(&nadp->RaStartTmStruct);
                               nadp->size = nadp->value*3600.0*24.0*7.0*1000000LL;
                               break;
 
@@ -247,7 +262,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                               nadp->RaStartTmStruct.tm_sec = 0;
                               nadp->RaStartTmStruct.tm_min = 0;
                               nadp->RaStartTmStruct.tm_hour = 0;
-//                            nadp->start.tv_sec = mktime(&nadp->RaStartTmStruct);
+                              tsec = mktime(&nadp->RaStartTmStruct);
                               nadp->size = nadp->value*3600.0*24.0*1000000LL;
                               break;
 
@@ -256,7 +271,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                               localtime_r(&tsec, &nadp->RaStartTmStruct);
                               nadp->RaStartTmStruct.tm_sec = 0;
                               nadp->RaStartTmStruct.tm_min = 0;
-//                            nadp->start.tv_sec = mktime(&nadp->RaStartTmStruct);
+                              tsec = mktime(&nadp->RaStartTmStruct);
                               nadp->size = nadp->value*3600.0*1000000LL;
                               break;
 
@@ -264,8 +279,8 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                               nadp->qual = ARGUSSPLITMINUTE;
                               localtime_r(&tsec, &nadp->RaStartTmStruct);
                               nadp->RaStartTmStruct.tm_sec = 0;
-//                            nadp->start.tv_sec = mktime(&nadp->RaStartTmStruct);
-                              nadp->size = nadp->value*60.0*1000000LL;
+                              tsec = nadp->value*60.0*1000000LL;
+                              nadp->size = tsec;
                               break;
                            }
 
@@ -336,8 +351,12 @@ ArgusClientInit (struct ArgusParserStruct *parser)
          }
       }
 
-      if ((RaBinProcess->size  = nadp->size) == 0)
-         ArgusLog (LOG_ERR, "ArgusClientInit: no bin size specified");
+      if ((RaBinProcess->size  = nadp->size) == 0) {
+//       ArgusLog (LOG_ERR, "ArgusClientInit: no bin size specified");
+         nadp->value = 0xEFFFFFFF;
+         nadp->size = nadp->value * 1000000LL;
+         RaBinProcess->size  = nadp->size;
+      }
 
       if (!(nadp->value))
          nadp->value = 1;
@@ -451,8 +470,13 @@ ArgusClientInit (struct ArgusParserStruct *parser)
          if ((parser->ArgusAggregator = ArgusParseAggregator(parser, parser->ArgusFlowModelFile, NULL)) == NULL)
             ArgusLog (LOG_ERR, "ArgusClientInit: ArgusParseAggregator error");
 
+         if (parser->ArgusAggregatorFile != NULL) 
+            free(parser->ArgusAggregatorFile);
+
+         parser->ArgusAggregatorFile = strdup(parser->ArgusFlowModelFile);
+
       } else
-         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL)) == NULL)
+         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL, ARGUS_RECORD_AGGREGATOR)) == NULL)
             ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
 
       if (RaBinProcess->size == 0)
@@ -475,6 +499,45 @@ ArgusClientInit (struct ArgusParserStruct *parser)
       if ((parser->ArgusMaskList) == NULL)
          if (parser->ArgusAggregator->correct != NULL) 
             parser->ArgusReverse = 1;
+
+
+      if (parser->dflag) {
+         int pid;
+
+         if (parser->Sflag)
+            parser->ArgusReliableConnection++;
+
+         ArgusLog(LOG_WARNING, "started");
+         if (chdir ("/") < 0)
+            ArgusLog (LOG_ERR, "Can't chdir to / %s", strerror(errno));
+
+         if ((pid = fork ()) < 0) {
+            ArgusLog (LOG_ERR, "Can't fork daemon %s", strerror(errno));
+         } else {
+            if (pid) {
+               struct timespec ts = {0, 20000000};
+               int status;
+               nanosleep(&ts, NULL);   
+               waitpid(pid, &status, WNOHANG);
+               if (kill(pid, 0) < 0) {
+                  exit (1);
+               } else
+                  exit (0);
+            } else {
+               FILE *tmpfile;
+
+               parser->ArgusSessionId = setsid();
+               if ((tmpfile = freopen ("/dev/null", "r", stdin)) == NULL)
+                  ArgusLog (LOG_ERR, "Cannot map stdout to /dev/null");
+
+               if ((tmpfile = freopen ("/dev/null", "a+", stdout)) == NULL)
+                  ArgusLog (LOG_ERR, "Cannot map stdout to /dev/null");
+
+               if ((tmpfile = freopen ("/dev/null", "a+", stderr)) == NULL)
+                  ArgusLog (LOG_ERR, "Cannot map stderr to /dev/null");
+            }
+         }
+      }
    }
 
 #ifdef ARGUSDEBUG
@@ -484,12 +547,11 @@ ArgusClientInit (struct ArgusParserStruct *parser)
 
 
 void RaArgusInputComplete (struct ArgusInput *input) { return; }
+int RaDeleteBinProcess(struct ArgusParserStruct *, struct RaBinProcessStruct *);
 
 void
 RaParseComplete (int sig)
 {
-   int startsecs = 0, endsecs = 0, i;
-   
    if (sig >= 0) {
       switch (sig) {
          case SIGINT:
@@ -498,99 +560,13 @@ RaParseComplete (int sig)
       }
       if (!(ArgusParser->RaParseCompleting++)) {
          if (RaBinProcess != NULL) {
-            struct RaBinStruct *bin = NULL;
-            struct ArgusRecordStruct *ns = NULL;
-            int max = ((ArgusParser->tflag && ArgusParser->RaExplicitDate) ? RaBinProcess->nadp.count : RaBinProcess->max) + 1;
-
-            char stimebuf[128], dtimebuf[128], etimebuf[128];
-            int bins;
-
-            if (RaBinProcess->array != NULL) {
-               if (!(ArgusParser->tflag)) {
-                  for (i = 0; i < max; i++) {
-                     if ((bin = RaBinProcess->array[i]) != NULL) {
-                        if (startsecs == 0)
-                           startsecs = bin->stime.tv_sec;
-                        endsecs = bin->etime.tv_sec;
-                     }
-                  }
-
-                  RaBinProcess->startpt.tv_sec = startsecs;
-                  RaBinProcess->scalesecs      = (endsecs - startsecs) + (RaBinProcess->size / 1000000);
-
-                  if ((ArgusParser->RaEndTime.tv_sec >  RaBinProcess->endpt.tv_sec) ||
-                           ((ArgusParser->RaEndTime.tv_sec == RaBinProcess->endpt.tv_sec) &&
-                            (ArgusParser->RaEndTime.tv_usec > RaBinProcess->endpt.tv_usec)))
-                     ArgusParser->RaEndTime = RaBinProcess->endpt;
-
-               } else {
-                  RaBinProcess->startpt.tv_sec  = ArgusParser->startime_t.tv_sec;
-                  RaBinProcess->startpt.tv_usec = ArgusParser->startime_t.tv_usec;
-                  RaBinProcess->endpt.tv_sec    = ArgusParser->lasttime_t.tv_sec;
-                  RaBinProcess->endpt.tv_usec   = ArgusParser->lasttime_t.tv_usec;
-                  RaBinProcess->scalesecs       = ArgusParser->lasttime_t.tv_sec - ArgusParser->startime_t.tv_sec;
-                  RaBinProcess->start           = ArgusParser->startime_t.tv_sec * 1000000LL;
-               }
-
-               if ((ArgusParser->ArgusWfileList == NULL) && (ArgusParser->Gflag)) {
-                  if (ArgusParser->Hstr != NULL) {
-
-                  } else {
-
-                     ArgusPrintTime(ArgusParser, stimebuf, &RaBinProcess->startpt);
-                     ArgusPrintTime(ArgusParser, dtimebuf, &RaBinProcess->endpt);
-                     ArgusPrintTime(ArgusParser, etimebuf, &ArgusParser->RaEndTime);
-
-                     stimebuf[strlen(stimebuf) - 1] = '\0';
-                     dtimebuf[strlen(dtimebuf) - 1] = '\0';
-                     etimebuf[strlen(etimebuf) - 1] = '\0';
-
-                     printf ("StartTime=%s\n", stimebuf);
-                     printf ("StopTime=%s\n",  dtimebuf);
-                     printf ("LastTime=%s\n",  etimebuf);
-                     printf ("Seconds=%d\n", RaBinProcess->scalesecs);
-                     printf ("BinSize=%1.*f\n", ArgusParser->pflag, (RaBinProcess->size * 1.0)/1000000);
-                     bins = ((RaBinProcess->scalesecs + (RaBinProcess->size/1000000 - 1))/(RaBinProcess->size / 1000000));
-                     printf ("Bins=%d\n", bins);
-                  }
-               }
-            }
-
-            for (i = RaBinProcess->index; i < max; i++) {
-               if ((RaBinProcess->array != NULL) && ((bin = RaBinProcess->array[i]) != NULL)) {
-                  struct ArgusAggregatorStruct *agg = bin->agg;
-                  while (agg) {
-                     int rank = 1;
-                     ArgusSortQueue(ArgusSorter, agg->queue);
-                     while ((ns = (struct ArgusRecordStruct *) ArgusPopQueue(agg->queue, ARGUS_NOLOCK)) != NULL) {
-                        ns->rank = rank++;
-                        if ((ArgusParser->eNoflag == 0 ) || ((ArgusParser->eNoflag >= ns->rank) && (ArgusParser->sNoflag <= ns->rank)))
-                           RaSendArgusRecord (ns);
-                        ArgusDeleteRecordStruct(ArgusParser, ns);
-                     }
-                     agg = agg->nxt;
-                  }
-
-               } else {
-                  if (RaBinProcess->nadp.zero && ((i >= RaBinProcess->index) && 
-                          ((((i - RaBinProcess->index) * 1.0) * RaBinProcess->size) < (RaBinProcess->scalesecs * 1000000LL)))) {
-                     long long tval = RaBinProcess->start + (RaBinProcess->size * (i - RaBinProcess->index));
-
-                     ns = ArgusGenerateRecordStruct(NULL, NULL, NULL);
-
-                     ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.start.tv_sec  = tval / 1000000;
-                     ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.start.tv_usec = tval % 1000000;
-
-                     tval += RaBinProcess->size;
-                     ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.end.tv_sec    = tval / 1000000;;
-                     ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.end.tv_usec   = tval % 1000000;
-
-                     RaSendArgusRecord (ns);
-                  }
-               }
-            }
+            RaDeleteBinProcess(ArgusParser, RaBinProcess);
+            RaBinProcess = NULL;
          }
 
+         if (ArgusSorter != NULL)
+            ArgusDeleteSorter(ArgusSorter);
+
 #ifdef ARGUSDEBUG
          ArgusDebug (2, "RaParseComplete(caught signal %d)\n", sig);
 #endif
@@ -684,6 +660,20 @@ ArgusClientTimeout ()
          if (rbps->array != NULL) {
             if ((bin = rbps->array[rbps->index]) != NULL) {
                struct ArgusAggregatorStruct *agg = bin->agg;
+               int tcnt = 0;
+
+               if (ArgusParser->ArgusGenerateManRecords) {
+                  struct ArgusRecordStruct *man = ArgusGenerateStatusMarRecord (NULL, ARGUS_START);
+                  struct ArgusRecord *rec = (struct ArgusRecord *)man->dsrs[0];
+                  rec->argus_mar.startime.tv_sec  = bin->stime.tv_sec;
+                  rec->argus_mar.startime.tv_usec = bin->stime.tv_usec;
+                  rec->argus_mar.now.tv_sec       = bin->stime.tv_sec;
+                  rec->argus_mar.now.tv_usec      = bin->stime.tv_usec;
+
+                  RaSendArgusRecord (man);
+                  ArgusDeleteRecordStruct(ArgusParser, man);
+               }
+
                while (agg) {
                   if (agg->queue->count) {
                      int cnt = 0;
@@ -692,9 +682,9 @@ ArgusClientTimeout ()
                      argus = ArgusCopyRecordStruct((struct ArgusRecordStruct *) agg->queue->array[0]);
 
                      if (nflag == 0)
-                        cnt = agg->queue->count;
+                        cnt = agg->queue->arraylen;
                      else
-                        cnt = nflag > agg->queue->count ? agg->queue->count : nflag;
+                        cnt = nflag > agg->queue->arraylen ? agg->queue->arraylen : nflag;
 
                      for (i = 1; i < cnt; i++)
                         ArgusMergeRecords (agg, argus, (struct ArgusRecordStruct *)agg->queue->array[i]);
@@ -706,33 +696,56 @@ ArgusClientTimeout ()
                            ((struct ArgusRecordStruct *)agg->queue->array[i])->rank = i + 1;
                         RaSendArgusRecord ((struct ArgusRecordStruct *) agg->queue->array[i]);
                      }
-
                      ArgusDeleteRecordStruct(ArgusParser, ArgusParser->ns);
 
                      while((argus = (struct ArgusRecordStruct *)ArgusPopQueue(agg->queue, ARGUS_LOCK)) != NULL)
                         ArgusDeleteRecordStruct(ArgusParser, argus);
-                  }
 
+                     tcnt += cnt;
+                  }
                   agg = agg->nxt;
                }
 
+               if (ArgusParser->ArgusGenerateManRecords) {
+                  struct ArgusRecordStruct *man = ArgusGenerateStatusMarRecord (NULL, ARGUS_STOP);
+                  struct ArgusRecord *rec = (struct ArgusRecord *)man->dsrs[0];
+                  rec->argus_mar.startime.tv_sec  = bin->etime.tv_sec;
+                  rec->argus_mar.startime.tv_usec = bin->etime.tv_usec;
+                  rec->argus_mar.now.tv_sec       = bin->etime.tv_sec;
+                  rec->argus_mar.now.tv_usec      = bin->etime.tv_usec;
+                  RaSendArgusRecord (man);
+                  ArgusDeleteRecordStruct(ArgusParser, man);
+               }
+
+#ifdef ARGUSDEBUG
+               ArgusDebug (2, "ArgusClientTimeout() RaBinProcess: Bflag %f rtime %lld start %d.%06d size %.06f arraylen %d count %d index %d\n",
+                  ArgusParser->Bflag, rtime, bin->stime.tv_sec, bin->stime.tv_usec, bin->size/1000000.0, rbps->arraylen, tcnt, rbps->index);
+#endif
                RaDeleteBin(ArgusParser, bin);
                rbps->array[rbps->index] = NULL;
 
             } else {
                if (RaBinProcess->nadp.zero) {
                   long long tval = RaBinProcess->start + (RaBinProcess->size * RaBinProcess->index);
+                  struct ArgusTimeObject *btime = NULL;
 
                   ns = ArgusGenerateRecordStruct(NULL, NULL, NULL);
 
-                  ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.start.tv_sec  = tval / 1000000;
-                  ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.start.tv_usec = tval % 1000000;
+                  btime = (struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX];
+                  btime->src.start.tv_sec  = tval / 1000000;
+                  btime->src.start.tv_usec = tval % 1000000;
 
                   tval += RaBinProcess->size;
-                  ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.end.tv_sec    = tval / 1000000;;
-                  ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.end.tv_usec   = tval % 1000000;
+                  btime->src.end.tv_sec    = tval / 1000000;;
+                  btime->src.end.tv_usec   = tval % 1000000;
 
                   RaSendArgusRecord (ns);
+
+#ifdef ARGUSDEBUG
+               ArgusDebug (2, "ArgusClientTimeout() RaBinProcess: Bflag %f rtime %lld start %d.%06d size %.06f arraylen %d count %d index %d\n",
+                  ArgusParser->Bflag, rtime, btime->src.start.tv_sec, btime->src.start.tv_usec, rbps->size/1000000.0, rbps->arraylen, 0, rbps->index);
+#endif
+
 /*
                   ArgusDeleteRecordStruct(ArgusParser, ns);
 */
@@ -768,11 +781,6 @@ ArgusClientTimeout ()
             }
          }
 
-#ifdef ARGUSDEBUG
-         ArgusDebug (2, "ArgusClientTimeout() RaBinProcess: Bflag %f rtime %lld start %lld end %lld size %lld arraylen %d count %d index %d\n",
-            ArgusParser->Bflag, rtime, RaBinProcess->start, RaBinProcess->end,
-            RaBinProcess->size, RaBinProcess->arraylen, RaBinProcess->count, RaBinProcess->index);
-#endif
          rtime += rbps->size;
          rbps->rtime.tv_sec  = rtime / 1000000;
          rbps->rtime.tv_usec = rtime % 1000000;
@@ -814,7 +822,10 @@ usage ()
    fprintf (stdout, "             proto            include the destination proto\n");
    fprintf (stdout, "             sport            include the source port\n");
    fprintf (stdout, "             dport            include the destination port\n");
-   fprintf (stdout, "             srcid            include the source identifier\n\n");
+   fprintf (stdout, "             srcid            include the source identifier\n");
+
+   fprintf (stdout, "         -P <sortfield>       specify the fields to sort records on.\n\n");
+
    fflush (stdout);
    exit(1);
 }
@@ -908,73 +919,76 @@ RaProcessRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *ns)
             struct ArgusAggregatorStruct *agg = parser->ArgusAggregator;
 
             if (agg && agg->ArgusMatrixMode) {
-               if (agg->mask & ((0x01LL << ARGUS_MASK_SADDR) | (0x01LL << ARGUS_MASK_DADDR))) {
-                  struct ArgusFlow *flow = (struct ArgusFlow *)ns->dsrs[ARGUS_FLOW_INDEX];
+               struct ArgusFlow *flow = (struct ArgusFlow *)ns->dsrs[ARGUS_FLOW_INDEX];
 
-                  if (flow != NULL) {
+               if (flow != NULL) {
+                  if (agg->mask & ((0x01LL << ARGUS_MASK_SADDR) | (0x01LL << ARGUS_MASK_DADDR))) {
                      switch (flow->hdr.subtype & 0x3F) {
                         case ARGUS_FLOW_LAYER_3_MATRIX:
-                        case ARGUS_FLOW_CLASSIC5TUPLE: {
-                           switch (flow->hdr.argus_dsrvl8.qual & 0x1F) {
-                              case ARGUS_TYPE_IPV4: {
-                                 if (flow->ip_flow.ip_src > flow->ip_flow.ip_dst)
-                                    ArgusReverseRecord(ns);
-                              }
-                              break;
-
-                              case ARGUS_TYPE_IPV6: {
-                                 int i;
-                                 for (i = 0; i < 4; i++) {
-                                    if (flow->ipv6_flow.ip_src[i] < flow->ipv6_flow.ip_dst[i])
-                                       break;
-
-                                    if (flow->ipv6_flow.ip_src[i] > flow->ipv6_flow.ip_dst[i]) {
+                           case ARGUS_FLOW_CLASSIC5TUPLE: {
+                              switch (flow->hdr.argus_dsrvl8.qual & 0x1F) {
+                                 case ARGUS_TYPE_IPV4: {
+                                    if (flow->ip_flow.ip_src > flow->ip_flow.ip_dst)
                                        ArgusReverseRecord(ns);
-                                       break;
+                                 }
+                                 break;
+
+                                 case ARGUS_TYPE_IPV6: {
+                                    int i;
+                                    for (i = 0; i < 4; i++) {
+                                       if (flow->ipv6_flow.ip_src[i] < flow->ipv6_flow.ip_dst[i])
+                                          break;
+
+                                       if (flow->ipv6_flow.ip_src[i] > flow->ipv6_flow.ip_dst[i]) {
+                                          ArgusReverseRecord(ns);
+                                          break;
+                                       }
                                     }
                                  }
+                                 break;
                               }
                               break;
                            }
-                           break;
+
+                           default:
+                              break;
                         }
 
-                        default:
-                           break;
-                     }
-                  }
+                  } else
+                  if (agg->mask & ((0x01LL << ARGUS_MASK_SMAC) | (0x01LL << ARGUS_MASK_DMAC))) {
 
-               } else
-               if (agg->mask & ((0x01LL << ARGUS_MASK_SMAC) | (0x01LL << ARGUS_MASK_DMAC))) {
+                     struct ArgusMacStruct *m1 = NULL;
+                     if ((m1 = (struct ArgusMacStruct *) ns->dsrs[ARGUS_MAC_INDEX]) != NULL) {
+                        switch (m1->hdr.subtype) {
+                           case ARGUS_TYPE_ETHER: {
+                              struct ether_header *e1 = &m1->mac.mac_union.ether.ehdr;
+                              int i;
 
-                  struct ArgusMacStruct *m1 = NULL;
-                  if ((m1 = (struct ArgusMacStruct *) ns->dsrs[ARGUS_MAC_INDEX]) != NULL) {
-                     switch (m1->hdr.subtype) {
-                        case ARGUS_TYPE_ETHER: {
-                           struct ether_header *e1 = &m1->mac.mac_union.ether.ehdr;
-                           int i;
-
-                           for (i = 0; i < 6; i++) {
-#if defined(HAVE_SOLARIS)
-                              if (e1->ether_shost.ether_addr_octet[i] < e1->ether_dhost.ether_addr_octet[i])
-                                 break;
-                              if (e1->ether_shost.ether_addr_octet[i] > e1->ether_dhost.ether_addr_octet[i]) {
-                                 ArgusReverseRecord(ns);
-                                 break;
-                              }
+                              for (i = 0; i < 6; i++) {
+#if defined(ARGUS_SOLARIS)
+                                 if (e1->ether_shost.ether_addr_octet[i] < e1->ether_dhost.ether_addr_octet[i])
+                                    break;
+                                 if (e1->ether_shost.ether_addr_octet[i] > e1->ether_dhost.ether_addr_octet[i]) {
+                                    ArgusReverseRecord(ns);
+                                    break;
+                                 }
 #else
-                              if (e1->ether_shost[i] < e1->ether_dhost[i])
-                                 break;
-                              if (e1->ether_shost[i] > e1->ether_dhost[i]) {
-                                 ArgusReverseRecord(ns);
-                                 break;
-                              }
+                                 if (e1->ether_shost[i] < e1->ether_dhost[i])
+                                    break;
+                                 if (e1->ether_shost[i] > e1->ether_dhost[i]) {
+                                    ArgusReverseRecord(ns);
+                                    break;
+                                 }
 #endif
+                              }
+                              break;
                            }
-                           break;
                         }
                      }
                   }
+
+                  flow->hdr.subtype &= ~ARGUS_REVERSE;
+                  flow->hdr.argus_dsrvl8.qual &= ~ARGUS_DIRECTION;
                }
             }
             RaProcessThisRecord(parser, ns);
@@ -984,16 +998,15 @@ RaProcessRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *ns)
 }
 
 void
-RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *ns)
+RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *argus)
 {
    extern struct RaBinProcessStruct *RaBinProcess;
-   struct ArgusRecordStruct *tns = NULL;
-   int retn = 0, fretn = -1, lretn = -1;
    extern int ArgusTimeRangeStrategy;
    struct ArgusAggregatorStruct *agg = parser->ArgusAggregator;
    int found = 0, offset, tstrat;
 
    while (agg && !found) {
+      int retn = 0, fretn = -1, lretn = -1;
       if (ArgusParser->tflag) {
          tstrat = ArgusTimeRangeStrategy;
          ArgusTimeRangeStrategy = 1;
@@ -1001,12 +1014,12 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
 
       if (agg->filterstr) {
          struct nff_insn *fcode = agg->filter.bf_insns;
-         fretn = ArgusFilterRecord (fcode, ns);
+         fretn = ArgusFilterRecord (fcode, argus);
       }
 
-      if (agg->labelstr) {
+      if (agg->grepstr) {
          struct ArgusLabelStruct *label;
-         if (((label = (void *)ns->dsrs[ARGUS_LABEL_INDEX]) != NULL)) {
+         if (((label = (void *)argus->dsrs[ARGUS_LABEL_INDEX]) != NULL)) {
             if (regexec(&agg->lpreg, label->l_un.label, 0, NULL, 0))
                lretn = 0;
             else
@@ -1018,6 +1031,13 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
       retn = (lretn < 0) ? ((fretn < 0) ? 1 : fretn) : ((fretn < 0) ? lretn : (lretn && fretn));
 
       if (retn != 0) {
+         struct ArgusRecordStruct *tns = NULL, *ns = NULL;
+
+         ns = ArgusCopyRecordStruct(argus);
+
+         if (agg->labelstr)
+            ArgusAddToRecordLabel(parser, ns, agg->labelstr);
+
          ArgusAlignInit(parser, ns, &RaBinProcess->nadp);
 
          offset = (ArgusParser->Bflag * 1000000)/RaBinProcess->nadp.size;
@@ -1034,6 +1054,8 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
             } else
                ArgusDeleteRecordStruct(parser, tns);
          }
+
+         ArgusDeleteRecordStruct(parser, ns);
          found++;
       }
       agg = agg->nxt;
@@ -1043,7 +1065,7 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
       ArgusTimeRangeStrategy = tstrat;
 
 #ifdef ARGUSDEBUG
-   ArgusDebug (6, "RaProcessThisRecord (0x%x) done\n", ns); 
+   ArgusDebug (6, "RaProcessThisRecord (0x%x) done\n", argus); 
 #endif
 }
 
@@ -1093,45 +1115,54 @@ RaSendArgusRecord(struct ArgusRecordStruct *argus)
 
    } else {
       if (!ArgusParser->qflag) {
-         if (ArgusParser->Lflag) {
-            if (ArgusParser->RaLabel == NULL)
-               ArgusParser->RaLabel = ArgusGenerateLabel(ArgusParser, argus);
+         switch (argus->hdr.type & 0xF0) {
+            case ARGUS_MAR:
+               if (!(ArgusParser->ArgusPrintMan))
+                break;
 
-            if (!(ArgusParser->RaLabelCounter++ % ArgusParser->Lflag)) {
-               if (ArgusParser->Gflag) {
-                  printf ("Columns=%s\n", ArgusParser->RaLabel);
-               } else
-                  printf ("%s", ArgusParser->RaLabel);
-            }
+            default: {
+               if (ArgusParser->Lflag) {
+                  if (ArgusParser->RaLabel == NULL)
+                     ArgusParser->RaLabel = ArgusGenerateLabel(ArgusParser, argus);
 
-            if (ArgusParser->Lflag < 0)
-               ArgusParser->Lflag = 0;
-
-            if (ArgusParser->Gflag) {
-               switch (ArgusParser->RaPrintMode) {
-                  case RA_PRINTSRCID:
-                     printf ("Probes=\n");
-                     break;
-
-                  case RA_PRINTPROTO: {
-                     printf ("Protos=\n");
-                     break;
+                  if (!(ArgusParser->RaLabelCounter++ % ArgusParser->Lflag)) {
+                     if (ArgusParser->Gflag) {
+                        printf ("Columns=%s\n", ArgusParser->RaLabel);
+                     } else
+                        printf ("%s", ArgusParser->RaLabel);
                   }
 
-                  default: {
-                     printf ("Objects=\n");
-                     break;
+                  if (ArgusParser->Lflag < 0)
+                     ArgusParser->Lflag = 0;
+
+                  if (ArgusParser->Gflag) {
+                     switch (ArgusParser->RaPrintMode) {
+                        case RA_PRINTSRCID:
+                           printf ("Probes=\n");
+                           break;
+
+                        case RA_PRINTPROTO: {
+                           printf ("Protos=\n");
+                           break;
+                        }
+
+                        default: {
+                           printf ("Objects=\n");
+                           break;
+                        }
+                     }
                   }
+                  printf ("\n");
                }
-            }
-            printf ("\n");
-         }
 
-         *(int *)&buf = 0;
-         ArgusPrintRecord(ArgusParser, buf, argus, MAXSTRLEN);
-         if (fprintf (stdout, "%s\n", buf) < 0)
-            RaParseComplete (SIGQUIT);
-         fflush(stdout);
+               *(int *)&buf = 0;
+               ArgusPrintRecord(ArgusParser, buf, argus, MAXSTRLEN);
+               if (fprintf (stdout, "%s\n", buf) < 0)
+                  RaParseComplete (SIGQUIT);
+               fflush(stdout);
+               break;
+            }
+         }
       }
    }
 
@@ -1251,3 +1282,136 @@ RaProcessSplitOptions(struct ArgusParserStruct *parser, char *str, int len, stru
    return (retn);
 }
 
+
+int
+RaDeleteBinProcess(struct ArgusParserStruct *parser, struct RaBinProcessStruct *rbps)
+{
+   struct RaBinStruct *bin = NULL;
+   struct ArgusRecordStruct *ns = NULL;
+   int max = ((parser->tflag && parser->RaExplicitDate) ? rbps->nadp.count : rbps->max) + 1;
+   int startsecs = 0, endsecs = 0, i;
+   int retn = 0;
+
+   char stimebuf[128], dtimebuf[128], etimebuf[128];
+   int bins;
+
+   if (rbps->array != NULL) {
+      if (!(parser->tflag)) {
+         for (i = 0; i < max; i++) {
+            if ((bin = rbps->array[i]) != NULL) {
+               if (startsecs == 0)
+                  startsecs = bin->stime.tv_sec;
+               endsecs = bin->etime.tv_sec;
+            }
+         }
+
+         rbps->startpt.tv_sec = startsecs;
+         rbps->scalesecs      = (endsecs - startsecs) + (rbps->size / 1000000);
+
+         if ((parser->RaEndTime.tv_sec >  rbps->endpt.tv_sec) ||
+                  ((parser->RaEndTime.tv_sec == rbps->endpt.tv_sec) &&
+                   (parser->RaEndTime.tv_usec > rbps->endpt.tv_usec)))
+            parser->RaEndTime = rbps->endpt;
+
+      } else {
+         rbps->startpt.tv_sec  = parser->startime_t.tv_sec;
+         rbps->startpt.tv_usec = parser->startime_t.tv_usec;
+         rbps->endpt.tv_sec    = parser->lasttime_t.tv_sec;
+         rbps->endpt.tv_usec   = parser->lasttime_t.tv_usec;
+         rbps->scalesecs       = parser->lasttime_t.tv_sec - parser->startime_t.tv_sec;
+         rbps->start           = parser->startime_t.tv_sec * 1000000LL;
+      }
+
+      if ((parser->ArgusWfileList == NULL) && (parser->Gflag)) {
+         if (parser->Hstr != NULL) {
+
+         } else {
+
+            ArgusPrintTime(parser, stimebuf, &rbps->startpt);
+            ArgusPrintTime(parser, dtimebuf, &rbps->endpt);
+            ArgusPrintTime(parser, etimebuf, &parser->RaEndTime);
+
+            stimebuf[strlen(stimebuf) - 1] = '\0';
+            dtimebuf[strlen(dtimebuf) - 1] = '\0';
+            etimebuf[strlen(etimebuf) - 1] = '\0';
+
+            printf ("StartTime=%s\n", stimebuf);
+            printf ("StopTime=%s\n",  dtimebuf);
+            printf ("LastTime=%s\n",  etimebuf);
+            printf ("Seconds=%d\n", rbps->scalesecs);
+            printf ("BinSize=%1.*f\n", parser->pflag, (rbps->size * 1.0)/1000000);
+            bins = ((rbps->scalesecs + (rbps->size/1000000 - 1))/(rbps->size / 1000000));
+            printf ("Bins=%d\n", bins);
+         }
+      }
+   }
+
+   for (i = rbps->index; i < max; i++) {
+      if ((rbps->array != NULL) && ((bin = rbps->array[i]) != NULL)) {
+         struct ArgusAggregatorStruct *agg = bin->agg;
+
+         if (ArgusParser->ArgusGenerateManRecords) {
+            struct ArgusRecordStruct *man = ArgusGenerateStatusMarRecord (NULL, ARGUS_START);
+            struct ArgusRecord *rec = (struct ArgusRecord *)man->dsrs[0];
+            rec->argus_mar.startime.tv_sec  = bin->stime.tv_sec;
+            rec->argus_mar.startime.tv_usec = bin->stime.tv_usec;
+            rec->argus_mar.now.tv_sec       = bin->stime.tv_sec;
+            rec->argus_mar.now.tv_usec      = bin->stime.tv_usec;
+
+            RaSendArgusRecord (man);
+            ArgusDeleteRecordStruct(ArgusParser, man);
+         }
+
+         while (agg) {
+            int rank = 1;
+            ArgusSortQueue(ArgusSorter, agg->queue);
+            while ((ns = (struct ArgusRecordStruct *) ArgusPopQueue(agg->queue, ARGUS_NOLOCK)) != NULL) {
+               ns->rank = rank++;
+               if ((parser->eNoflag == 0 ) || ((parser->eNoflag >= ns->rank) && (parser->sNoflag <= ns->rank)))
+                  RaSendArgusRecord (ns);
+               ArgusDeleteRecordStruct(parser, ns);
+            }
+            agg = agg->nxt;
+         }
+
+         if (ArgusParser->ArgusGenerateManRecords) {
+            struct ArgusRecordStruct *man = ArgusGenerateStatusMarRecord (NULL, ARGUS_STOP);
+            struct ArgusRecord *rec = (struct ArgusRecord *)man->dsrs[0];
+            rec->argus_mar.startime.tv_sec  = bin->etime.tv_sec;
+            rec->argus_mar.startime.tv_usec = bin->etime.tv_usec;
+            rec->argus_mar.now.tv_sec       = bin->etime.tv_sec;
+            rec->argus_mar.now.tv_usec      = bin->etime.tv_usec;
+
+            RaSendArgusRecord (man);
+            ArgusDeleteRecordStruct(ArgusParser, man);
+         }
+
+      } else {
+         if (rbps->nadp.zero && ((i >= rbps->index) && 
+                 ((((i - rbps->index) * 1.0) * rbps->size) < (rbps->scalesecs * 1000000LL)))) {
+            long long tval = rbps->start + (rbps->size * (i - rbps->index));
+
+            ns = ArgusGenerateRecordStruct(NULL, NULL, NULL);
+
+            ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.start.tv_sec  = tval / 1000000;
+            ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.start.tv_usec = tval % 1000000;
+
+            tval += rbps->size;
+            ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.end.tv_sec    = tval / 1000000;;
+            ((struct ArgusTimeObject *)ns->dsrs[ARGUS_TIME_INDEX])->src.end.tv_usec   = tval % 1000000;
+
+            RaSendArgusRecord (ns);
+         }
+      }
+   }
+
+   for (i = rbps->index; i < max; i++) {
+      if ((rbps->array != NULL) && ((bin = rbps->array[i]) != NULL)) {
+         RaDeleteBin(parser, bin);
+      }
+   }
+
+   if (rbps->array != NULL) ArgusFree(rbps->array);
+   ArgusFree(rbps);
+   return (retn);
+}
