@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/clients/racount.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/clients/racount.c#42 $
+ * $DateTime: 2014/06/09 14:18:59 $
+ * $Change: 2847 $
  */
 
 /*
@@ -218,15 +218,36 @@ ArgusClientInit (struct ArgusParserStruct *parser)
          while (mode) {
             if (!(strncasecmp (mode->mode, "addr", 4)))
                RaAddrMode++;
+            else
             if (!(strncasecmp (mode->mode, "proto", 4)))
                RaProtoMode++;
+            else
             if (!(strncasecmp (mode->mode, "oui", 3)))
                parser->ArgusPrintEthernetVendors++;
+            else
+            if (!(strncasecmp (mode->mode, "poll", 4)))
+               parser->RaPollMode++;
+            else
+            if (!(strncasecmp (mode->mode, "rmon", 4)))
+               parser->RaMonMode++;
+            else
+            if (!(strncasecmp (mode->mode, "uni", 3)))
+               parser->RaUniMode++;
+            else
+            if (!(strncasecmp (mode->mode, "oui", 3)))
+               parser->ArgusPrintEthernetVendors++;
+            else
+            if (!(strncasecmp (mode->mode, "man", 3)))
+               parser->ArgusPrintMan = 1;
+            else
+            if (!(strncasecmp (mode->mode, "noman", 5)))
+               parser->ArgusPrintMan = 0;
+
             mode = mode->nxt;
          }
       }
 
-      if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL)) == NULL)
+      if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL, ARGUS_RECORD_AGGREGATOR)) == NULL)
          ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
 
       bzero(ArgusIPProtoRecs, sizeof(ArgusIPProtoRecs));
@@ -249,16 +270,18 @@ void RaArgusInputComplete (struct ArgusInput *input) { return; }
 void
 RaParseComplete (int sig)
 {
+   int ArgusExitStatus = 0;
+
    if (sig >= 0) {
       if (!(ArgusParser->RaParseCompleting++)) {
 
          printf ("racount   records     total_pkts     src_pkts       dst_pkts       total_bytes        src_bytes          dst_bytes\n");
-#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(HAVE_SOLARIS)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(ARGUS_SOLARIS)
          printf ("    sum   %-11lld %-14lld %-14lld %-14lld %-18lld %-18lld %-18lld\n",
 #else
          printf ("    sum   %-11Ld %-14Ld %-14Ld %-14Ld %-18Ld %-18Ld %-18Ld\n",
 #endif
-                       ArgusParser->ArgusTotalRecords,
+                       ArgusTotalFlowRecs,
                        ArgusParser->ArgusTotalPkts, ArgusParser->ArgusTotalSrcPkts, ArgusParser->ArgusTotalDstPkts,
                        ArgusParser->ArgusTotalBytes, ArgusParser->ArgusTotalSrcBytes, ArgusParser->ArgusTotalDstBytes);
 
@@ -273,6 +296,7 @@ RaParseComplete (int sig)
          }
 
          fflush (stdout);
+         ArgusShutDown(sig);
 
 #ifdef ARGUSDEBUG
          ArgusDebug (2, "RaParseComplete(caught signal %d)\n", sig);
@@ -284,8 +308,6 @@ RaParseComplete (int sig)
             case SIGQUIT: {
                struct ArgusWfileStruct *wfile = NULL;
 
-               ArgusShutDown(sig);
-
                if (ArgusParser->ArgusWfileList != NULL) {
                   struct ArgusListObjectStruct *lobj = NULL;
                   int i, count = ArgusParser->ArgusWfileList->count;
@@ -306,10 +328,42 @@ RaParseComplete (int sig)
                      }
                   }
                }
-               exit(0);
                break;
             }
          }
+
+         ArgusDeleteHashTable(ArgusSrcAddrTable);
+         ArgusDeleteHashTable(ArgusDstAddrTable);
+
+#if defined(ARGUS_THREADS)
+         if (ArgusParser->Sflag) {
+            struct ArgusInput *addr;
+            void *retn = NULL;
+
+            while ((addr = (void *)ArgusPopQueue(ArgusParser->ArgusRemoteHosts, ARGUS_LOCK)) != NULL) {
+               if (addr->tid != (pthread_t) 0) {
+                  pthread_join(addr->tid, &retn);
+               }
+               ArgusFree(addr);
+            }
+
+            while ((addr = (void *)ArgusPopQueue(ArgusParser->ArgusActiveHosts, ARGUS_LOCK)) != NULL) {
+               if (addr->tid != (pthread_t) 0) {
+                  pthread_join(addr->tid, &retn);
+               }
+               ArgusFree(addr);
+            }
+         }
+
+         if (ArgusParser->timer != (pthread_t) 0)
+            pthread_join(ArgusParser->timer, NULL);
+
+         if (ArgusParser->dns != (pthread_t) 0)
+            pthread_join(ArgusParser->dns, NULL);
+#endif
+         ArgusExitStatus = ArgusParser->ArgusExitStatus;
+         ArgusCloseParser(ArgusParser);
+         exit (ArgusExitStatus);
       }
    }
 }
@@ -445,7 +499,6 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
             case ARGUS_FLOW_CLASSIC5TUPLE: {
 
                switch (flow->hdr.argus_dsrvl8.qual & 0x1F) {
-                  case ARGUS_TYPE_IPV6:
                   case ARGUS_TYPE_IPV4: {
                      struct ArgusRecordStruct *tns;
 
@@ -455,6 +508,15 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
                         ArgusMergeRecords (parser->ArgusAggregator, tns, argus);
                      break;
                   }
+                  case ARGUS_TYPE_IPV6: {
+                     struct ArgusRecordStruct *tns;
+
+                     if ((tns = ArgusIPProtoRecs[flow->ipv6_flow.ip_p]) == NULL)
+                        ArgusIPProtoRecs[flow->ipv6_flow.ip_p] = ArgusCopyRecordStruct(argus); 
+                     else
+                        ArgusMergeRecords (parser->ArgusAggregator, tns, argus);
+                     break;
+                  }
 
                   case ARGUS_TYPE_ARP: {
                      if ((tns = ArgusArpRecords) == NULL)
@@ -481,6 +543,14 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
                         ArgusMergeRecords (parser->ArgusAggregator, tns, argus);
                      break;
                   }
+
+                  default: {
+                     if ((tns = ArgusUnknownRecords) == NULL)
+                        ArgusUnknownRecords = ArgusCopyRecordStruct(argus);
+                     else
+                        ArgusMergeRecords (parser->ArgusAggregator, tns, argus);
+                     break;
+                  }
                }
             }
 
@@ -609,7 +679,7 @@ RaTallyIPv4AddressType(struct ArgusParserStruct *parser, unsigned int addr, int
       }
       if ((addr & 0xFF000000) == 0xEF000000) {
          ArgusIPv4AddrMulticastAdmin[type]++;
-         if (((addr & 0x00FF0000) >= 0x00000000) && ((addr & 0x00FF0000) <  0x00C00000)) {
+         if (((addr & 0x00FF0000) >  0x00000000) && ((addr & 0x00FF0000) <  0x00C00000)) {
             ArgusIPv4AddrMulticastReserved[type]++;
          }
          if (((addr & 0x00FF0000) >= 0x00C00000) && ((addr & 0x00FF0000) <  0x00FC0000)) {
@@ -624,7 +694,7 @@ RaTallyIPv4AddressType(struct ArgusParserStruct *parser, unsigned int addr, int
       if (((addr & 0xFF000000) == 0x00000000)) {
          ArgusIPv4AddrUnicastThisNet[type]++;
       } else 
-      if (((addr & 0xFF000000) >= 0x00000000) && ((addr & 0xFF000000) <  0x03000000)) {
+      if (((addr & 0xFF000000) > 0x00000000) && ((addr & 0xFF000000) <  0x03000000)) {
          ArgusIPv4AddrUnicastReserved[type]++;
       } else 
       if ((addr & 0xFF000000) == 0x05000000) {
@@ -732,39 +802,39 @@ RaPrintAddressTally(void)
    printf ("Address Summary\n");
    if (ArgusIPv4AddrUnicast[0] || ArgusIPv4AddrUnicast[1])
       printf ("  IPv4 Unicast              src %-10d  dst %-10d\n", ArgusIPv4AddrUnicast[0], ArgusIPv4AddrUnicast[1]);
-   if (ArgusIPv4AddrUnicastThisNet[0] || ArgusIPv4AddrUnicastThisNet[0])
+   if (ArgusIPv4AddrUnicastThisNet[0] || ArgusIPv4AddrUnicastThisNet[1])
       printf ("  IPv4 Unicast This Network src %-10d  dst %-10d\n", ArgusIPv4AddrUnicastThisNet[0], ArgusIPv4AddrUnicastThisNet[1]);
-   if (ArgusIPv4AddrUnicastPrivate[0] || ArgusIPv4AddrUnicastPrivate[0])
+   if (ArgusIPv4AddrUnicastPrivate[0] || ArgusIPv4AddrUnicastPrivate[1])
       printf ("  IPv4 Unicast Private      src %-10d  dst %-10d\n", ArgusIPv4AddrUnicastPrivate[0], ArgusIPv4AddrUnicastPrivate[1]);
-   if (ArgusIPv4AddrUnicastLoopBack[0] || ArgusIPv4AddrUnicastLoopBack[0])
+   if (ArgusIPv4AddrUnicastLoopBack[0] || ArgusIPv4AddrUnicastLoopBack[1])
       printf ("  IPv4 Unicast LoopBack     src %-10d  dst %-10d\n", ArgusIPv4AddrUnicastLoopBack[0], ArgusIPv4AddrUnicastLoopBack[1]);
-   if (ArgusIPv4AddrUnicastLinkLocal[0] || ArgusIPv4AddrUnicastLinkLocal[0])
+   if (ArgusIPv4AddrUnicastLinkLocal[0] || ArgusIPv4AddrUnicastLinkLocal[1])
       printf ("  IPv4 Unicast Link Local   src %-10d  dst %-10d\n", ArgusIPv4AddrUnicastLinkLocal[0], ArgusIPv4AddrUnicastLinkLocal[1]);
-   if (ArgusIPv4AddrUnicastLinkLocal[0] || ArgusIPv4AddrUnicastLinkLocal[0])
+   if (ArgusIPv4AddrUnicastLinkLocal[0] || ArgusIPv4AddrUnicastLinkLocal[1])
       printf ("  IPv4 Unicast Test Net     src %-10d  dst %-10d\n", ArgusIPv4AddrUnicastTestNet[0], ArgusIPv4AddrUnicastTestNet[1]);
-   if (ArgusIPv4AddrUnicastReserved[0] || ArgusIPv4AddrUnicastReserved[0])
+   if (ArgusIPv4AddrUnicastReserved[0] || ArgusIPv4AddrUnicastReserved[1])
       printf ("  IPv4 Unicast Reserved     src %-10d  dst %-10d\n", ArgusIPv4AddrUnicastReserved[0], ArgusIPv4AddrUnicastReserved[1]);
-   if (ArgusIPv4AddrMulticastLocal[0] || ArgusIPv4AddrMulticastLocal[0])
+   if (ArgusIPv4AddrMulticastLocal[0] || ArgusIPv4AddrMulticastLocal[1])
       printf ("  IPv4 Multicast Local      src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastLocal[0], ArgusIPv4AddrMulticastLocal[1]);
-   if (ArgusIPv4AddrMulticastInternet[0] || ArgusIPv4AddrMulticastInternet[0])
+   if (ArgusIPv4AddrMulticastInternet[0] || ArgusIPv4AddrMulticastInternet[1])
       printf ("  IPv4 Multicast Internet   src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastInternet[0], ArgusIPv4AddrMulticastInternet[1]);
-   if (ArgusIPv4AddrMulticastAdHoc[0] || ArgusIPv4AddrMulticastAdHoc[0])
+   if (ArgusIPv4AddrMulticastAdHoc[0] || ArgusIPv4AddrMulticastAdHoc[1])
       printf ("  IPv4 Multicast AdHoc      src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastAdHoc[0], ArgusIPv4AddrMulticastAdHoc[1]);
-   if (ArgusIPv4AddrMulticastReserved[0] || ArgusIPv4AddrMulticastReserved[0])
+   if (ArgusIPv4AddrMulticastReserved[0] || ArgusIPv4AddrMulticastReserved[1])
       printf ("  IPv4 Multicast Reserved   src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastReserved[0], ArgusIPv4AddrMulticastReserved[1]);
-   if (ArgusIPv4AddrMulticastSdpSap[0] || ArgusIPv4AddrMulticastSdpSap[0])
+   if (ArgusIPv4AddrMulticastSdpSap[0] || ArgusIPv4AddrMulticastSdpSap[1])
       printf ("  IPv4 Multicast SdpSap     src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastSdpSap[0], ArgusIPv4AddrMulticastSdpSap[1]);
-   if (ArgusIPv4AddrMulticastNasdaq[0] || ArgusIPv4AddrMulticastNasdaq[0])
+   if (ArgusIPv4AddrMulticastNasdaq[0] || ArgusIPv4AddrMulticastNasdaq[1])
       printf ("  IPv4 Multicast Nasdaq     src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastNasdaq[0], ArgusIPv4AddrMulticastNasdaq[1]);
-   if (ArgusIPv4AddrMulticastDisTrans[0] || ArgusIPv4AddrMulticastDisTrans[0])
+   if (ArgusIPv4AddrMulticastDisTrans[0] || ArgusIPv4AddrMulticastDisTrans[1])
       printf ("  IPv4 Multicast DisTrans   src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastDisTrans[0], ArgusIPv4AddrMulticastDisTrans[1]);
-   if (ArgusIPv4AddrMulticastSrcSpec[0] || ArgusIPv4AddrMulticastSrcSpec[0])
+   if (ArgusIPv4AddrMulticastSrcSpec[0] || ArgusIPv4AddrMulticastSrcSpec[1])
       printf ("  IPv4 Multicast Src Spec   src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastSrcSpec[0], ArgusIPv4AddrMulticastSrcSpec[1]);
-   if (ArgusIPv4AddrMulticastGlop[0] || ArgusIPv4AddrMulticastGlop[0])
+   if (ArgusIPv4AddrMulticastGlop[0] || ArgusIPv4AddrMulticastGlop[1])
       printf ("  IPv4 Multicast GLOP Blk   src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastGlop[0], ArgusIPv4AddrMulticastGlop[1]);
-   if (ArgusIPv4AddrMulticastOrgLocal[0] || ArgusIPv4AddrMulticastOrgLocal[0])
+   if (ArgusIPv4AddrMulticastOrgLocal[0] || ArgusIPv4AddrMulticastOrgLocal[1])
       printf ("  IPv4 Multicast OrgLocal   src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastOrgLocal[0], ArgusIPv4AddrMulticastOrgLocal[1]);
-   if (ArgusIPv4AddrMulticastSiteLocal[0] || ArgusIPv4AddrMulticastSiteLocal[0])
+   if (ArgusIPv4AddrMulticastSiteLocal[0] || ArgusIPv4AddrMulticastSiteLocal[1])
       printf ("  IPv4 Multicast SiteLocal  src %-10d  dst %-10d\n", ArgusIPv4AddrMulticastSiteLocal[0], ArgusIPv4AddrMulticastSiteLocal[1]);
    if (ArgusIPv6AddrUnspecified[0] || ArgusIPv6AddrUnspecified[1])
       printf ("  IPv6 Unspecified          src %-10d  dst %-10d\n", ArgusIPv6AddrUnspecified[0], ArgusIPv6AddrUnspecified[1]);
@@ -881,7 +951,7 @@ RaPrintProtoTally(void)
          value  = ArgusFetchDstByteCount (ns);
          dbytes = value;
 
-#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(HAVE_SOLARIS)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(ARGUS_SOLARIS)
          printf ("%8.8s  %-11lld %-14lld %-14lld %-14lld %-18lld %-18lld %-18lld\n",
 #else
          printf ("%8.8s  %-11Ld %-14Ld %-14Ld %-14Ld %-18Ld %-18Ld %-18Ld\n",
@@ -912,7 +982,7 @@ RaPrintProtoTally(void)
          value  = ArgusFetchDstByteCount (ns);
          dbytes = value;
 
-#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(HAVE_SOLARIS)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(ARGUS_SOLARIS)
          printf ("%8.8s  %-11lld %-14lld %-14lld %-14lld %-18lld %-18lld %-18lld\n",
 #else
          printf ("%8.8s  %-11Ld %-14Ld %-14Ld %-14Ld %-18Ld %-18Ld %-18Ld\n",
@@ -942,7 +1012,7 @@ RaPrintProtoTally(void)
       value  = ArgusFetchDstByteCount (ns);
       dbytes = value;
  
-#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(HAVE_SOLARIS)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(ARGUS_SOLARIS)
       printf ("%8.8s  %-11lld %-14lld %-14lld %-14lld %-18lld %-18lld %-18lld\n",
 #else
       printf ("%8.8s  %-11Ld %-14Ld %-14Ld %-14Ld %-18Ld %-18Ld %-18Ld\n",
@@ -971,7 +1041,7 @@ RaPrintProtoTally(void)
       value  = ArgusFetchDstByteCount (ns);
       dbytes = value;
  
-#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(HAVE_SOLARIS)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(ARGUS_SOLARIS)
       printf ("%8.8s  %-11lld %-14lld %-14lld %-14lld %-18lld %-18lld %-18lld\n",
 #else
       printf ("%8.8s  %-11Ld %-14Ld %-14Ld %-14Ld %-18Ld %-18Ld %-18Ld\n",
@@ -1000,13 +1070,11 @@ RaPrintProtoTally(void)
       value  = ArgusFetchDstByteCount (ns);
       dbytes = value;
  
-#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(HAVE_SOLARIS)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(ARGUS_SOLARIS)
       printf ("%8.8s  %-11lld %-14lld %-14lld %-14lld %-18lld %-18lld %-18lld\n",
 #else
       printf ("%8.8s  %-11Ld %-14Ld %-14Ld %-14Ld %-18Ld %-18Ld %-18Ld\n",
 #endif
       buf, recs, pkts, spkts, dpkts, bytes, sbytes, dbytes);
    }
-
-
 }
