@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,9 +18,9 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * 
- * $Id: //depot/argus-3.0.6/clients/clients/ranonymize.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/clients/ranonymize.c#35 $
+ * $DateTime: 2014/06/09 14:18:59 $
+ * $Change: 2847 $
  */
 
 /*
@@ -202,8 +202,26 @@ ArgusClientInit (struct ArgusParserStruct *parser)
 
       if ((mode = parser->ArgusModeList) != NULL) {
          while (mode) {
-            struct ArgusModeStruct *tmode;;
+            struct ArgusModeStruct *tmode;
 
+            if (!(strncasecmp (mode->mode, "poll", 4)))
+               parser->RaPollMode++;
+            else
+            if (!(strncasecmp (mode->mode, "rmon", 4)))
+               parser->RaMonMode++;
+            else
+            if (!(strncasecmp (mode->mode, "uni", 3)))
+               parser->RaUniMode++;
+            else
+            if (!(strncasecmp (mode->mode, "oui", 3)))
+               parser->ArgusPrintEthernetVendors++;
+            else
+            if (!(strncasecmp (mode->mode, "man", 3)))
+               parser->ArgusPrintMan = 1;
+            else
+            if (!(strncasecmp (mode->mode, "noman", 5)))
+               parser->ArgusPrintMan = 0;
+            else
             if (!(strncasecmp ("replace", mode->mode, strlen("replace")))) {
                ArgusProcessFileIndependantly = 1;
                parser->ArgusReplaceMode++;
@@ -237,6 +255,10 @@ ArgusClientInit (struct ArgusParserStruct *parser)
          ArgusDSRFields[ARGUS_ASN_INDEX] = 1;
       }
 
+
+      if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL, ARGUS_RECORD_AGGREGATOR)) == NULL)
+         ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
+
       if ((RaMapHashTable.array = (struct RaMapHashTableHeader **)
                   ArgusCalloc (RaHashSize, sizeof (struct RaMapHashTableHeader))) == NULL)
          ArgusLog (LOG_ERR, "RaMapInit: ArgusCalloc error %s\n", strerror(errno));
@@ -473,7 +495,7 @@ void
 RaArgusInputComplete (struct ArgusInput *input)
 {
    if (ArgusProcessFileIndependantly) {
-      ArgusParser->ArgusCurrentFile = input;
+      ArgusParser->ArgusCurrentInput = input;
 
       RaParseComplete (0);
 
@@ -496,7 +518,7 @@ RaArgusInputComplete (struct ArgusInput *input)
          }
       }
       ArgusParser->RaInitialized = 0;
-      ArgusParser->ArgusCurrentFile = NULL;
+      ArgusParser->ArgusCurrentInput = NULL;
       ArgusClientInit(ArgusParser);
    }
 
@@ -1868,6 +1890,8 @@ RaNonParseResourceFile (char *file)
             }
          }
 
+         fclose(fd);
+
       } else {
 #ifdef ARGUSDEBUG
          ArgusDebug (2, "%s: %s\n", file, strerror(errno));
