@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,9 +18,9 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * 
- * $Id: //depot/argus-3.0.6/clients/clients/rasort.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/clients/rasort.c#51 $
+ * $DateTime: 2014/06/09 14:18:59 $
+ * $Change: 2847 $
  */
 
 /*
@@ -79,12 +79,30 @@ ArgusClientInit (struct ArgusParserStruct *parser)
          while (mode) {
             if (!(strcmp ("replace", mode->mode))) {
                ArgusProcessFileIndependantly = 1;
-               ArgusSorter->ArgusReplaceMode++;
+               parser->ArgusReplaceMode |= ARGUS_REPLACE_MODE_TRUE;
+
+               if ((parser->ArgusWfileList != NULL) && (!(ArgusListEmpty(parser->ArgusWfileList)))) {
+                  ArgusLog (LOG_ERR, "replace mode and -w option are incompatible\n");
+               }
             } else
-            if (!(strncasecmp (mode->mode, "oui", 3))) {
+            if (!(strncasecmp (mode->mode, "poll", 4)))
+               parser->RaPollMode++;
+            else
+            if (!(strncasecmp (mode->mode, "rmon", 4)))
+               parser->RaMonMode++;
+            else
+            if (!(strncasecmp (mode->mode, "uni", 3)))
+               parser->RaUniMode++;
+            else
+            if (!(strncasecmp (mode->mode, "oui", 3)))
                parser->ArgusPrintEthernetVendors++;
-            }
- 
+            else
+            if (!(strncasecmp (mode->mode, "man", 3)))
+               parser->ArgusPrintMan = 1;
+            else
+            if (!(strncasecmp (mode->mode, "noman", 5)))
+               parser->ArgusPrintMan = 0;
+
             mode = mode->nxt;
          }
       }
@@ -131,11 +149,11 @@ void
 RaArgusInputComplete (struct ArgusInput *input)
 {
    if (ArgusProcessFileIndependantly) {
-      ArgusParser->ArgusCurrentFile = input;
+      ArgusParser->ArgusCurrentInput = input;
       RaParseComplete (0);
 
       ArgusParser->RaInitialized = 0;
-      ArgusParser->ArgusCurrentFile = NULL;
+      ArgusParser->ArgusCurrentInput = NULL;
       ArgusClientInit(ArgusParser);
    }
 }
@@ -144,7 +162,7 @@ RaArgusInputComplete (struct ArgusInput *input)
 void
 RaParseComplete (int sig)
 {
-   struct ArgusInput *file = ArgusParser->ArgusCurrentFile;
+   struct ArgusInput *file = ArgusParser->ArgusCurrentInput;
    int i = 0, count = 0;
    char buf[MAXSTRLEN];
    int label;
@@ -155,7 +173,21 @@ RaParseComplete (int sig)
 
          ArgusParser->RaParseCompleting += sig;
 
-         if (ArgusSorter->ArgusReplaceMode && file) {
+         if (ArgusParser->ArgusReplaceMode && file) {
+            if (ArgusParser->ArgusReplaceMode & ARGUS_REPLACE_COMPRESSED_GZ) {
+               char *ptr;
+               if ((ptr = strstr(file->filename, ".gz")) != NULL) {
+                  ArgusParser->ArgusReplaceMode |= ARGUS_REPLACE_FILENAME_MODIFIED;
+                  *ptr = '\0';
+               }
+            }
+            if (ArgusParser->ArgusReplaceMode & ARGUS_REPLACE_COMPRESSED_BZ) {
+               char *ptr;
+               if ((ptr = strstr(file->filename, ".bz2")) != NULL) {
+                  ArgusParser->ArgusReplaceMode |= ARGUS_REPLACE_FILENAME_MODIFIED;
+                  *ptr = '\0';
+               }
+            }
 
             if (!(ArgusParser->ArgusRandomSeed))
                srandom(ArgusParser->ArgusRandomSeed);
@@ -173,6 +205,7 @@ RaParseComplete (int sig)
 
          if (count > 0) {
             ArgusSortQueue (ArgusSorter, ArgusSorter->ArgusRecordQueue);
+            count = ArgusSorter->ArgusRecordQueue->arraylen;
  
             for (i = 0; i < count; i++) {
                struct ArgusRecordStruct *ns = (struct ArgusRecordStruct *) ArgusSorter->ArgusRecordQueue->array[i];
@@ -187,7 +220,7 @@ RaParseComplete (int sig)
          }
       }
 
-      if (ArgusSorter->ArgusReplaceMode && file) {
+      if (ArgusParser->ArgusReplaceMode && file) {
          if (ArgusParser->ArgusWfileList != NULL) {
             struct ArgusWfileStruct *wfile = NULL;
 
@@ -204,6 +237,21 @@ RaParseComplete (int sig)
             if (ArgusParser->Vflag)
                ArgusLog(LOG_INFO, "file %s aggregated", file->filename);
          }
+
+         if (ArgusParser->ArgusReplaceMode & ARGUS_REPLACE_COMPRESSED_GZ) {
+            char cmdbuf[MAXSTRLEN], *cmd = cmdbuf;
+
+            sprintf(cmd, "gzip -q %s\n", file->filename);
+            if (system(cmd) < 0)
+               ArgusLog (LOG_ERR, "compressing file %s failed");
+         } else
+         if (ArgusParser->ArgusReplaceMode & ARGUS_REPLACE_COMPRESSED_BZ) {
+            char cmdbuf[MAXSTRLEN], *cmd = cmdbuf;
+
+            sprintf(cmd, "bzip2 -f -q %s\n", file->filename);
+            if (system(cmd) < 0)
+               ArgusLog (LOG_ERR, "compressing file %s failed");
+         }
       }
 
 #ifdef ARGUSDEBUG
@@ -310,10 +358,9 @@ RaProcessRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *ns)
 
    switch (ns->hdr.type & 0xF0) {
       case ARGUS_MAR:
-      case ARGUS_EVENT: {
          break;
-      }
 
+      case ARGUS_EVENT:
       case ARGUS_NETFLOW:
       case ARGUS_FAR: {
          if ((tns = ArgusCopyRecordStruct(ns)) == NULL)
@@ -381,6 +428,46 @@ RaSendArgusRecord(struct ArgusRecordStruct *ns)
          ArgusPrintRecord(ArgusParser, buf, ns, MAXSTRLEN);
          if (fprintf (stdout, "%s\n", buf) < 0)
            RaParseComplete(SIGQUIT);
+
+         if (ArgusParser->eflag == ARGUS_HEXDUMP) {
+            int i;
+            for (i = 0; i < MAX_PRINT_ALG_TYPES; i++) {
+               if (ArgusParser->RaPrintAlgorithmList[i] != NULL) {
+                  struct ArgusDataStruct *user = NULL;
+                  if (ArgusParser->RaPrintAlgorithmList[i]->print == ArgusPrintSrcUserData) {
+                     int slen = 0, len = ArgusParser->RaPrintAlgorithmList[i]->length;
+                     if (len > 0) {
+                        if ((user = (struct ArgusDataStruct *)ns->dsrs[ARGUS_SRCUSERDATA_INDEX]) != NULL) {
+                           if (user->hdr.type == ARGUS_DATA_DSR) {
+                              slen = (user->hdr.argus_dsrvl16.len - 2 ) * 4;
+                           } else
+                              slen = (user->hdr.argus_dsrvl8.len - 2 ) * 4;
+
+                           slen = (user->count < slen) ? user->count : slen;
+                           slen = (slen > len) ? len : slen;
+                           ArgusDump ((const u_char *) &user->array, slen, "      ");
+                        }
+                     }
+                  }
+                  if (ArgusParser->RaPrintAlgorithmList[i]->print == ArgusPrintDstUserData) {
+                     int slen = 0, len = ArgusParser->RaPrintAlgorithmList[i]->length;
+                     if (len > 0) {
+                        if ((user = (struct ArgusDataStruct *)ns->dsrs[ARGUS_DSTUSERDATA_INDEX]) != NULL) {
+                           if (user->hdr.type == ARGUS_DATA_DSR) {
+                              slen = (user->hdr.argus_dsrvl16.len - 2 ) * 4;
+                           } else
+                              slen = (user->hdr.argus_dsrvl8.len - 2 ) * 4;
+
+                           slen = (user->count < slen) ? user->count : slen;
+                           slen = (slen > len) ? len : slen;
+                           ArgusDump ((const u_char *) &user->array, slen, "      ");
+                        }
+                     }
+                  }
+               } else
+                  break;
+            }
+         }
          fflush(stdout);
       }
    }
