@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,9 +18,9 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * 
- * $Id: //depot/argus-3.0.6/clients/clients/rasplit.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/clients/rasplit.c#68 $
+ * $DateTime: 2014/04/14 19:25:16 $
+ * $Change: 2800 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -182,10 +182,29 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                   }
                }
             } else
+            if (!(strncasecmp (mode->mode, "poll", 4)))
+               parser->RaPollMode++;
+            else
+            if (!(strncasecmp (mode->mode, "rmon", 4)))
+               parser->RaMonMode++;
+            else
+            if (!(strncasecmp (mode->mode, "uni", 3)))
+               parser->RaUniMode++;
+            else
+            if (!(strncasecmp (mode->mode, "oui", 3)))
+               parser->ArgusPrintEthernetVendors++;
+            else
+            if (!(strncasecmp (mode->mode, "man", 3)))
+               parser->ArgusPrintMan = 1;
+            else
+            if (!(strncasecmp (mode->mode, "noman", 5)))
+               parser->ArgusPrintMan = 0;
+            else
             if (isdigit((int) *mode->mode)) {
                ind = 0;
             } else {
-                  for (i = 0, ind = -1; i < ARGUSSPLITMODENUM; i++) {
+                  int done = 0;
+                  for (i = 0, ind = -1; (i < ARGUSSPLITMODENUM) && !done; i++) {
                      if (!(strncasecmp (mode->mode, RaSplitModes[i], 3))) {
                         ind = i;
                         switch (ind) {
@@ -194,6 +213,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                            case ARGUSSPLITCOUNT:
                               if ((mode = mode->nxt) == NULL)
                                  usage();
+                              done = 1;
                               break;
                         }
                      }
@@ -425,28 +445,41 @@ ArgusClientInit (struct ArgusParserStruct *parser)
 #define ARGUS_MAX_MASK		32
 
       if ((ptr = strchr(outputfile, '$')) != NULL) {
-         char *file = strdup(ptr);
          char **ap, *mask[ARGUS_MAX_MASK];
+         char *file = strdup(ptr);
+         char *sptr = file;
 
          bzero (mask, sizeof(mask));
+/*
+   ptr = l1buf;
+   while ((obj = strtok(ptr, ":")) != NULL) {
+      if (l1labsindex < 256) {
+         l1labs[l1labsindex].object = obj;
+         l1labsindex++;
+      }
+      ptr = NULL;
+   }
+*/
 
-         for (ap = mask; (*ap = strsep(&file, "$")) != NULL;)
+         for (ap = mask; (*ap = strtok(sptr, "$")) != NULL;) {
+            sptr = NULL;
             if (**ap != '\0')
                if (++ap >= &mask[ARGUS_MAX_MASK])
                   break;
+         }
 
          for (i = 0; i < ARGUS_MAX_MASK; i++) {
             char *word;
             if (mask[i] != NULL) {
-               word = strsep(&mask[i], " ,./_\t\n");
-
-               if (!(ArgusAddMaskList (parser, word)))
-                  ArgusLog(LOG_ERR, "%s: error: mask arg %s", file, word);
+               if ((word = strtok((char *)mask[i], " ,./_\t\n")) != NULL) {
+                  if (!(ArgusAddMaskList (parser, word)))
+                     ArgusLog(LOG_ERR, "%s: error: mask arg %s", file, word);
+               }
             } else
                break;
          }
 
-         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL)) == NULL)
+         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL, ARGUS_OBJ_AGGREGATOR)) == NULL)
             ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
 
          free(file);
@@ -505,11 +538,17 @@ RaParseComplete (int sig)
                         lobj = lobj->nxt;
                      }
                   }
+
+                  ArgusDeleteList(ArgusParser->ArgusWfileList, ARGUS_WFILE_LIST);
                }
+
+               if (ArgusNadp->filename != NULL)
+                  free (ArgusNadp->filename);
                exit(0);
                break;
             }
          }
+
       }
    }
 
@@ -552,7 +591,7 @@ ArgusClientTimeout ()
                }
                ArgusFree(obj);
             } else
-               ArgusAddToQueue(queue, &obj->qhdr, ARGUS_NOLOCK);
+               ArgusAddToQueue (queue, &obj->qhdr, ARGUS_NOLOCK);
          }
       }
 
@@ -762,6 +801,7 @@ ArgusInitNewFilename(struct ArgusParserStruct *parser, struct ArgusWfileStruct *
    /* got new filename, need to check the
       path to be sure that all the directories exist */
 
+   bzero (tmpbuf, sizeof(tmpbuf));
    strncpy (tmpbuf, wfile->filename, MAXSTRLEN);
    if ((tptr = strrchr(tmpbuf, (int) '/')) != NULL) {   /* if there is a path */
       *tptr = '\0';
@@ -833,7 +873,7 @@ RaSendArgusRecord(struct ArgusRecordStruct *argus)
             ArgusGenerateNewFlow(agg, ns);
 
             if ((hstruct = ArgusGenerateHashStruct(agg, ns, (struct ArgusFlow *)&agg->fstruct)) == NULL)
-               ArgusLog (LOG_ERR, "RaProcessThisRecord: ArgusGenerateHashStruct error %s", strerror(errno));
+               ArgusLog (LOG_ERR, "RaSendArgusRecord: ArgusGenerateHashStruct error %s", strerror(errno));
 
             if ((obj = ArgusFindObject(agg->htable, hstruct)) == NULL) {
                if ((obj = (struct ArgusObjectStruct *) ArgusCalloc(1, sizeof(*obj))) == NULL)
@@ -843,7 +883,7 @@ RaSendArgusRecord(struct ArgusRecordStruct *argus)
                ArgusAddToQueue (agg->queue, &obj->qhdr, ARGUS_LOCK);
 
                if ((wfile = (struct ArgusWfileStruct *)ArgusFrontList(ArgusParser->ArgusWfileList)) == NULL)
-                  ArgusLog (LOG_ERR, "RaProcessRecord: no output file specified");
+                  ArgusLog (LOG_ERR, "RaSendArgusRecord: no output file specified");
 
                bcopy(wfile, &obj->wfile, sizeof(*wfile));
                obj->wfile.filename = NULL;
@@ -857,7 +897,7 @@ RaSendArgusRecord(struct ArgusRecordStruct *argus)
 
          } else {
             if ((wfile = (struct ArgusWfileStruct *)ArgusFrontList(ArgusParser->ArgusWfileList)) == NULL)
-               ArgusLog (LOG_ERR, "RaProcessRecord: no output file specified");
+               ArgusLog (LOG_ERR, "RaSendArgusRecord: no output file specified");
          }
          break;
       }
@@ -1057,6 +1097,10 @@ RaSendArgusRecord(struct ArgusRecordStruct *argus)
    }
 
    argus->status |= ARGUS_RECORD_WRITTEN;
+
+#ifdef ARGUSDEBUG
+   ArgusDebug (6, "RaSendArgusRecord () returning %d\n", retn);
+#endif
    return (retn);
 }
 
