@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -66,9 +66,9 @@
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/common/argus_auth.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/common/argus_auth.c#27 $
+ * $DateTime: 2014/06/05 13:44:29 $
+ * $Change: 2845 $
  */
 
 
@@ -116,6 +116,8 @@ extern void ArgusLog (int, char *, ...);
 
 int ArgusInitializeAuthentication (struct ArgusInput *);
 int ArgusAuthenticate (struct ArgusInput *);
+int RaSaslNegotiate(struct ArgusInput *input);
+
 int iptostring(const struct sockaddr *, socklen_t, char *, unsigned);
 
 #ifdef ARGUS_SASL
@@ -123,21 +125,70 @@ int iptostring(const struct sockaddr *, socklen_t, char *, unsigned);
 extern int ArgusMaxSsf;
 extern int ArgusMinSsf;
 
-static int RaGetRealm(void *context, int, const char **, const char **);
-static int RaSimple(void *context, int, const char **, unsigned *);
-static int RaGetSecret(sasl_conn_t *, void *context, int, sasl_secret_t **);
+int RaGetRealm(void *context, int, const char **, const char **);
+int RaSimple(void *context, int, const char **, unsigned *);
+int RaGetSecret(sasl_conn_t *, void *context, int, sasl_secret_t **);
+int ArgusSaslGetPath(void *context __attribute__((unused)), char **);
+int ArgusSaslLog (void *context __attribute__((unused)), int, const char *);
 
-int RaSaslNegotiate(int, int, sasl_conn_t *);
-int RaGetSaslString (int, char *, int);
+#define PLUGINDIR "/usr/lib/sasl2"
+char *searchpath = NULL;
+
+int
+ArgusSaslGetPath(void *context __attribute__((unused)), char ** path)
+{
+  if (! path)
+    return SASL_BADPARAM;
+  if (searchpath)
+    *path = searchpath;
+   else
+    *path = PLUGINDIR;
+
+#ifdef ARGUSDEBUG
+  ArgusDebug(2, "SASL path %s", *path);
+#endif
+
+  return SASL_OK;
+}
+
+int
+ArgusSaslLog (void *context __attribute__((unused)), int priority, const char *message)
+{
+#ifdef ARGUSDEBUG
+  const char *label;
+
+  if (! message)
+    return SASL_BADPARAM;
+
+  switch (priority) {
+     case SASL_LOG_ERR:  label = "Error"; break;
+     case SASL_LOG_NOTE: label = "Info"; break;
+     default:            label = "Other"; break;
+  }
+
+  ArgusDebug(1, "ArgusSaslLog %s: %s", label, message);
+#endif
+
+  return SASL_OK;
+}
+
+
+unsigned int RaGetSaslString (int, char *, int);
 int RaSendSaslString (int, const char *, int, int);
+void RaChop (char *);
 
 /* RaCallBacks we support */
 
-static sasl_callback_t RaCallBacks[] = {
-  { SASL_CB_GETREALM, &RaGetRealm,  NULL },
-  { SASL_CB_USER,     &RaSimple,    NULL },
-  { SASL_CB_AUTHNAME, &RaSimple,    NULL },
-  { SASL_CB_PASS,     &RaGetSecret, NULL },
+typedef int (*funcptr)();
+
+
+
+sasl_callback_t RaCallBacks[] = {
+  { SASL_CB_GETREALM, (funcptr)&RaGetRealm,  NULL },
+  { SASL_CB_USER,     (funcptr)&RaSimple,    NULL },
+  { SASL_CB_AUTHNAME, (funcptr)&RaSimple,    NULL },
+  { SASL_CB_PASS,     (funcptr)&RaGetSecret, NULL },
+  { SASL_CB_GETPATH,  (funcptr)&ArgusSaslGetPath, NULL },
   { SASL_CB_LIST_END, NULL, NULL }
 };
 
@@ -208,10 +259,6 @@ ArgusInitializeAuthentication (struct ArgusInput *input)
 
       if ((retn = sasl_setprop(input->sasl_conn, SASL_IP_REMOTE, &remoteaddr)) != SASL_OK)
          ArgusLog (LOG_ERR, "ArgusInitializeAuthentication() error setting remoteaddr %d", retn);
-
-      retn = 1;
-
-   } else
 */
       retn = 1;
 
@@ -232,7 +279,25 @@ int
 ArgusAuthenticate (struct ArgusInput *input)
 {
    int retn = 0;
+   char *user = NULL, *pass = NULL;
 
+#if defined(ARGUS_THREADS)
+   pthread_mutex_lock(&ArgusParser->lock);
+#endif
+
+   if (input->user) {
+      user = ArgusParser->ustr;
+      ArgusParser->ustr = input->user;
+   }
+
+   if (input->pass) {
+      pass = ArgusParser->pstr;
+      ArgusParser->pstr = input->pass;
+   }
+#if defined(ARGUS_THREADS)
+   pthread_mutex_unlock(&ArgusParser->lock);
+#endif
+   
    if (ArgusInitializeAuthentication(input)) {
 #ifdef ARGUS_SASL
       int fd = input->fd;
@@ -243,12 +308,23 @@ ArgusAuthenticate (struct ArgusInput *input)
       if ((input->out = fd) < 0)
          ArgusLog (LOG_ERR, "ArgusAuthenticate(0x%x) out fd not set");
 
-      if ((retn = RaSaslNegotiate(input->in, input->out, input->sasl_conn)) == SASL_OK)
+      if ((retn = RaSaslNegotiate(input)) == SASL_OK)
          retn = 1;
       else
          retn = 0;
 #endif /* ARGUS_SASL */
    }
+#if defined(ARGUS_THREADS)
+   pthread_mutex_lock(&ArgusParser->lock);
+#endif
+
+   if (user != NULL) ArgusParser->ustr = user;
+   if (pass != NULL) ArgusParser->pstr = pass;
+
+#if defined(ARGUS_THREADS)
+   pthread_mutex_unlock(&ArgusParser->lock);
+#endif
+
 
 #ifdef ARGUSDEBUG
    ArgusDebug (2, "ArgusAuthenticate (0x%x) returning %d\n", input, retn);
@@ -260,7 +336,7 @@ ArgusAuthenticate (struct ArgusInput *input)
 
 #ifdef ARGUS_SASL
 
-static void RaChop (char *s)          /* remove \r\n at end of the line */
+void RaChop (char *s)          /* remove \r\n at end of the line */
 {
    char *p;
 
@@ -274,11 +350,12 @@ static void RaChop (char *s)          /* remove \r\n at end of the line */
       *p-- = '\0';
 }
 
-static int
+int
 RaGetRealm(void *context __attribute__((unused)), int id,
            const char **availrealms, const char **result)
 {
    static char buf[1024];
+   char *ptr;
 
    if (id != SASL_CB_GETREALM)
       return SASL_BADPARAM;
@@ -293,18 +370,18 @@ RaGetRealm(void *context __attribute__((unused)), int id,
    }
    printf("): ");
 
-   fgets(buf, sizeof buf, stdin);
-   RaChop(buf);
-   *result = buf;
+   if ((ptr = fgets(buf, sizeof buf, stdin)) != NULL) {
+      RaChop(buf);
+      *result = buf;
+   }
   
    return SASL_OK;
 }
 
-static char RaSimpleBuf[1024];
+char RaSimpleBuf[1024];
 
-static int
-RaSimple(void *context __attribute__((unused)), int id,
-         const char **result, unsigned *len)
+int
+RaSimple(void *context __attribute__((unused)), int id, const char **result, unsigned *len)
 {
    char *ptr = NULL;
 
@@ -315,7 +392,7 @@ RaSimple(void *context __attribute__((unused)), int id,
       case SASL_CB_USER:
          if (ArgusParser->ustr == NULL) {
             printf("Username: ");
-            fgets(RaSimpleBuf, sizeof RaSimpleBuf, stdin);
+            ptr = fgets(RaSimpleBuf, sizeof RaSimpleBuf, stdin);
 
          } else {
             if ((ptr = strchr(ArgusParser->ustr, '/')) != NULL)
@@ -331,13 +408,17 @@ RaSimple(void *context __attribute__((unused)), int id,
          break;
 
       case SASL_CB_AUTHNAME:
-         if (ArgusParser->ustr != NULL)
-            if ((ptr = strchr(ArgusParser->ustr, '/')) != NULL)
+         if (ArgusParser->ustr != NULL) {
+            if ((ptr = strchr(ArgusParser->ustr, '/')) != NULL) {
                ptr++;
+            } else {
+               ptr = ArgusParser->ustr;
+            }
+         }
 
          if (ptr == NULL) {
-            printf("Username: ");
-            fgets(RaSimpleBuf, sizeof RaSimpleBuf, stdin);
+            printf("Authname: ");
+            ptr = fgets(RaSimpleBuf, sizeof RaSimpleBuf, stdin);
          } else 
             sprintf (RaSimpleBuf, "%s", ptr);
 #ifdef ARGUSDEBUG
@@ -368,13 +449,13 @@ getpassphrase(const char *prompt)
 }
 #endif /* ! HAVE_GETPASSPHRASE */
 
-static int
+int
 RaGetSecret(sasl_conn_t *conn, void *context __attribute__((unused)),
             int id, sasl_secret_t **psecret)
 {
+   static sasl_secret_t *x = NULL;
    char *password;
    size_t len;
-   static sasl_secret_t *x;
 
    if (! conn || ! psecret || id != SASL_CB_PASS)
       return SASL_BADPARAM;
@@ -389,7 +470,8 @@ RaGetSecret(sasl_conn_t *conn, void *context __attribute__((unused)),
 
    len = strlen(password);
 
-   x = (sasl_secret_t *) realloc(x, sizeof(sasl_secret_t) + len);
+   if ((x = (sasl_secret_t *) malloc(sizeof(sasl_secret_t) + len)) == NULL)
+      ArgusLog (LOG_ERR, "RaGetSecret: malloc(%d) error %s\n",  (sizeof(sasl_secret_t) + len));
   
    if (!x) {
       memset(password, 0, len);
@@ -405,13 +487,18 @@ RaGetSecret(sasl_conn_t *conn, void *context __attribute__((unused)),
 
 
 int
-RaSaslNegotiate(int in, int out, sasl_conn_t *conn)
+RaSaslNegotiate(struct ArgusInput *input)
 {
    char buf[8192], mechs[1024];
    const char *chosenmech = NULL;
    const char *data = NULL;
    unsigned int len = 0;
-   int retn = 0;
+   int in, out, retn = 0;
+   sasl_conn_t *conn;
+
+   conn = input->sasl_conn;
+   in  = input->in;
+   out = input->out;
 
 #ifdef ARGUSDEBUG
    ArgusDebug (2, "RaSaslNegotiate(0x%x, 0x%x, 0x%x) receiving capability list... ", in, out, conn);
@@ -419,7 +506,7 @@ RaSaslNegotiate(int in, int out, sasl_conn_t *conn)
 
    bzero(buf, sizeof(buf));
 
-   if ((len = RaGetSaslString(in, buf, sizeof(buf))) <= 0)
+   if ((len = RaGetSaslString(in, buf, sizeof(buf))) == 0)
       ArgusLog (LOG_ERR, "RaSaslNegotiate: RaGetSaslString(0x%x, 0x%x, %d) error %s\n", in, buf, sizeof(buf), strerror(errno));
 
    strcpy(mechs, buf);
@@ -466,14 +553,12 @@ RaSaslNegotiate(int in, int out, sasl_conn_t *conn)
 #ifdef ARGUSDEBUG
          ArgusDebug (2, "waiting for server reply...\n");
 #endif
-         if ((len = RaGetSaslString(in, buf, sizeof(buf))) < 0)
-            ArgusLog (LOG_ERR, "RaSaslNegotiate: RaGetSaslString(%d, 0x%x, %d) returned %d\n", in, buf, sizeof(buf), len);
-
+         len = RaGetSaslString(in, buf, sizeof(buf));
          retn = sasl_client_step(conn, buf, len, NULL, &data, &len);
 
          if ((retn != SASL_OK) && (retn != SASL_CONTINUE)) {
             RaSendSaslString(out, NULL, 0, retn);
-            ArgusLog (LOG_ERR, "RaSaslNegotiate: error performing SASL negotiation");
+            ArgusLog (LOG_ERR, "RaSaslNegotiate: sasl_client_step returned %s", sasl_errdetail(conn));
          }
 
          if (data) {
@@ -490,8 +575,7 @@ RaSaslNegotiate(int in, int out, sasl_conn_t *conn)
       }
 
       if (retn == SASL_OK)
-         if ((len = RaGetSaslString(in, buf, sizeof(buf))) < 0)
-            ArgusLog (LOG_ERR, "RaSaslNegotiate: RaGetSaslString(%d, 0x%x, %d) returned %d\n", in, buf, sizeof(buf), len);
+         len = RaGetSaslString(in, buf, sizeof(buf));
 
    } else  {
       ArgusLog (LOG_ERR, "SASL negotiation: no mechanisms");
@@ -580,7 +664,7 @@ RaSendSaslString (int fd, const char *s, int l, int mode)
    return len;
 }
 
-int
+unsigned int
 RaGetSaslString (int fd, char *buf, int buflen)
 {
    unsigned int len;
