@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,9 +28,9 @@
  */
  
 /*
- * $Id: //depot/argus-3.0.6/clients/common/argus_event.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/common/argus_event.c#20 $
+ * $DateTime: 2014/02/13 18:39:51 $
+ * $Change: 2754 $
  */
  
 
@@ -259,10 +259,11 @@ ArgusProcessSQLEvent (struct ArgusParserStruct *parser, struct ArgusEventObject
    char *RaDatabase = NULL, *RaHost = NULL, *RaUser = NULL;
    char *RaPass = NULL, *RaTable = NULL;
    char *ArgusEventTableName = NULL;
-   char sbuf[MAXSTRLEN], mbuf[MAXSTRLEN], username[1024], *uid; 
-   char userbuf[MAXSTRLEN], *sptr, *hptr; 
+   char sbuf[MAXBUFFERLEN], mbuf[MAXBUFFERLEN];
+   char username[256], userbuf[256]; 
    char rbuf[ARGUS_MAXRECORDSIZE];
    char *ptr, *aname, *accounts = NULL;
+   char *sptr, *hptr; 
    struct ArgusRecord *argus = NULL;
    char stim[128], etim[128];
    struct timeval stvp, etvp;
@@ -328,14 +329,18 @@ ArgusProcessSQLEvent (struct ArgusParserStruct *parser, struct ArgusEventObject
                int ind = 0;
 
                while ((row = mysql_fetch_row(mysqlRes))) {
-                  unsigned long *lengths;
+                  if (ind > RA_MAX_TABLE_ARRAY) {
+                     unsigned long *lengths;
                 
-                  lengths = mysql_fetch_lengths(mysqlRes);
-                  bzero(sbuf, sizeof(sbuf));
-                  for (x = 0; x < retn; x++)
-                     sprintf(&sbuf[strlen(sbuf)], "%.*s", (int) lengths[x], row[x] ? row[x] : "NULL");
+                     lengths = mysql_fetch_lengths(mysqlRes);
+                     bzero(sbuf, sizeof(sbuf));
+                     for (x = 0; x < retn; x++)
+                        sprintf(&sbuf[strlen(sbuf)], "%.*s", (int) lengths[x], row[x] ? row[x] : "NULL");
 
-                  RaExistTableNames[ind++] = strdup (sbuf);
+                     RaExistTableNames[ind++] = strdup (sbuf);
+
+                  } else
+                     break;
                }
 
             } else {
@@ -406,7 +411,7 @@ ArgusProcessSQLEvent (struct ArgusParserStruct *parser, struct ArgusEventObject
       accounts = getpwuid(geteuid())->pw_name;
 
    while ((aname = strtok (accounts, " ")) != NULL) {
-      char *name = NULL, *pass = NULL, *stat = NULL;
+      char *uid = NULL, *name = NULL, *pass = NULL, *stat = NULL;
       char *fullname = NULL, *address = NULL, *telephone = NULL;
       char *mobile = NULL, *fax = NULL, *email = NULL, *url = NULL, *filter = NULL;
 
@@ -442,6 +447,19 @@ ArgusProcessSQLEvent (struct ArgusParserStruct *parser, struct ArgusEventObject
             mysql_free_result(mysqlRes);
          }
       }
+
+      if (uid) free(uid);
+      if (name) free(name);
+      if (fullname) free(fullname);
+      if (address) free(address);
+      if (telephone) free(telephone);
+      if (mobile) free(mobile);
+      if (fax) free(fax);
+      if (email) free(email);
+      if (url) free(url);
+      if (pass) free(pass);
+      if (filter) free(filter);
+      if (stat) free(stat);
     
       accounts = NULL;
    }
