@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -61,9 +61,9 @@
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/common/argus_filter.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/common/argus_filter.c#32 $
+ * $DateTime: 2014/06/17 00:51:52 $
+ * $Change: 2871 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -74,9 +74,20 @@
 #define ArgusFilter
 #endif
 
+
 #include <stdlib.h>
 #include <unistd.h>
 
+#if defined(ARGUS_PLURIBUS)
+#define _STDC_C99
+#else
+#if !defined(__USE_ISOC99)
+#define __USE_ISOC99
+#endif
+#endif
+
+#include <math.h>
+
 #include <errno.h>
 #include <string.h>
 #include <syslog.h>
@@ -85,7 +96,7 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 
-#if defined(HAVE_SOLARIS)
+#if defined(ARGUS_SOLARIS)
 #include <fcntl.h>
 #endif
 
@@ -123,10 +134,41 @@ extern void ArgusLog (int, char *, ...);
 #define EXTRACT_DOUBLE(p)	(*(double *)p)
 #define EXTRACT_LONGLONG(p)	(*(unsigned long long *)p)
 
+static int floatisequal(double, double);
+static int floatisgreaterthan(double, double);
+static int floatislessthan(double, double);
+static int floatisgreaterthanequal(double, double);
+
 #include <ctype.h>
 
-/* Hex digit to integer. */
+#if defined(ARGUS_PLURIBUS)
+#ifndef signbit
+#include <ieeefp.h>
+
 inline int
+signbit(double x)
+{
+   switch (fpclass(x)) {
+     case FP_SNAN:
+     case FP_QNAN:
+     case FP_PZERO:
+     case FP_PNORM:
+     case FP_PDENORM:
+     case FP_PINF:
+        return(0);
+
+     case FP_NDENORM:
+     case FP_NZERO:
+     case FP_NINF:
+     case FP_NNORM:
+        return(1);
+   }
+}
+#endif
+#endif
+
+/* Hex digit to integer. */
+int
 xdtoi(int c)
 {
    if (isdigit(c))
@@ -141,10 +183,10 @@ xdtoi(int c)
  * Execute the filter program starting at pc on the packet p
  * buflen is the amount of data present
  */
-unsigned int ArgusFilterRecord (struct nff_insn *, struct ArgusRecordStruct *);
-unsigned int ArgusFilterOrig (struct nff_insn *, struct ArgusRecordStruct *, int, int);
+int ArgusFilterRecord (struct nff_insn *, struct ArgusRecordStruct *);
+int ArgusFilterOrig (struct nff_insn *, struct ArgusRecordStruct *, int, int);
 
-unsigned int
+int
 ArgusFilterRecord (struct nff_insn *pc, struct ArgusRecordStruct *argus)
 {
    int buflen = sizeof(*argus);
@@ -152,14 +194,55 @@ ArgusFilterRecord (struct nff_insn *pc, struct ArgusRecordStruct *argus)
    if (pc)
       return ArgusFilterOrig (pc, argus, buflen, buflen);
    else
-      return (unsigned int) -1;
+      return -1;
 }
 
-unsigned int
+static int
+floatisequal(double F, double f)
+{
+   extern struct ArgusParserStruct *ArgusParser;
+   double epsilon = pow(0.1, ((ArgusParser->pflag + 1) * 1.0));
+   double pval = pow(10.0, ArgusParser->pflag);
+
+   F = round(F * pval) / pval;
+   f = round(f * pval) / pval;
+   return ((fabs(F - f) <= epsilon) ? ((F == 0) ? (signbit(F) == signbit(f))  : 1) : 0);
+}
+
+static int
+floatisgreaterthan(double F, double f)
+{
+   extern struct ArgusParserStruct *ArgusParser;
+   double pval = pow(10.0, ArgusParser->pflag);
+
+   F = round(F * pval) / pval;
+   f = round(f * pval) / pval;
+   return ((F > f) ? 1 : (((F == 0) && (f == 0)) ? ((!(signbit(F)) && signbit(f)) ? 1 : 0)  : 0));
+}
+
+static int
+floatislessthan(double F, double f)
+{
+   extern struct ArgusParserStruct *ArgusParser;
+   double pval = pow(10.0, ArgusParser->pflag);
+
+   F = round(F * pval) / pval;
+   f = round(f * pval) / pval;
+   return ((F < f) ? 1 : (((F == 0) && (f == 0)) ? ((!(signbit(F)) && signbit(f)) ? 0 : 1)  : 0));
+}
+
+static int
+floatisgreaterthanequal(double F, double f)
+{
+   return (!(floatislessthan(F, f)));
+}
+
+int
 ArgusFilterOrig (struct nff_insn *pc, struct ArgusRecordStruct *argus, int wirelen, int buflen)
 {
-   unsigned long long A = 0, X = 0;
-   long long k, mem [NFF_MEMWORDS];
+   nff_int64 A = 0, X = 0;
+   nff_int64 mem [NFF_MEMWORDS];
+   int k, retn;
    u_char *p = (u_char *)argus;
    float F;
 
@@ -167,415 +250,478 @@ ArgusFilterOrig (struct nff_insn *pc, struct ArgusRecordStruct *argus, int wirel
    while (1) {
       ++pc;
       switch (pc->code) {
-      default:
-         abort();
+         default:
+            abort();
 
-      case NFF_RET|NFF_K:
-         return (unsigned int)pc->data.k;
-
-      case NFF_RET|NFF_A:
-         return (unsigned int)A;
-
-      case NFF_LD|NFF_D|NFF_DSR: {
-         if (pc->dsr >= 0) {
-            struct ArgusDSRHeader *dsr;
-            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
-               u_char *ptr = (u_char *)dsr;
-               k = pc->data.k;
-               if (k + sizeof(double) > (dsr->argus_dsrvl8.len * 4)) 
-                  A = -1;
-               else
-                  A = EXTRACT_DOUBLE(&ptr[k]);
-            } else
-               A = -1;
-         } else {
-            struct ArgusRecordHeader *hdr;
-            if ((hdr = &argus->hdr) != NULL) {
-               u_char *ptr = (u_char *)hdr;
-               k = pc->data.k;
-               if (k + sizeof(double) > sizeof(hdr))
-                  A = -1;
-               else
-                  A = EXTRACT_DOUBLE(&ptr[k]);
-            } else
-               A = -1;
+         case NFF_RET|NFF_K: {
+            retn = pc->data.k;
+            return retn;
          }
-         continue;
-      }
 
-      case NFF_LD|NFF_D|NFF_ABS:
-         k = pc->data.k;
-         if (k + sizeof(double) > buflen) {
-            A = -1;
-         } else
-            A = EXTRACT_DOUBLE(&p[k]);
-         continue;
+         case NFF_RET|NFF_A:
+            return (unsigned int)A;
 
-      case NFF_LD|NFF_F|NFF_DSR: {
-         if (pc->dsr >= 0) {
-            struct ArgusDSRHeader *dsr;
-            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
-               u_char *ptr = (u_char *)dsr;
-               k = pc->data.k;
-               if (k + sizeof(float) > (dsr->argus_dsrvl8.len * 4))
+         case NFF_LD|NFF_D|NFF_DSR: {
+            if (pc->dsr >= 0) {
+               struct ArgusDSRHeader *dsr;
+               if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+                  u_char *ptr = (u_char *)dsr;
+                  k = pc->data.k;
+                  if (k + sizeof(double) > (dsr->argus_dsrvl8.len * 4)) 
+                     A = -1;
+                  else
+                     A = EXTRACT_DOUBLE(&ptr[k]);
+               } else
+                  A = -1;
+            } else {
+               if (argus != NULL) {
+                  u_char *ptr = (u_char *)&argus->hdr;
+                  k = pc->data.k;
+                  if ((k + sizeof(double)) > sizeof(struct ArgusRecordStruct))
+                     A = -1;
+                  else
+                     A = EXTRACT_DOUBLE(&ptr[k]);
+               } else
+                  A = -1;
+            }
+            continue;
+         }
+
+         case NFF_LD|NFF_D|NFF_ABS: {
+            k = pc->data.k;
+            if (k + sizeof(double) > buflen) {
+               A = -1;
+            } else
+               A = EXTRACT_DOUBLE(&p[k]);
+            continue;
+         }
+
+         case NFF_LD|NFF_F|NFF_DSR: {
+            if (pc->dsr >= 0) {
+               struct ArgusDSRHeader *dsr;
+               if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+                  u_char *ptr = (u_char *)dsr;
+                  k = pc->data.k;
+                  if ((k + sizeof(float)) > (dsr->argus_dsrvl8.len * 4))
+                     F = -1;
+                  else
+                     F = EXTRACT_FLOAT(&ptr[k]);
+               } else
                   F = -1;
-               else
-                  F = EXTRACT_FLOAT(&ptr[k]);
-            } else
-               F = -1;
-         } else {
-            struct ArgusRecordHeader *hdr;
-            if ((hdr = &argus->hdr) != NULL) {
-               u_char *ptr = (u_char *)hdr;
-               k = pc->data.k;
-               if (k + sizeof(float) > sizeof(hdr))
+            } else {
+               if (argus != NULL) {
+                  u_char *ptr = (u_char *)&argus->hdr;
+                  k = pc->data.k;
+                  if ((k + sizeof(float)) > sizeof(struct ArgusRecordStruct))
+                     F = -1;
+                  else
+                     F = EXTRACT_FLOAT(&ptr[k]);
+               } else
                   F = -1;
-               else
-                  F = EXTRACT_FLOAT(&ptr[k]);
-            } else
+            }
+            continue;
+         }
+
+         case NFF_LD|NFF_F|NFF_ABS: {
+            k = pc->data.k;
+            if (k + sizeof(float) > buflen) {
                F = -1;
+            } else
+               F = EXTRACT_FLOAT(&p[k]);
+            continue;
          }
-         continue;
-      }
 
-      case NFF_LD|NFF_F|NFF_ABS:
-         k = pc->data.k;
-         if (k + sizeof(float) > buflen) {
-            F = -1;
-         } else
-            F = EXTRACT_FLOAT(&p[k]);
-         continue;
-
-      case NFF_LD|NFF_L|NFF_DSR: {
-         if (pc->dsr >= 0) {
-            struct ArgusDSRHeader *dsr;
-            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
-               u_char *ptr = (u_char *)dsr;
-               k = pc->data.k;
-               if (k + sizeof(long long) > (dsr->argus_dsrvl8.len * 4)) 
+         case NFF_LD|NFF_L|NFF_DSR: {
+            if (pc->dsr >= 0) {
+               struct ArgusDSRHeader *dsr;
+               if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+                  u_char *ptr = (u_char *)dsr;
+                  k = pc->data.k;
+                  if (k + sizeof(long long) > (dsr->argus_dsrvl8.len * 4)) 
+                     A = -1;
+                  else
+                     A = EXTRACT_LONGLONG(&ptr[k]);
+               } else
                   A = -1;
-               else
-                  A = EXTRACT_LONGLONG(&ptr[k]);
-            } else
-               A = -1;
-         } else {
-            struct ArgusRecordHeader *hdr;
-            if ((hdr = &argus->hdr) != NULL) {
-               u_char *ptr = (u_char *)hdr;
-               k = pc->data.k;
-               if (k + sizeof(long long) > sizeof(hdr))
+            } else {
+               if (argus != NULL) {
+                  u_char *ptr = (u_char *)&argus->hdr;
+                  k = pc->data.k;
+                  if ((k + sizeof(long long)) > sizeof(struct ArgusRecordStruct))
+                     A = -1;
+                  else
+                     A = EXTRACT_LONGLONG(&ptr[k]);
+               } else
                   A = -1;
-               else
-                  A = EXTRACT_LONGLONG(&ptr[k]);
-            } else
-               A = -1;
+            }
+            continue;
          }
-         continue;
-      }
 
-      case NFF_LD|NFF_L|NFF_ABS:
-         k = pc->data.k;
-         if ((k + sizeof(long long)) > buflen) {
-            A = -1;
-         } else
-            A = EXTRACT_LONGLONG(&p[k]);
-         continue;
-
-      case NFF_LD|NFF_W|NFF_DSR: {
-         if (pc->dsr >= 0) {
-            struct ArgusDSRHeader *dsr;
-            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
-               u_char *ptr = (u_char *)dsr;
-               k = pc->data.k;
-               if ((k + sizeof(int)) > (dsr->argus_dsrvl8.len * 4))
-                  A = -1;
-               else
-                  A = EXTRACT_LONG(&ptr[k]);
-            } else
+         case NFF_LD|NFF_L|NFF_ABS: {
+            k = pc->data.k;
+            if ((k + sizeof(long long)) > buflen) {
                A = -1;
-
-         } else {
-            struct ArgusRecordHeader *hdr;
-            if ((hdr = &argus->hdr) != NULL) {
-               u_char *ptr = (u_char *)hdr;
-               k = pc->data.k;
-               if (k + sizeof(int) > sizeof(hdr))
-                  A = -1;
-               else
-                  A = EXTRACT_LONG(&ptr[k]);
             } else
-               A = -1;
+               A = EXTRACT_LONGLONG(&p[k]);
+            continue;
          }
-         continue;
-      }
 
-      case NFF_LD|NFF_W|NFF_ABS:
-         k = pc->data.k;
-         if ((k + sizeof(int)) > buflen) {
-            A = -1;
-         } else
-            A = EXTRACT_LONG(&p[k]);
-         continue;
+         case NFF_LD|NFF_W|NFF_DSR: {
+            if (pc->dsr >= 0) {
+               struct ArgusDSRHeader *dsr;
+               if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+                  u_char *ptr = (u_char *)dsr;
+                  k = pc->data.k;
+                  if ((k + sizeof(int)) > (dsr->argus_dsrvl8.len * 4))
+                     A = -1;
+                  else
+                     A = EXTRACT_LONG(&ptr[k]);
+               } else
+                  A = -1;
 
-      case NFF_LD|NFF_H|NFF_DSR: {
-         if (pc->dsr >= 0) {
-            struct ArgusDSRHeader *dsr;
-            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
-               u_char *ptr = (u_char *)dsr;
-               k = pc->data.k;
-               if ((k + sizeof(short)) > (dsr->argus_dsrvl8.len * 4))
+            } else {
+               if (argus != NULL) {
+                  u_char *ptr = (u_char *)&argus->hdr;
+                  k = pc->data.k;
+                  if ((k + sizeof(int)) > sizeof(struct ArgusRecordStruct))
+                     A = -1;
+                  else
+                     A = EXTRACT_LONG(&ptr[k]);
+               } else
                   A = -1;
-               else
-                  A = EXTRACT_SHORT(&ptr[k]);
-            } else
-               A = -1;
-         } else {
-            struct ArgusRecordHeader *hdr;
-            if ((hdr = &argus->hdr) != NULL) {
-               u_char *ptr = (u_char *)hdr;
-               k = pc->data.k;
-               if (k + sizeof(short) > sizeof(hdr))
-                  A = -1;
-               else
-                  A = EXTRACT_SHORT(&ptr[k]);
-            } else
-               A = -1;
+            }
+            continue;
          }
-         continue;
-      }
 
-      case NFF_LD|NFF_H|NFF_ABS:
-         k = pc->data.k;
-         if ((k + sizeof(short)) > buflen)
-            A = -1;
-         else
-            A = EXTRACT_SHORT(&p[k]);
-         continue;
-
-      case NFF_LD|NFF_B|NFF_DSR: {
-         if (pc->dsr >= 0) {
-            struct ArgusDSRHeader *dsr;
-            if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
-               u_char *ptr = (u_char *)dsr;
-               k = pc->data.k;
-               if (k > (dsr->argus_dsrvl8.len * 4))
-                  A = -1;
-               else
-                  A = ptr[k];
-            } else
+         case NFF_LD|NFF_W|NFF_ABS: {
+            k = pc->data.k;
+            if ((k + sizeof(int)) > buflen) {
                A = -1;
-         } else {
-            struct ArgusRecordHeader *hdr;
-            if ((hdr = &argus->hdr) != NULL) {
-               u_char *ptr = (u_char *)hdr;
-               k = pc->data.k;
-               if (k > sizeof(hdr))
-                  A = -1;
-               else
-                  A = ptr[k];
             } else
-               A = -1;
+               A = EXTRACT_LONG(&p[k]);
+            continue;
          }
-         continue;
-      }
 
-      case NFF_LD|NFF_B|NFF_ABS:
-         k = pc->data.k;
-         if (k >= buflen) {
-            A = -1;
-         } else
-            A = p[k];
-         continue;
+         case NFF_LD|NFF_H|NFF_DSR: {
+            if (pc->dsr >= 0) {
+               struct ArgusDSRHeader *dsr;
+               if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+                  u_char *ptr = (u_char *)dsr;
+                  k = pc->data.k;
+                  if ((k + sizeof(short)) > (dsr->argus_dsrvl8.len * 4))
+                     A = -1;
+                  else
+                     A = EXTRACT_SHORT(&ptr[k]);
+               } else
+                  A = -1;
+            } else {
+               if (argus != NULL) {
+                  u_char *ptr = (u_char *)&argus->hdr;
+                  k = pc->data.k;
+                  if ((k + sizeof(short)) > sizeof(struct ArgusRecordStruct))
+                     A = -1;
+                  else
+                     A = EXTRACT_SHORT(&ptr[k]);
+               } else
+                  A = -1;
+            }
+            continue;
+         }
 
-      case NFF_LD|NFF_W|NFF_LEN:
-         A = wirelen;
-         continue;
+         case NFF_LD|NFF_H|NFF_ABS: {
+            k = pc->data.k;
+            if ((k + sizeof(short)) > buflen)
+               A = -1;
+            else
+               A = EXTRACT_SHORT(&p[k]);
+            continue;
+         }
 
-      case NFF_LDX|NFF_W|NFF_LEN:
-         X = wirelen;
-         continue;
+         case NFF_LD|NFF_B|NFF_DSR: {
+            if (pc->dsr >= 0) {
+               struct ArgusDSRHeader *dsr;
+               if ((dsr = argus->dsrs[pc->dsr]) != NULL) {
+                  u_char *ptr = (u_char *)dsr;
+                  k = pc->data.k;
+                  if (k > (dsr->argus_dsrvl8.len * 4))
+                     A = -1;
+                  else
+                     A = ptr[k];
+               } else
+                  A = -1;
+            } else {
+               if (argus != NULL) {
+                  u_char *ptr = (u_char *)&argus->hdr;
+                  k = pc->data.k;
+                  if (k > sizeof(struct ArgusRecordStruct))
+                     A = -1;
+                  else
+                     A = ptr[k];
+               } else
+                  A = -1;
+            }
+            continue;
+         }
 
-      case NFF_LD|NFF_W|NFF_IND:
-         k = X + pc->data.k;
-         if ((k + sizeof(int)) > buflen) {
-            A = -1;
-         } else
-            A = EXTRACT_LONG(&p[k]);
-         continue;
+         case NFF_LD|NFF_B|NFF_ABS: {
+            k = pc->data.k;
+            if (k >= buflen) {
+               A = -1;
+            } else
+               A = p[k];
+            continue;
+         }
 
-      case NFF_LD|NFF_H|NFF_IND:
-         k = X + pc->data.k;
-         if ((k + sizeof(short)) > buflen) {
-            A = -1;
-         } else
-            A = EXTRACT_SHORT(&p[k]);
-         continue;
+         case NFF_LD|NFF_W|NFF_LEN: {
+            A = wirelen;
+            continue;
+         }
 
-      case NFF_LD|NFF_B|NFF_IND:
-         k = X + pc->data.k;
-         if (k >= buflen) {
-            A = -1;
-         } else
-            A = p[k];
-         continue;
+         case NFF_LDX|NFF_W|NFF_LEN: {
+            X = wirelen;
+            continue;
+         }
 
-      case NFF_LDX|NFF_MSH|NFF_B:
-         k = pc->data.k;
-         if (k >= buflen) {
-            X = -1;
-         } else
-            X = (p[pc->data.k] & 0xf) << 2;
-         continue;
+         case NFF_LD|NFF_W|NFF_IND: {
+            k = X + pc->data.k;
+            if ((k + sizeof(int)) > buflen) {
+               A = -1;
+            } else
+               A = EXTRACT_LONG(&p[k]);
+            continue;
+         }
 
-      case NFF_LD|NFF_IMM:
-         A = pc->data.k;
-         continue;
+         case NFF_LD|NFF_H|NFF_IND: {
+            k = X + pc->data.k;
+            if ((k + sizeof(short)) > buflen) {
+               A = -1;
+            } else
+               A = EXTRACT_SHORT(&p[k]);
+            continue;
+         }
 
-      case NFF_LDX|NFF_IMM:
-         X = pc->data.k;
-         continue;
+         case NFF_LD|NFF_B|NFF_IND: {
+            k = X + pc->data.k;
+            if (k >= buflen) {
+               A = -1;
+            } else
+               A = p[k];
+            continue;
+         }
 
-      case NFF_LD|NFF_MEM:
-         A = mem[pc->data.k];
-         continue;
-         
-      case NFF_LDX|NFF_MEM:
-         X = mem[pc->data.k];
-         continue;
+         case NFF_LDX|NFF_MSH|NFF_B: {
+            k = pc->data.k;
+            if (k >= buflen) {
+               X = -1;
+            } else
+               X = (p[k] & 0xf) << 2;
+            continue;
+         }
 
-      case NFF_ST:
-         mem[pc->data.k] = A;
-         continue;
+         case NFF_LD|NFF_IMM: {
+            A = pc->data.k;
+            continue;
+         }
 
-      case NFF_STX:
-         mem[pc->data.k] = X;
-         continue;
+         case NFF_LDX|NFF_IMM: {
+            X = pc->data.k;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JA:
-         pc += pc->data.k;
-         continue;
+         case NFF_LD|NFF_MEM: {
+            k = pc->data.k;
+            A = mem[k];
+            continue;
+         }
+            
+         case NFF_LDX|NFF_MEM: {
+            k = pc->data.k;
+            X = mem[k];
+            continue;
+         }
 
-      case NFF_JMP|NFF_JGT|NFF_K:
-         pc += ((A > pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_ST: {
+            k = pc->data.k;
+            mem[k] = A;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JGE|NFF_K:
-         pc += ((A >= pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_STX: {
+            k = pc->data.k;
+            mem[k] = X;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JEQ|NFF_K:
-         pc += ((A == pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_JMP|NFF_JA: {
+            k = pc->data.k;
+            pc += k;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JGT|NFF_F:
-         pc += ((F > pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_JMP|NFF_JGT|NFF_K: {
+            pc += ((A > pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JGE|NFF_F:
-         pc += ((F >= pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_JMP|NFF_JGE|NFF_K: {
+            pc += ((A >= pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JEQ|NFF_F:
-         pc += ((F == pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_JMP|NFF_JEQ|NFF_K: {
+            pc += ((A == pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JSET|NFF_K:
-         pc += ((A & pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+/*
+         case NFF_JMP|NFF_JGT|NFF_F:
+            pc += ((F > pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
 
-      case NFF_JMP|NFF_JGT|NFF_X:
-         pc += ((A > X) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_JMP|NFF_JGE|NFF_F:
+            pc += ((F >= pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
 
-      case NFF_JMP|NFF_JGE|NFF_X:
-         pc += ((A >= X) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_JMP|NFF_JEQ|NFF_F:
+            pc += ((F == pc->data.f) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+*/
+         case NFF_JMP|NFF_JGT|NFF_F: {
+            pc += ((floatisgreaterthan(F, pc->data.f)) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JEQ|NFF_X:
-         pc += ((A == X) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_JMP|NFF_JGE|NFF_F: {
+            pc += ((floatisgreaterthanequal(F, pc->data.f)) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_JMP|NFF_JSET|NFF_X:
-         pc += ((A & X) && (A != -1)) ? pc->jt : pc->jf;
-         continue;
+         case NFF_JMP|NFF_JEQ|NFF_F: {
+            pc += ((floatisequal(F, pc->data.f)) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_ALU|NFF_ADD|NFF_X:
-         A += X;
-         continue;
-         
-      case NFF_ALU|NFF_SUB|NFF_X:
-         A -= X;
-         continue;
-         
-      case NFF_ALU|NFF_MUL|NFF_X:
-         A *= X;
-         continue;
-         
-      case NFF_ALU|NFF_DIV|NFF_X:
-         if (X == 0)
-            return 0;
-         A /= X;
-         continue;
-         
-      case NFF_ALU|NFF_AND|NFF_X:
-         A &= X;
-         continue;
-         
-      case NFF_ALU|NFF_OR|NFF_X:
-         A |= X;
-         continue;
+         case NFF_JMP|NFF_JSET|NFF_K: {
+            pc += ((A & pc->data.k) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_ALU|NFF_LSH|NFF_X:
-         A <<= X;
-         continue;
+         case NFF_JMP|NFF_JGT|NFF_X: {
+            pc += ((A > X) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_ALU|NFF_RSH|NFF_X:
-         A >>= X;
-         continue;
+         case NFF_JMP|NFF_JGE|NFF_X: {
+            pc += ((A >= X) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_ALU|NFF_ADD|NFF_K:
-         A += pc->data.k;
-         continue;
-         
-      case NFF_ALU|NFF_SUB|NFF_K:
-         A -= pc->data.k;
-         continue;
-         
-      case NFF_ALU|NFF_MUL|NFF_K:
-         A *= pc->data.k;
-         continue;
-         
-      case NFF_ALU|NFF_DIV|NFF_K:
-         A /= pc->data.k;
-         continue;
-         
-      case NFF_ALU|NFF_AND|NFF_K:
-         A &= pc->data.k;
-         continue;
-         
-      case NFF_ALU|NFF_OR|NFF_K:
-         A |= pc->data.k;
-         continue;
+         case NFF_JMP|NFF_JEQ|NFF_X: {
+            pc += ((A == X) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_ALU|NFF_LSH|NFF_K:
-         A <<= pc->data.k;
-         continue;
+         case NFF_JMP|NFF_JSET|NFF_X: {
+            pc += ((A & X) && (A != -1)) ? pc->jt : pc->jf;
+            continue;
+         }
 
-      case NFF_ALU|NFF_RSH|NFF_K:
-         A >>= pc->data.k;
-         continue;
+         case NFF_ALU|NFF_ADD|NFF_X: {
+            A += X;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_SUB|NFF_X: {
+            A -= X;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_MUL|NFF_X: {
+            A *= X;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_DIV|NFF_X: {
+            if (X == 0)
+               return 0;
+            A /= X;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_AND|NFF_X: {
+            A &= X;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_OR|NFF_X: {
+            A |= X;
+            continue;
+         }
 
-      case NFF_ALU|NFF_NEG:
-         A = -A;
-         continue;
+         case NFF_ALU|NFF_LSH|NFF_X: {
+            A <<= X;
+            continue;
+         }
 
-      case NFF_MISC|NFF_TAX:
-         X = A;
-         continue;
+         case NFF_ALU|NFF_RSH|NFF_X: {
+            A >>= X;
+            continue;
+         }
 
-      case NFF_MISC|NFF_TXA:
-         A = X;
-         continue;
+         case NFF_ALU|NFF_ADD|NFF_K: {
+            A += pc->data.k;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_SUB|NFF_K: {
+            A -= pc->data.k;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_MUL|NFF_K: {
+            A *= pc->data.k;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_DIV|NFF_K: {
+            A /= pc->data.k;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_AND|NFF_K: {
+            A &= pc->data.k;
+            continue;
+         }
+            
+         case NFF_ALU|NFF_OR|NFF_K: {
+            A |= pc->data.k;
+            continue;
+         }
+
+         case NFF_ALU|NFF_LSH|NFF_K: {
+            A <<= pc->data.k;
+            continue;
+         }
+
+         case NFF_ALU|NFF_RSH|NFF_K: {
+            A >>= pc->data.k;
+            continue;
+         }
+
+         case NFF_ALU|NFF_NEG: {
+            A = -A;
+            continue;
+         }
+
+         case NFF_MISC|NFF_TAX: {
+            X = A;
+            continue;
+         }
+
+         case NFF_MISC|NFF_TXA: {
+            A = X;
+            continue;
+         }
       }
    }
 }
@@ -743,7 +889,7 @@ argus_next_etherent(FILE *fp)
  */
 
 
-#ifdef HAVE_SOLARIS
+#ifdef ARGUS_SOLARIS
 #include <sys/sockio.h>
 #endif
 
@@ -888,18 +1034,14 @@ ArgusLookupNet(char *device, unsigned int *netp, unsigned int *maskp, char *ebuf
    return (0);
 }
 
-#ifndef HAVE_SYS_ERRLIST
+#if defined(ARGUS_SYS_ERRLIST) && defined(ARGUS_SOLARIS)
 static char ArgusErrorString[128];
 #endif
 
-
 char *
 argus_strerror(int errnum)
 {
-#ifndef HAVE_SYS_ERRLIST
-   extern int sys_nerr;
-   extern char *sys_errlist[];
-
+#if defined(ARGUS_SYS_ERRLIST) && defined(ARGUS_SOLARIS)
    if ((unsigned int)errnum < sys_nerr)
       return ((char *) sys_errlist[errnum]);
 
@@ -1308,8 +1450,10 @@ atomdef(struct stmt *s)
       return X_ATOM;
 
    case NFF_ST:
-   case NFF_STX:
-      return s->data.k;
+   case NFF_STX: {
+      int k = s->data.k;
+      return k;
+   }
 
    case NFF_MISC:
       return NFF_MISCOP(s->code) == NFF_TAX ? X_ATOM : A_ATOM;
@@ -1665,8 +1809,7 @@ opt_peep(struct ablock *b)
           * j  #0   j  x
           */
          last->s.code = NOP;
-         b->s.code = NFF_CLASS(b->s.code) | NFF_OP(b->s.code) |
-            NFF_X;
+         b->s.code = NFF_CLASS(b->s.code) | NFF_OP(b->s.code) |NFF_X;
          done = 0;
       }
    }
@@ -1712,15 +1855,15 @@ opt_peep(struct ablock *b)
       switch (NFF_OP(b->s.code)) {
 
       case NFF_JEQ:
-         v = v == b->s.data.k;
+         v = (v == b->s.data.k);
          break;
 
       case NFF_JGT:
-         v = (unsigned)v > b->s.data.k;
+         v = v > b->s.data.k;
          break;
 
       case NFF_JGE:
-         v = (unsigned)v >= b->s.data.k;
+         v = v >= b->s.data.k;
          break;
 
       case NFF_JSET:
@@ -2850,6 +2993,7 @@ convert_code_r(struct ablock *p)
          continue;
       dst->dsr = src->s.dsr;
       dst->code = (u_short)src->s.code;
+      dst->type = src->s.type;
       dst->data.k = src->s.data.k;
 
       /* fill block-local relative jump */
@@ -2917,6 +3061,7 @@ filled:
 #endif
    dst->dsr = p->s.dsr;
    dst->code = (u_short)p->s.code;
+   dst->type = p->s.type;
    dst->data.k = p->s.data.k;
    if (JT(p)) {
       extrajmps = 0;
@@ -2932,6 +3077,7 @@ filled:
           dst->jt = extrajmps;
           extrajmps++;
           dst[extrajmps].code = NFF_JMP|NFF_JA;
+          dst[extrajmps].type = dst->type;
           dst[extrajmps].data.k = off - extrajmps;
       }
       else
@@ -2949,6 +3095,7 @@ filled:
           dst->jf = extrajmps;
           extrajmps++;
           dst[extrajmps].code = NFF_JMP|NFF_JA;
+          dst[extrajmps].type = dst->type;
           dst[extrajmps].data.k = off - extrajmps;
       }
       else
@@ -3023,14 +3170,14 @@ nff_dump(struct nff_program *p, int option)
    if (option > 2) {
       printf("%d\n", n);
       for (i = 0; i < n; ++insn, ++i) {
-         printf("%lu %lu %lu %Lu\n", (long)insn->code,
+         printf("%lu %lu %lu %lld\n", (long)insn->code,
                 (long)insn->jt, (long)insn->jf, insn->data.k);
       }
       return ;
    }
    if (option > 1) {
       for (i = 0; i < n; ++insn, ++i)
-         printf("{ 0x%x, %d, %d, 0x%08Lx },\n",
+         printf("{ 0x%x, %d, %d, 0x%08llx },\n",
                 insn->code, insn->jt, insn->jf, insn->data.k);
       return;
    }
@@ -3074,12 +3221,13 @@ nff_image(struct nff_insn *p, int n)
    int v, dsr;
    static char image[256];
    char operand[64], op[32];
-   char *fmt;
+   char *fmt, *s;
    float f;
 
    dsr = p->dsr;
-   v = p->data.k;
-   f = p->data.f;
+     v = p->data.k;
+     f = p->data.f;
+     s = p->data.s;
 
    switch (p->code) {
 
@@ -3262,7 +3410,10 @@ nff_image(struct nff_insn *p, int n)
 
    case NFF_JMP|NFF_JEQ|NFF_K:
       sprintf(op, "jeq");
-      fmt = "#0x%x";
+      switch (p->type) {
+         case Q_STRING:  fmt = "\"%s\""; break;
+         case Q_DEFAULT: fmt = "#0x%x"; break;
+      }
       break;
 
    case NFF_JMP|NFF_JSET|NFF_K:
@@ -3389,13 +3540,16 @@ nff_image(struct nff_insn *p, int n)
    if (!(strcmp (fmt, "#%f")))
       (void)snprintf(operand, 64, fmt, f);
    else
+   if (!(strcmp (fmt, "\"%s\""))) {
+      (void)snprintf(operand, 64, fmt, s);
+   } else
       (void)snprintf(operand, 64, fmt, v);
 
-   (void)snprintf(image, 256, 
-            (NFF_CLASS(p->code) == NFF_JMP &&
-             NFF_OP(p->code) != NFF_JA) ?
-            "(%03d) %-8s %-16s jt %d\tjf %d"
-            : "(%03d) %-8s%s",
-            n, op, operand, n + 1 + p->jt, n + 1 + p->jf);
+   if ((NFF_CLASS(p->code) == NFF_JMP) && (NFF_OP(p->code) != NFF_JA))
+      (void)snprintf(image, 256, "(%03d) %-8s %-16s jt %d\tjf %d",
+            n, op, operand, (n + 1 + p->jt), (n + 1 + p->jf));
+   else
+      (void)snprintf(image, 256, "(%03d) %-8s%s", n, op, operand);
+
    return image;
 }
