@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,9 +32,9 @@
  */
 
 /*
- * $Id: //depot/argus-3.0.6/clients/examples/raconvert/raconvert.c#2 $
- * $DateTime: 2012/04/18 14:18:40 $
- * $Change: 2370 $
+ * $Id: //depot/argus/clients/examples/raconvert/raconvert.c#13 $
+ * $DateTime: 2014/05/12 10:49:40 $
+ * $Change: 2818 $
  */
 
 
@@ -71,7 +71,7 @@
 #include <ctype.h>
 #include <strings.h>
 
-#if defined(HAVE_SOLARIS)
+#if defined(ARGUS_SOLARIS)
 #include <string.h>
 #endif
 
@@ -100,7 +100,6 @@ unsigned int ArgusIdType = 0;
 int
 main (int argc, char **argv)
 {
-   extern int optind, opterr;
    extern char *optarg;
    int ArgusExitStatus;
    int i, cc;
@@ -466,9 +465,11 @@ RaConvertParseRecordString (struct ArgusParserStruct *parser, char *str)
    sprintf (delim, "%c", RaConvertDelimiter[0]);
    tptr = ptr;
 
-   for (ap = argv; (*ap = strsep(&tptr, delim)) != NULL;)
+   for (ap = argv; (*ap = strtok(tptr, delim)) != NULL;) {
+      tptr = NULL;
       if (++ap >= &argv[ARGUS_MAX_PRINT_FIELDS])
          break;
+   }
 
    numfields = ((char *)ap - (char *)argv)/sizeof(ap);
 
@@ -884,6 +885,9 @@ ArgusParseStartDateLabel (struct ArgusParserStruct *parser, char *buf)
 
    parser->canon.time.src.start.tv_sec  = tvp->tv_sec;
    parser->canon.time.src.start.tv_usec = tvp->tv_usec;
+
+   parser->canon.time.dst.start.tv_sec  = tvp->tv_sec;
+   parser->canon.time.dst.start.tv_usec = tvp->tv_usec;
 }
 
 
@@ -1063,10 +1067,10 @@ ArgusParseFlagsLabel (struct ArgusParserStruct *parser, char *buf)
 
    if (str[3] != ' ') {
       switch (str[3]) {
-         case '*': RaFlagsIndicationStatus[3] = ARGUS_SRC_PKTS_RETRANS || ARGUS_DST_PKTS_RETRANS; break;
+         case '*': RaFlagsIndicationStatus[3] = ARGUS_SRC_PKTS_RETRANS | ARGUS_DST_PKTS_RETRANS; break;
          case 's': RaFlagsIndicationStatus[3] = ARGUS_SRC_PKTS_RETRANS; break;
          case 'd': RaFlagsIndicationStatus[3] = ARGUS_DST_PKTS_RETRANS; break;
-         case '&': RaFlagsIndicationStatus[3] = ARGUS_SRC_OUTOFORDER || ARGUS_DST_OUTOFORDER; break;
+         case '&': RaFlagsIndicationStatus[3] = ARGUS_SRC_OUTOFORDER | ARGUS_DST_OUTOFORDER; break;
          case 'i': RaFlagsIndicationStatus[3] = ARGUS_SRC_OUTOFORDER; break;
          case 'r': RaFlagsIndicationStatus[3] = ARGUS_DST_OUTOFORDER; break;
       }
@@ -1074,7 +1078,7 @@ ArgusParseFlagsLabel (struct ArgusParserStruct *parser, char *buf)
 
    if (str[4] != ' ') {
       switch (str[4]) {
-         case '@': RaFlagsIndicationStatus[4] = ARGUS_SRC_WINDOW_SHUT || ARGUS_DST_WINDOW_SHUT; break;
+         case '@': RaFlagsIndicationStatus[4] = ARGUS_SRC_WINDOW_SHUT | ARGUS_DST_WINDOW_SHUT; break;
          case 'S': RaFlagsIndicationStatus[4] = ARGUS_SRC_WINDOW_SHUT; break;
          case 'D': RaFlagsIndicationStatus[4] = ARGUS_DST_WINDOW_SHUT; break;
          case '*': RaFlagsIndicationStatus[4] = ARGUS_RTP_SRCSILENCE | ARGUS_RTP_DSTSILENCE; break;
@@ -1085,7 +1089,7 @@ ArgusParseFlagsLabel (struct ArgusParserStruct *parser, char *buf)
 
    if (str[5] != ' ') {
       switch (str[5]) {
-         case 'E': RaFlagsIndicationStatus[5] = ARGUS_SRC_CONGESTED || ARGUS_DST_CONGESTED; break;
+         case 'E': RaFlagsIndicationStatus[5] = ARGUS_SRC_CONGESTED | ARGUS_DST_CONGESTED; break;
          case 'x': RaFlagsIndicationStatus[5] = ARGUS_SRC_CONGESTED; break;
          case 't': RaFlagsIndicationStatus[5] = ARGUS_DST_CONGESTED; break;
       }
