@@ -1,4 +1,4 @@
-/* @(#) $Header: //depot/argus-3.0.6/clients/examples/radump/nlpid.h#1 $ (LBL) */
+/* @(#) $Header: //depot/argus/clients/examples/radump/nlpid.h#6 $ (LBL) */
 /* 
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that: (1) source code
@@ -17,7 +17,7 @@
 
 /*
  * Argus Software.  Common include files - mdp protocol
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  */
@@ -27,7 +27,7 @@
 
 #include <sys/time.h>
 /*
-#if defined(HAVE_SOLARIS)
+#if defined(ARGUS_SOLARIS)
 #include <net/etherdefs.h>
 #endif
 */
