@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,9 +20,9 @@
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/examples/radump/radump.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/examples/radump/radump.c#9 $
+ * $DateTime: 2014/05/12 10:49:40 $
+ * $Change: 2818 $
  */
 
 /*
@@ -293,9 +293,13 @@ RaDumpUserBuffer (struct ArgusParserStruct *parser, struct ArgusRecordStruct *ar
    unsigned short sport = 0, dport = 0;
    int type, proto, process = 0;
    struct ArgusDataStruct *user = NULL;
-   u_char buf[MAXSTRLEN], *bp = NULL, dchr;
+   u_char buf[MAXSTRLEN], *bp = NULL;
    int slen = 0, done = 0;
 
+   if ((user = (struct ArgusDataStruct *)argus->dsrs[ind]) == NULL)
+      return (ArgusBuf);
+
+/*
    switch (ind) {
       case ARGUS_SRCUSERDATA_INDEX:
          dchr = 's';
@@ -304,14 +308,13 @@ RaDumpUserBuffer (struct ArgusParserStruct *parser, struct ArgusRecordStruct *ar
          dchr = 'd';
          break;
    }
+*/
 
-   if ((user = (struct ArgusDataStruct *)argus->dsrs[ind]) != NULL) {
-      bp = (u_char *) &user->array;
-      slen = (user->hdr.argus_dsrvl16.len - 2 ) * 4;
-      slen = (user->count < slen) ? user->count : slen;
-      slen = (slen > len) ? len : slen;
-      snapend = bp + slen;
-   }
+   bp = (u_char *) &user->array;
+   slen = (user->hdr.argus_dsrvl16.len - 2 ) * 4;
+   slen = (user->count < slen) ? user->count : slen;
+   slen = (slen > len) ? len : slen;
+   snapend = bp + slen;
 
    if (flow != NULL) {
       switch (flow->hdr.subtype & 0x3F) {
@@ -443,9 +446,8 @@ struct ArgusMacFlow {
             else if (ISPORT(LDP_PORT))
                ldp_print(bp, slen);
             else {
-               int elen = 0;
                parser->eflag = ArgusThisEflag;
-               elen = ArgusEncode (parser, (const char *)bp, NULL, slen, ArgusBuf, sizeof(ArgusBuf));
+               ArgusEncode (parser, (const char *)bp, NULL, slen, ArgusBuf, sizeof(ArgusBuf));
                parser->eflag = ARGUS_HEXDUMP;
             }
             break;
@@ -500,9 +502,8 @@ struct ArgusMacFlow {
             else if (ISPORT(ISAKMP_PORT_USER1) || ISPORT(ISAKMP_PORT_USER2))
                isakmp_print(bp, slen);
             else {
-               int elen = 0;
                parser->eflag = ArgusThisEflag;
-               elen = ArgusEncode (parser, (const char *)bp, NULL, slen, ArgusBuf, sizeof(ArgusBuf));
+               ArgusEncode (parser, (const char *)bp, NULL, slen, ArgusBuf, sizeof(ArgusBuf));
                parser->eflag = ARGUS_HEXDUMP;
             }
 /*
