@@ -1,7 +1,7 @@
 #!@PERLBIN@
 # 
 #  Argus Software
-#  Copyright (c) 2000-2012 QoSient, LLC
+#  Copyright (c) 2000-2014 QoSient, LLC
 #  All rights reserved.
 # 
 #  This program is free software; you can redistribute it and/or modify
@@ -43,8 +43,8 @@ my $RRD     = $tmpfile.".rrd";
 my $RRDARG  = $tmpfile.".rrdargs";
 my $PNG     = "ragraph.png";
 
-my $RABINS = "@BINDIR@/rabins";
-my $VERSION = "3.0.0";
+my $RABINS = "/usr/local/bin/rabins";
+my $VERSION = "3.0.8";
 my @arglist = ();
 
 my $input = 0;
@@ -73,6 +73,7 @@ my $saddrindex = -1;
 my $daddrindex = -1;
 my $sportindex = -1;
 my $dportindex = -1;
+my $labelindex = -1;
 my $serviceindex = -1;
 my $inodeindex = -1;
 my $protoindex = -1;
@@ -155,6 +156,7 @@ my $nogridfit = 0;
 my $xgrid = "";
 my $ygrid = "";
 my $altygrid = 0;
+my $norrdwmark = 0;
 
 my ($START, $FRAC, $END, $LAST, $SECONDS, $STEP, $COLUMNS, $PROTOS, $PROBES, $RUNS, $ERROR);
 my @line_args;
@@ -173,6 +175,7 @@ ARG: while (my $arg = shift(@ARGV)) {
          s/^-watermark$//         && do { $watermark = shift(@ARGV); next ARG; };
          s/^-imginfo$//           && do { $imginfo = shift(@ARGV); next ARG; };
          s/^-zoom$//              && do { $zoom = shift(@ARGV); next ARG; };
+         s/^-norrdwmark$//        && do { $norrdwmark++; next ARG; };
 
          s/^-fill$//              && do { $fill = 0; next ARG; };
          s/^-stack$//             && do { $stack = 0; next ARG; };
@@ -230,7 +233,9 @@ ARG: while (my $arg = shift(@ARGV)) {
          /^win/                  && do { $arg = "swin dwin";};
          /^mac/                  && do { $arg = "smac";};
          /^addr/                 && do { $arg = "saddr";};
+         /^dport/                && do { $serviceindex++; $arg = "dport"; };
          /^svc/                  && do { $serviceindex++; $arg = "sport dport"; };
+         /^label/                && do { $labelindex++; $arg = "label"; };
          /^pdspkts/              && do { $percent++; $arglist[@arglist + 0] = "dspkts spkts"; next ARG; };
          /^pddpkts/              && do { $percent++; $arglist[@arglist + 0] = "ddpkts dpkts"; next ARG; };
          /^pdsbytes/             && do { $percent++; $arglist[@arglist + 0] = "dsbytes sbytes"; next ARG; };
@@ -281,7 +286,7 @@ exit;
 
 sub RagraphCleanUp {
    print "DEBUG: RagraphCleanUp - rm -f $tmpfile $RRD $RRDARG\n" if $debug;
-   `rm -f $tmpfile $RRD $RRDARG`
+   `rm -f $tmpfile $RRD $RRDARG` if !$debug;
 }
 
 sub RagraphReadInitialValues {
@@ -316,9 +321,9 @@ sub RagraphReadInitialValues {
 sub RagraphProcessArgusData {
    my @args = "";
    if ($histo) {
-      @args = ("$RABINS -M hard zero -p6 -GL0 -s ", @arglist, "> $tmpfile");
+      @args = ("$RABINS -c, -M hard zero -p6 -GL0 -s ", @arglist, "> $tmpfile");
    } else {
-      @args = ("$RABINS -M hard zero -p6 -GL0 -s ltime ", @arglist, "> $tmpfile");
+      @args = ("$RABINS -c, -M hard zero -p6 -GL0 -s ltime ", @arglist, "> $tmpfile");
    }
 
    print "DEBUG: RagraphProcessArgusData: exec: @args\n" if $debug;
@@ -371,6 +376,7 @@ sub RagraphGenerateRRDParameters {
 
          /Inode/	and do {push @typeobjects, $x; $inodeindex = $x; $objwidth = 12; };
          /Service/	and do {push @typeobjects, $x; $serviceindex = $x; $objwidth = 12; };
+         /Label/	and do {push @typeobjects, $x; $labelindex = $x; $objwidth = 20; };
 
          /TotPkts/	and do {push @dataobjects, $x; $srcpktindex = $x; $yaxislabels++; $yaxislabel = "Total Packets" ;};
          /SrcPkts/	and do {push @dataobjects, $x; $srcpktindex  = $x; $yaxislabels++; $yaxislabel = " pkts/sec" ;};
@@ -669,7 +675,7 @@ sub RagraphGenerateRRDParameters {
       }
    
       for my $objs (sort { $objectlist{$b} <=> $objectlist{$a} } keys %objectlist) {
-         if ($objs && (!(($objs eq "0.0.0.0") || ($objs eq "0:0:0:0:0:0") || ($objs eq " ")))) {
+         if ($objs && (!(($objs eq "0.0.0.0") || ($objs eq "0:0:0:0:0:0") || ($objs eq " ") || $objs eq "(null)"))) {
             if (!(($objs eq "ip") && ($objectlist{"ip"} == 0))) {
                push @objects, $objs;
             }
@@ -697,7 +703,7 @@ sub RagraphGenerateRRDParameters {
       for (my $i = 0; $i < (@objects + 0); $i++) {
          for (my $x = 0; $x < (@dataobjects + 0); $x++) {
             for ($columns[$dataobjects[$x]]) {
-               /Tot|Host|Src|Out|SApp|ds/             and do {
+               /Label|Tot|Host|Src|Out|SApp|ds/             and do {
                   $opts[$ind++]          = "DS:$objectIn$ncnt:GAUGE:$STEP:U:$maxval";
                   $line_args[$lineind++] = "DEF:alphaIn$ncnt=$RRD:objectIn$ncnt:AVERAGE";
                   $line_args[$lineind++] = "CDEF:ralphaIn$ncnt=alphaIn$ncnt,$STEP,/";
@@ -742,7 +748,7 @@ sub RagraphGenerateRRDParameters {
                };
             };
 
-            /Host|Out|SApps/             and do {
+            /Label|Host|Out|SApps/             and do {
                $opts[$ind++]          = "DS:$objectIn$ncnt:GAUGE:$STEP:U:$maxval";
                $line_args[$lineind++] = "DEF:alphaIn$ncnt=$RRD:objectIn$ncnt:AVERAGE";
                $line_args[$lineind++] = "CDEF:ralphaIn$ncnt=alphaIn$ncnt,$STEP,/";
@@ -789,12 +795,12 @@ sub RagraphGenerateRRDParameters {
     
    if ($protoindex >= 0) {
       %outColors  = (
-         tcp   => "#a00000", udp   => "#008000", icmp  => "#000080", igmp  => "#008080",
-         ospf  => "#800080", pim   => "#808080", mcast => "#808000", ip    => "#707070"
+         tcp   => "#3030A0", udp   => "#008000", icmp  => "#800000", igmp  => "#008080",
+         ospf  => "#8000A0", pim   => "#808080", mcast => "#808000", ip    => "#707070"
       );
     
       %inColors = (
-         tcp   => "#f00000", udp   => "#00f000", icmp  => "#0000e0", igmp  => "#00e0e0",
+         tcp   => "#4040e0", udp   => "#00f000", icmp  => "#e00000", igmp  => "#00e0e0",
          ospf  => "#e000e0", pim   => "#e0e0e0", mcast => "#e0e000", ip    => "#b0b0b0"
       );
     
@@ -804,6 +810,7 @@ sub RagraphGenerateRRDParameters {
             afs3call => "#808080",    afs3fsrv => "#808080",   afs3kasrv => "#808080",
            afs3prsrv => "#808080",   afs3vlsrv => "#808080",  afs3volsrv => "#808080",
                  aol => "#de2d31",        arcp => "#808080",       argus => "#f5d541",
+              ashwin => "#8e0d01",
               bootps => "#cf4334",  cslistener => "#808080",     dditcpl => "#808080",
               domain => "#800080",      dtspcd => "#808080",      finger => "#808080",
                  ftp => "#808080", gnutellartr => "#808080", gnutellasvc => "#808080",
@@ -814,7 +821,9 @@ sub RagraphGenerateRRDParameters {
                  ipp => "#808080",         irc => "#808080",       irdmi => "#808080",
                kazaa => "#808080",    kerberos => "#808080",  kerberosiv => "#808080",
                 kpop => "#808080",      krb524 => "#808080",        ldap => "#808080",
-         microsoftds => "#808080",        msnp => "#808080", msstreaming => "#808080",
+      "microsoft-ds" => "#800080",        msnp => "#808080", msstreaming => "#808080",
+    "micromuse-ncps" => "#00B000",  "ms-sql-s" => "#B0B000",      msdfsr => "#20A0A0",
+    "micromuse-ncpw" => "#008000",
                 name => "#808080",  netbiosdgm => "#808080",   netbiosns => "#808080",
           netbiosssn => "#808080",  nfsdstatus => "#808080",     nicname => "#808080",
                  nkd => "#808080",        nntp => "#808080",       ntalk => "#808080",
@@ -833,7 +842,8 @@ sub RagraphGenerateRRDParameters {
          %inColors  = (
             afs3call => "#b0b0b0",    afs3fsrv => "#b0b0b0",   afs3kasrv => "#b0b0b0",
            afs3prsrv => "#b0b0b0",   afs3vlsrv => "#b0b0b0",  afs3volsrv => "#b0b0b0",
-                 aol => "#fca633",        arcp => "#b0b0b0",        argus => "#ffff66",
+                 aol => "#fca633",        arcp => "#b0b0b0",       argus => "#ffff66",
+              ashwin => "#bc2613",
               bootps => "#fe5442",  cslistener => "#b0b0b0",     dditcpl => "#b0b0b0",
               domain => "#e000e0",      dtspcd => "#b0b0b0",      finger => "#b0b0b0",
                  ftp => "#b0b0b0", gnutellartr => "#b0b0b0", gnutellasvc => "#b0b0b0",
@@ -844,7 +854,9 @@ sub RagraphGenerateRRDParameters {
                  ipp => "#b0b0b0",         irc => "#b0b0b0",       irdmi => "#b0b0b0",
                kazaa => "#b0b0b0",    kerberos => "#b0b0b0",  kerberosiv => "#b0b0b0",
                 kpop => "#b0b0b0",      krb524 => "#b0b0b0",        ldap => "#b0b0b0",
-         microsoftds => "#b0b0b0",        msnp => "#b0b0b0", msstreaming => "#b0b0b0",
+      "microsoft-ds" => "#b000b0",        msnp => "#b0b0b0", msstreaming => "#b0b0b0",
+    "micromuse-ncps" => "#00E000",  "ms-sql-s" => "#E0E000",      msdfsr => "#20C0C0",
+    "micromuse-ncpw" => "#00B000",
                 name => "#b0b0b0",  netbiosdgm => "#b0b0b0",   netbiosns => "#b0b0b0",
           netbiosssn => "#b0b0b0",  nfsdstatus => "#b0b0b0",     nicname => "#b0b0b0",
                  nkd => "#b0b0b0",        nntp => "#b0b0b0",       ntalk => "#b0b0b0",
@@ -936,7 +948,7 @@ sub RagraphGenerateRRDParameters {
             $tobj =~ s/:/\\:/g;
             for (my $x = 0; $x < (@dataobjects + 0); $x++) {
                for ($columns[$dataobjects[$x]]) {
-                  /Host|Tot|Src|Out|SApp|ds/   and do {
+                  /Label|Host|Tot|Src|Out|SApp|ds/   and do {
                      if (!($col = $inColors{$objects[$i]})) {
                         $col = "#808070";
                      }
@@ -963,13 +975,10 @@ sub RagraphGenerateRRDParameters {
          }
       }
 
-      if ($ncnt > 0) {
-         $line_args[$lineind++] = "COMMENT:\\s";
-         $line_args[$lineind++] = "COMMENT:\\s";
-         if ($ncnt > 7) {
-            $line_args[$lineind++] = "COMMENT:\\s";
-         }
-      }
+      $line_args[$lineind++] = "COMMENT:\\s";
+      $line_args[$lineind++] = "COMMENT:\\s";
+      $line_args[$lineind++] = "COMMENT:\\s";
+      $line_args[$lineind++] = "COMMENT:\\s";
 
       for (my $i = 0; $i < (@objects + 0); $i++) {
          if (!($objects[$i] eq "")) {
@@ -979,7 +988,7 @@ sub RagraphGenerateRRDParameters {
                for ($columns[$dataobjects[$x]]) {
                   /Dst|In|DApp|dd/  and do {
                      if (!($col = $outColors{$objects[$i]})) {
-                        $col = "#b0b0b0";
+                        $col = "#707070";
                      }
                      for ($columns[$dataobjects[$x]]) {
                         /In/  and do {
@@ -998,7 +1007,7 @@ sub RagraphGenerateRRDParameters {
                   };
                   /Trans|Dur|Time/  and do {
                      if (!($col = $inColors{$objects[$i]})) {
-                        $col = "#808070";
+                        $col = "#a0a0a0";
                      }
                      $object = sprintf "%-*.*s \l", $objwidth, $objwidth, $tobj;
                      if ($ocnt == 0) {
@@ -1013,6 +1022,9 @@ sub RagraphGenerateRRDParameters {
          }
       }
 
+      $line_args[$lineind++] = "COMMENT:\\s";
+      $line_args[$lineind++] = "COMMENT:\\s";
+
    } else {
       my $ncnt = 0;
       my $ocnt = 0;
@@ -1022,7 +1034,7 @@ sub RagraphGenerateRRDParameters {
          for (my $object = $objects[$x]) {
             /Tot|Src|ds/          and do {
                if (!($col = $outcolors[$ncnt])) {
-                  $col = "#707070";
+                  $col = "#a0a0a0";
                }
                if ($percent) {
                   for ($objects[$x]) {
@@ -1047,9 +1059,9 @@ sub RagraphGenerateRRDParameters {
                }
             };
 
-            /Host|Out|SApp/    and do {
+            /Label|Host|Out|SApp/    and do {
                if (!($col = $outcolors[$ncnt])) {
-                  $col = "#707070";
+                  $col = "#a0a0a0";
                }
                if ($ncnt == 0) {
                   $line_args[$lineind++] = "$style:ralphaIn$ncnt$col:$object";
@@ -1061,7 +1073,7 @@ sub RagraphGenerateRRDParameters {
 
             /Dst|dd/   and do {
                if (!($col = $outcolors[$ocnt])) {
-                  $col = "#b0b0b0";
+                  $col = "#707070";
                }
                if ($percent) {
                   for ($objects[$x]) {
@@ -1088,7 +1100,7 @@ sub RagraphGenerateRRDParameters {
 
             /In|DApp|Trans|Dur|Time/   and do {
                if (!($col = $outcolors[$ocnt % $colorcnt])) {
-                  $col = "#b0b0b0";
+                  $col = "#707070";
                }
                if ($ocnt == 0) {
                   $line_args[$lineind++] = "$style:ralphaOut$ocnt$col:$object";
@@ -1101,14 +1113,13 @@ sub RagraphGenerateRRDParameters {
       }
    }
    
-
    for (my $x = 0; $x < (@columns + 0); $x++) {
       for ($columns[$x]) {
          /^SIntPkt/ and do {$power[$x] = $STEP ; };
          /^DIntPkt/ and do {$power[$x] = $STEP ; };
          /Pkts/     and do {$power[$x] = 1.0 ; };
-         /Rate/      and do {$power[$x] = $STEP ; };
-         /Load/      and do {$power[$x] = $STEP ; };
+         /Rate/     and do {$power[$x] = $STEP ; };
+         /Load/     and do {$power[$x] = $STEP ; };
          /Bytes/    and do {$power[$x] = 1.0 * 8.0; };
          /Jitter/   and do {$power[$x] = 1.0; };
          /Loss/     and do {$power[$x] = 1.0 ; };
@@ -1259,7 +1270,7 @@ sub RagraphGenerateRRD {
                for (my $x = 0; $x < (@dataobjects + 0); $x++) {
                   my $dataindex = $dataobjects[$x];
                   for ($columns[$dataindex]) {
-                     /Host|Tot|Src|Out|SApp|ds/       and do {
+                     /Label|Host|Tot|Src|Out|SApp|ds/       and do {
                         $objectvals{$thistime, $thisobject, "src"} += $opts[$dataindex] * $power[$dataindex];
                      };
     
@@ -1279,7 +1290,7 @@ sub RagraphGenerateRRD {
 
          for (my $i = 0; $i < (@objects + 0); $i++) {
             for ($objects[$i]) {
-               /Host|Tot|Src|Out|SApp|ds/       and do { $label = "src"; };
+               /Label|Host|Tot|Src|Out|SApp|ds/       and do { $label = "src"; };
                /Dst|In|DApp|Trans|Dur|Time|dd/  and do { $label = "dst"; };
             }
             $objectvals{$thistime, $objects[$i], $label} += $opts[$i + 1] * $power[$i + 1];
@@ -1320,7 +1331,7 @@ sub RagraphGenerateRRD {
                      }
                   };
  
-                  /Host|Tot|Src|Out|SApp|ds/       and do {
+                  /Label|Host|Tot|Src|Out|SApp|ds/       and do {
                      if (!($ovalue = $objectvals{$thisbin, $objects[$i], "src"})) {
                         $ovalue = 0;
                      }
@@ -1366,7 +1377,7 @@ sub RagraphGenerateRRD {
                   }
                };
 
-               /Host|Out|SApp/            and do { $label = "src"; };
+               /Label|Host|Out|SApp/      and do { $label = "src"; };
                /In|DApp|Trans|Dur|Time/   and do { $label = "dst"; };
             }
 
@@ -1420,7 +1431,7 @@ sub RagraphGeneratePNG {
          "--color", "BACK#EAEAE0",
          "--color", "SHADEA#EAEAE0",
          "--color", "SHADEB#EAEAE0",
-         "--color", "CANVAS#FCFCF0",
+         "--color", "CANVAS#FAFAF0",
          "--interlace", "--imgformat","PNG",
    );
 
@@ -1546,6 +1557,10 @@ sub RagraphGeneratePNG {
       $rrd_args[++$#rrd_args] = $lowerval;
    }
 
+   if ($norrdwmark) {
+      $rrd_args[++$#rrd_args] = "--disable-rrdtool-tag";
+   }
+
    print "DEBUG: RRDs::last $RRD\n" if $debug;
    
    my $last = RRDs::last $RRD;
