@@ -1,7 +1,7 @@
 #!@PERLBIN@
 # 
 #  Argus Client Software
-#  Copyright (c) 2000-2012 QoSient, LLC
+#  Copyright (c) 2000-2014 QoSient, LLC
 #  All rights reserved.
 # 
 #  This program is free software; you can redistribute it and/or modify
@@ -298,9 +298,9 @@ sub RagraphReadInitialValues {
 sub RagraphProcessArgusData {
    my @args = "";
    if ($histo) {
-      @args = ("$RABINS -M soft zero -p6 -GL0 -s ", @arglist, "> $tmpfile");
+      @args = ("$RABINS -c, -M hard zero -p6 -GL0 -s ", @arglist, "> $tmpfile");
    } else {
-      @args = ("$RABINS -M soft zero -p6 -GL0 -s ltime ", @arglist, "> $tmpfile");
+      @args = ("$RABINS -c, -M hard zero -p6 -GL0 -s ltime ", @arglist, "> $tmpfile");
    }
 
    print "DEBUG: RagraphProcessArgusData: exec: @args\n" if $debug;
