@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,9 +25,9 @@
  * written by Carter Bullard
  * QoSient, LLC
  * 
- * $Id: //depot/argus-3.0.6/clients/examples/rahisto/rahisto.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/examples/rahisto/rahisto.c#10 $
+ * $DateTime: 2014/05/12 10:49:40 $
+ * $Change: 2818 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -109,7 +109,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
          }
       }
  
-      if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL)) == NULL)
+      if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL, ARGUS_RECORD_AGGREGATOR)) == NULL)
          ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
  
       if (parser->Hstr) {
@@ -176,7 +176,7 @@ RaParseComplete (int sig)
                char buf[MAXSTRLEN];
                if ((tagr = (void *)ns->dsrs[ARGUS_AGR_INDEX]) != NULL) {
                   if (parser->ArgusWfileList == NULL) {
-                     int len, tlen, numModes, pflag = parser->pflag;
+                     int len, tlen, numModes = 0, pflag = parser->pflag;
                      double modeValues[1024];
                      double median = 0.0, percentile = 0.0;
                      char *meanStr = NULL, *medianStr = NULL, *percentStr = NULL;
@@ -268,6 +268,13 @@ RaParseComplete (int sig)
                      if ((c = ArgusParser->RaFieldDelimiter) != '\0')
                         printf ("\n");
                         
+                     if (stdStr)     free(stdStr);
+                     if (meanStr)    free(meanStr);
+                     if (medianStr)  free(medianStr);
+                     if (percentStr) free(percentStr);
+                     if (maxValStr)  free(maxValStr);
+                     if (minValStr)  free(minValStr);
+                     if (modeStr)    free(modeStr);
                   }
                }
 
@@ -294,10 +301,10 @@ RaParseComplete (int sig)
                            printf ("\n");
                      } else {
                         if (ArgusPrintInterval) 
-                           printf (" Class    %*.*s%s%*.*s       Freq    Rel.Freq     Cum.Freq    %s\n",
+                           printf (" Class     %*.*s%s%*.*s       Freq    Rel.Freq     Cum.Freq    %s\n",
                                 rblen, rblen, " ", "Interval", rblen, rblen, " ", ArgusParser->RaLabel);
                         else
-                           printf (" Class   %*.*s%s%*.*s       Freq    Rel.Freq     Cum.Freq    %s\n",
+                           printf (" Class    %*.*s%s%*.*s       Freq    Rel.Freq     Cum.Freq    %s\n",
                                 rblen, rblen, " ", "Interval", rblen, rblen, " ", ArgusParser->RaLabel);
                      }
                   }
@@ -449,10 +456,10 @@ RaParseComplete (int sig)
                                     printf ("\n");
                               } else {
                                  if (ArgusPrintInterval) {
-                                    printf ("%6d   %*.*e-%*.*e %10d   %8.4f%%    %8.4f%%    %s\n",
+                                    printf ("%6d   % *.*e-%*.*e %10d   %8.4f%%    %8.4f%%    %s\n",
                                           class++, size, size, bs, size, size, be, freq, rel * 100.0, relcum * 100.0, buf);
                                  } else {
-                                    printf ("%6d   %*.*e %10d   %8.4f%%    %8.4f%%    %s\n",
+                                    printf ("%6d   % *.*e %10d   %8.4f%%    %8.4f%%    %s\n",
                                           class++, size, size, bs, freq, rel * 100.0, relcum * 100.0, buf);
                                  }
                               }
