@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,9 +28,9 @@
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/examples/ramysql/rasql.c#2 $
- * $DateTime: 2012/04/18 14:18:40 $
- * $Change: 2370 $
+ * $Id: //depot/argus/clients/examples/ramysql/rasql.c#20 $
+ * $DateTime: 2014/06/17 17:45:53 $
+ * $Change: 2877 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -92,7 +92,7 @@ struct timeval ArgusLastTime         = {0, 0};
 struct timeval ArgusThisTime         = {0, 0};
 struct timeval ArgusCurrentTime      = {0, 0};
  
-char ArgusSQLSaveTableNameBuf[1024];
+char ArgusSQLSaveTableNameBuf[MAXSTRLEN];
 struct tm ArgusSaveTableTmStruct;
 time_t ArgusSaveTableSeconds = 0;
  
@@ -125,10 +125,10 @@ char  *RaTableCreateNames[RA_MINTABLES];
 char *RaTableCreateString[RA_MINTABLES];
 char *RaTableDeleteString[RA_MINTABLES];
 
-#define ARGUSSQLMAXCOLUMNS	128
+#define ARGUSSQLMAXCOLUMNS	256
 char *ArgusTableColumnName[ARGUSSQLMAXCOLUMNS];
 
-char ArgusSQLTableNameBuf[1024];
+char ArgusSQLTableNameBuf[MAXSTRLEN];
 
 char *RaSource         = NULL;
 char *RaArchive        = NULL;
@@ -147,6 +147,10 @@ int ArgusSQLBulkBufferSize = 0;
 int ArgusSQLBulkBufferIndex = 0;
 char *ArgusSQLBulkLastTable = NULL;
 char *ArgusSQLBulkBuffer = NULL;
+char *ArgusSQLVersion = NULL;
+int MySQLVersionMajor = 0;
+int MySQLVersionMinor = 0;
+int MySQLVersionSub = 0;
 
 extern int ArgusSOptionRecord;
 int ArgusDeleteTable = 0;
@@ -170,7 +174,7 @@ struct RaMySQLFileStruct {
    unsigned int fileindex;
    unsigned int second;
    char *filename;
-   unsigned int ostart, ostop;
+   int ostart, ostop;
 };
 
 #define RAMYSQL_SECONDTABLE_PROBE       0
@@ -185,7 +189,7 @@ struct RaMySQLSecondsTable {
    char *filename;
    unsigned int probe;
    unsigned int second;
-   unsigned int ostart, ostop;
+   int ostart, ostop;
 };
 
 #define RAMYSQL_PROBETABLE_PROBE        0
@@ -214,14 +218,14 @@ RaParseComplete (int sig)
       if (!ArgusParser->RaParseCompleting++) {
          ArgusShutDown(sig);
          if ((sig >= 0) && ArgusParser->aflag) {
-            printf (" Totalrecords %-8Ld  TotalManRecords %-8Ld  TotalFarRecords %-8Ld TotalPkts %-8Ld TotalBytes %-8Ld\n",
+            printf (" Totalrecords %-8lld  TotalManRecords %-8lld  TotalFarRecords %-8lld TotalPkts %-8lld TotalBytes %-8lld\n",
                           ArgusParser->ArgusTotalRecords,
                           ArgusParser->ArgusTotalMarRecords, ArgusParser->ArgusTotalFarRecords,
                           ArgusParser->ArgusTotalPkts, ArgusParser->ArgusTotalBytes);
          }
       }
       fflush(stdout);
-      mysql_close(&mysql);
+      mysql_close(RaMySQL);
       exit(0);
    }
 }
@@ -294,7 +298,7 @@ RaProcessRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *arg
 void
 RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *argus)
 {
-   static char buf[MAXSTRLEN];
+   static char buf[MAXARGUSRECORD];
 
    if (parser->ArgusWfileList != NULL) {
       struct ArgusWfileStruct *wfile = NULL;
@@ -313,7 +317,7 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
                if (retn != 0) {
                   if ((parser->exceptfile == NULL) || strcmp(wfile->filename, parser->exceptfile)) {
                      struct ArgusRecord *argusrec = NULL;
-                     static char sbuf[0x10000];
+                     static char sbuf[MAXARGUSRECORD];
                      if ((argusrec = ArgusGenerateRecord (argus, 0L, sbuf)) != NULL) {
 #ifdef _LITTLE_ENDIAN
                         ArgusHtoN(argusrec);
@@ -410,13 +414,16 @@ void ArgusWindowClose(void) {
 void
 RaMySQLInit ()
 {
-   struct ArgusInput *input = NULL;
-   char userbuf[1024], sbuf[1024];
+   my_bool reconnectbuf = 1, *reconnect = &reconnectbuf;
+   char userbuf[1024], sbuf[1024], db[1024], *dbptr = NULL;
    char *sptr = NULL, *ptr;
    MYSQL_RES *mysqlRes;
    int retn = 0, x;
 
-   bzero(&RaTableExistsNames, sizeof(RaTableExistsNames));
+   bzero((char *)RaTableExistsNames,  sizeof(RaTableExistsNames));
+   bzero((char *)RaTableCreateNames,  sizeof(RaTableCreateNames));
+   bzero((char *)RaTableCreateString, sizeof(RaTableCreateString));
+   bzero((char *)RaTableDeleteString, sizeof(RaTableDeleteString));
 
    if ((RaUser == NULL) && (ArgusParser->dbuserstr != NULL)) {
       bzero(userbuf, sizeof(userbuf));
@@ -438,128 +445,129 @@ RaMySQLInit ()
       else if (ArgusParser->readDbstr != NULL)
          RaDatabase = strdup(ArgusParser->readDbstr);
 
-      if (RaDatabase)
-         if (!(strncmp("mysql:", RaDatabase, 6)))
-            RaDatabase = &RaDatabase[6];
+      if (RaDatabase) 
+         if (!(strncmp("mysql:", RaDatabase, 6))) {
+            char *tmp = RaDatabase;
+            RaDatabase = strdup(&RaDatabase[6]);
+            free(tmp);
+         }
    }
       
-   if (RaDatabase == NULL)
-      ArgusLog(LOG_ERR, "must specify database "); 
+   if (RaDatabase == NULL) {
+      ArgusLog(LOG_ERR, "must specify database");
 
+   } else {
+      sprintf(db, "%s", RaDatabase);
+      dbptr = db;
 /*
       //[[username[:password]@]hostname[:port]]/database/tablename
 */
 
-   if ((input = ArgusParser->ArgusInputFileList) != NULL) {
-      char *table = input->filename;
-
-// assumption here is that if "-r tableSpec" is used, without the
-// mysql: keyword, with rasql.c we should still be able to find
-// the string as if it were a table.
-
-      if (!(strncmp ("//", table, 2))) {
-         if ((strncmp ("///", table, 3))) {
-            RaDatabase = &table[2];
-            RaHost = RaDatabase;
-            if ((ptr = strchr (RaDatabase, '/')) != NULL) {
+      if (!(strncmp ("//", dbptr, 2))) {
+         char *rhost = NULL, *ruser = NULL, *rpass = NULL;
+         if ((strncmp ("///", dbptr, 3))) {
+            dbptr = &dbptr[2];
+            rhost = dbptr;
+            if ((ptr = strchr (dbptr, '/')) != NULL) {
                *ptr++ = '\0';
-               RaDatabase = ptr;
+               dbptr = ptr;
 
-               if ((ptr = strchr (RaHost, '@')) != NULL) {
-                  RaUser = RaHost;
+               if ((ptr = strchr (rhost, '@')) != NULL) {
+                  ruser = rhost;
                   *ptr++ = '\0';
-                  RaHost = ptr;
-                  if ((ptr = strchr (RaUser, ':')) != NULL) {
+                  rhost = ptr;
+                  if ((ptr = strchr (ruser, ':')) != NULL) {
                      *ptr++ = '\0';
-                     RaPass = ptr;
+                     rpass = ptr;
                   } else {
-                     RaPass = NULL;
+                     rpass = NULL;
                   }
                }
 
-               if ((ptr = strchr (RaHost, ':')) != NULL) {
+               if ((ptr = strchr (rhost, ':')) != NULL) {
                   *ptr++ = '\0';
                   RaPort = atoi(ptr);
                }
             } else
-               RaDatabase = NULL;
+               dbptr = NULL;
 
          } else {
-            RaDatabase = &table[3];
+            dbptr = &dbptr[3];
          }
-      }
-      if ((ptr = strchr (table, '/')) != NULL) {
-         RaDatabase = table;
-         *ptr++ = '\0';
-         RaTable = ptr;
-      } else
-         RaTable = table;
 
-   } else {
-      if (!(strncmp ("//", RaDatabase, 2))) {
-         if ((strncmp ("///", RaDatabase, 3))) {
-            RaDatabase = &RaDatabase[2];
-            RaHost = RaDatabase;
-            if ((ptr = strchr (RaDatabase, '/')) != NULL) {
-               *ptr++ = '\0';
-               RaDatabase = ptr;
-
-               if ((ptr = strchr (RaHost, '@')) != NULL) {
-                  RaUser = RaHost;
-                  *ptr++ = '\0';
-                  RaHost = ptr;
-                  if ((ptr = strchr (RaUser, ':')) != NULL) {
-                     *ptr++ = '\0';
-                     RaPass = ptr;
-                  } else {
-                     RaPass = NULL;
-                  }
-               }
-
-               if ((ptr = strchr (RaHost, ':')) != NULL) {
-                  *ptr++ = '\0';
-                  RaPort = atoi(ptr);
-               }
-            } else
-               RaDatabase = NULL;
-
-         } else {
-            RaDatabase = &RaDatabase[3];
+         if (ruser != NULL) {
+            if (RaUser != NULL) free(RaUser);
+            RaUser = strdup(ruser);
          }
+         if (rpass != NULL) {
+            if (RaPass != NULL) free(RaPass);
+            RaPass = strdup(rpass);
+         }
+         if (rhost != NULL) {
+            if (RaHost != NULL) free(RaHost);
+            RaHost = strdup(rhost);
+         }
+         free(RaDatabase);
+         RaDatabase = strdup(dbptr);
       }
-    
-      if ((ptr = strchr (RaDatabase, '/')) != NULL) {
-         *ptr++ = '\0';
-         RaTable = ptr;
+   }
 
-         if (ArgusParser->writeDbstr != NULL)
-            RaSQLSaveTable = strdup(RaTable);
-      }
+   if ((ptr = strchr (RaDatabase, '/')) != NULL) {
+      *ptr++ = '\0';
+      RaTable = ptr;
+
+      if (ArgusParser->writeDbstr != NULL)
+         RaSQLSaveTable = strdup(RaTable);
    }
 
    if (!(ArgusParser->status & ARGUS_REAL_TIME_PROCESS))
       ArgusLastTime = ArgusParser->ArgusRealTime;
- 
-   if ((mysql_init(&mysql)) == NULL)
+
+   if (RaMySQL == NULL)
+      if ((RaMySQL = (void *) ArgusCalloc(1, sizeof(*RaMySQL))) == NULL)
+         ArgusLog(LOG_ERR, "RaMySQLInit: ArgusCalloc error %s", strerror(errno));
+
+   if ((mysql_init(RaMySQL)) == NULL)
       ArgusLog(LOG_ERR, "mysql_init error %s");
 
    if (!mysql_thread_safe())
       ArgusLog(LOG_INFO, "mysql not thread-safe");
 
-   mysql_options(&mysql, MYSQL_READ_DEFAULT_GROUP, ArgusParser->ArgusProgramName);
+   mysql_options(RaMySQL, MYSQL_READ_DEFAULT_GROUP, ArgusParser->ArgusProgramName);
+   mysql_options(RaMySQL, MYSQL_OPT_RECONNECT, reconnect);
 
-   if ((mysql_real_connect(&mysql, RaHost, RaUser, RaPass, NULL, RaPort, NULL, 0)) == NULL)
-      ArgusLog(LOG_ERR, "mysql_connect error %s", mysql_error(&mysql));
+   if ((mysql_real_connect(RaMySQL, RaHost, RaUser, RaPass, NULL, RaPort, NULL, 0)) == NULL)
+      ArgusLog(LOG_ERR, "mysql_connect error %s", mysql_error(RaMySQL));
 
-   RaMySQL = &mysql;
+   bzero(sbuf, sizeof(sbuf));
+   sprintf (sbuf, "SHOW VARIABLES LIKE 'version'");
+
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
+
+   if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
+      if ((retn = mysql_num_fields(mysqlRes)) > 0) {
+         while ((row = mysql_fetch_row(mysqlRes))) {
+            int matches = 0;
+            unsigned long *lengths;
+            lengths = mysql_fetch_lengths(mysqlRes);
+            sprintf(sbuf, "%.*s", (int) lengths[1], row[1] ? row[1] : "NULL");
+
+           ArgusSQLVersion = strdup(sbuf);
+           if ((matches = sscanf(ArgusSQLVersion,"%d.%d.%d", &MySQLVersionMajor, &MySQLVersionMinor, &MySQLVersionSub)) > 0) {
+            }
+         }
+      }
+      mysql_free_result(mysqlRes);
+   }
 
    bzero(sbuf, sizeof(sbuf));
    sprintf (sbuf, "SHOW VARIABLES LIKE 'bulk_insert_buffer_size'");
 
-   if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0)
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
-   if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+   if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
       if ((retn = mysql_num_fields(mysqlRes)) > 0) {
          while ((row = mysql_fetch_row(mysqlRes))) {
             unsigned long *lengths;
@@ -575,10 +583,10 @@ RaMySQLInit ()
    bzero(sbuf, sizeof(sbuf));
    sprintf (sbuf, "SHOW VARIABLES LIKE 'max_allowed_packet'");
 
-   if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0)
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
-   if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+   if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
       if ((retn = mysql_num_fields(mysqlRes)) > 0) {
          while ((row = mysql_fetch_row(mysqlRes))) {
             unsigned long *lengths;
@@ -599,15 +607,15 @@ RaMySQLInit ()
    bzero(sbuf, sizeof(sbuf));
    sprintf (sbuf, "CREATE DATABASE IF NOT EXISTS %s", RaDatabase);
 
-   if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0)  
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)  
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
    sprintf (sbuf, "USE %s", RaDatabase);
 
-   if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0)
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
-   if ((mysqlRes = mysql_list_tables(&mysql, NULL)) != NULL) {
+   if ((mysqlRes = mysql_list_tables(RaMySQL, NULL)) != NULL) {
       char sbuf[MAXSTRLEN];
 
       if ((retn = mysql_num_fields(mysqlRes)) > 0) {
@@ -665,6 +673,9 @@ RaMySQLInit ()
       }
    }
 
+   if (ArgusParser->MySQLDBEngine == NULL)
+      ArgusParser->MySQLDBEngine = strdup("MyISAM");
+
 #ifdef ARGUSDEBUG
    ArgusDebug (1, "RaMySQLInit () RaSource %s RaArchive %s RaFormat %s", RaSource, RaArchive, RaFormat);
 #endif
@@ -674,11 +685,10 @@ RaMySQLInit ()
 void
 RaSQLQueryTable (char **tables)
 {
-   char ArgusSQLStatement[1024];
-   char buf[0x10000], sbuf[0x10000], *table;
+   char ArgusSQLStatement[MAXSTRLEN];
+   char buf[MAXARGUSRECORD], sbuf[MAXARGUSRECORD], *table;
    MYSQL_RES *mysqlRes;
    struct timeval now;
-   int ArgusTimeSchema = 0;
    int retn, x, i;
 
    if ((ArgusInput = (struct ArgusInput *) ArgusCalloc (1, sizeof(struct ArgusInput))) == NULL)
@@ -727,12 +737,10 @@ RaSQLQueryTable (char **tables)
 
       if (timeField == NULL) 
          timeField = "second";
-      else
-         ArgusTimeSchema = 1;
 
       if (ArgusParser->ArgusSQLStatement != NULL) {
       } else {
-         snprintf (ArgusSQLStatement, 1024, "%s >= %d and %s < %d", timeField, (int)ArgusParser->startime_t.tv_sec, timeField, (int)ArgusParser->lasttime_t.tv_sec);
+         snprintf (ArgusSQLStatement, MAXSTRLEN, "%s >= %d and %s <= %d", timeField, (int)ArgusParser->startime_t.tv_sec, timeField, (int)ArgusParser->lasttime_t.tv_sec);
          ArgusParser->ArgusSQLStatement = strdup(ArgusSQLStatement);
       }
    }
@@ -753,8 +761,8 @@ RaSQLQueryTable (char **tables)
 #ifdef ARGUSDEBUG
          ArgusDebug (1, "SQL Query %s\n", buf);
 #endif
-         if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) == 0) {
-            if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+         if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) == 0) {
+            if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
                if ((retn = mysql_num_fields(mysqlRes)) > 0) {
                   while ((row = mysql_fetch_row(mysqlRes))) {
                      unsigned long *lengths = mysql_fetch_lengths(mysqlRes);
@@ -776,7 +784,7 @@ RaSQLQueryTable (char **tables)
                      if (((struct ArgusRecord *)sbuf)->hdr.type & ARGUS_MAR) {
                         bcopy ((char *) &sbuf, (char *)&ArgusInput->ArgusInitCon, sizeof (struct ArgusRecord));
                      } else {
-                        ArgusHandleDatum (ArgusParser, ArgusInput, (struct ArgusRecord *)&sbuf, &ArgusParser->ArgusFilterCode);
+                        ArgusHandleRecord (ArgusParser, ArgusInput, (struct ArgusRecord *)&sbuf, &ArgusParser->ArgusFilterCode);
                      }
                   }
                }
@@ -804,11 +812,11 @@ RaSQLQuerySecondsTable (unsigned int start, unsigned int stop)
    ArgusDebug (1, "SQL Query %s\n", buf);
 #endif
 
-   if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
    else {
-      if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+      if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
          if ((retn = mysql_num_fields(mysqlRes)) > 0) {
             while ((row = mysql_fetch_row(mysqlRes))) {
                unsigned long *lengths;
@@ -926,11 +934,11 @@ RaSQLProcessQueue (struct ArgusQueueStruct *queue)
             str = "SELECT filename from Filename WHERE id = %d",
             sprintf (buf, str, fstruct->fileindex);
 
-            if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-               ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+            if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+               ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
             else {
-               if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+               if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
                   if ((retn = mysql_num_fields(mysqlRes)) > 0) {
                      while ((row = mysql_fetch_row(mysqlRes))) {
                         char file[MAXSTRLEN];
@@ -1118,7 +1126,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
             break;
       }
 
-      if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL)) == NULL)
+      if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL, ARGUS_RECORD_AGGREGATOR)) == NULL)
          ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
 
       if ((ArgusModelerQueue = ArgusNewQueue()) == NULL)
@@ -1289,7 +1297,9 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                      if (ptr != endptr)
                         parser->ArgusTimeMultiplier = value;
                   }
-               }
+               } else
+               if (!(strncasecmp (mode->mode, "oui", 3)))
+                  parser->ArgusPrintEthernetVendors++;
             }
 
             mode = mode->nxt;
@@ -1341,14 +1351,14 @@ ArgusClientInit (struct ArgusParserStruct *parser)
       while (RaTables[tableIndex] != NULL) {
          if (strcmp("Seconds", RaTables[tableIndex])) {
             sprintf (ArgusSQLStatement, "desc %s", RaTables[tableIndex]);
-            if ((retn = mysql_real_query(&mysql, ArgusSQLStatement , strlen(ArgusSQLStatement))) == 0)
+            if ((retn = mysql_real_query(RaMySQL, ArgusSQLStatement , strlen(ArgusSQLStatement))) == 0)
                break;
          }
          tableIndex++;
       }
 
       if (retn == 0) {
-         if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+         if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
             if ((retn = mysql_num_fields(mysqlRes)) > 0) {
                int ind = 0;
                while ((row = mysql_fetch_row(mysqlRes)))
@@ -1506,7 +1516,7 @@ extern int RaDaysInAMonth[12];
 time_t ArgusTableStartSecs = 0;
 time_t ArgusTableEndSecs = 0;
 
-#define ARGUS_MAX_TABLE_LIST_SIZE	1024
+#define ARGUS_MAX_TABLE_LIST_SIZE	0x10000
 
 char **
 ArgusCreateSQLTimeTableNames (struct ArgusParserStruct *parser, char *table)
@@ -1748,10 +1758,7 @@ ArgusCreateSQLSaveTableName (struct ArgusParserStruct *parser, struct ArgusRecor
             ArgusLog (LOG_ERR, "RaSendArgusRecord () ArgusCalloc %s\n", strerror(errno));
 
          RaProcessSplitOptions(ArgusParser, ArgusSQLSaveTableNameBuf, MAXSTRLEN, ns);
-/* 
-         if (strcmp(wfile->filename, ArgusSQLSaveTableNameBuf))
-            ArgusInitNewFilename(ArgusParser, wfile, ArgusSQLSaveTableNameBuf);
-*/
+
          ArgusTableStartSecs = tableSecs;
 
          switch (nadp->qual) {
@@ -1772,17 +1779,7 @@ ArgusCreateSQLSaveTableName (struct ArgusParserStruct *parser, struct ArgusRecor
                break;
          }
       }
-/*
-      if (tableSecs > ArgusSaveTableSeconds) {
-         if (strftime(tmpbuf, 1024, table, localtime_r(&tSecs, &tmval)) <= 0)
-            ArgusLog (LOG_ERR, "RaSendArgusRecord () ArgusCalloc %s\n", strerror(errno));
 
-         if (ArgusSaveTableSeconds < tableSecs)
-            ArgusSaveTableSeconds = tableSecs;
-
-         RaProcessSplitOptions(parser, tmpbuf, 1024, ns);
-      }
-*/
       retn = ArgusSQLSaveTableNameBuf;
 
    } else {
@@ -1894,7 +1891,11 @@ ArgusCreateSQLSaveTable(char *table)
       if (ArgusSOptionRecord)
          sprintf (&sbuf[strlen(sbuf)], ", record blob");
 
-      sprintf (&sbuf[strlen(sbuf)], ") TYPE=MyISAM");
+      if ((MySQLVersionMajor > 4) || ((MySQLVersionMajor == 4) &&
+                                      (MySQLVersionMinor >= 1)))
+         sprintf (&sbuf[strlen(sbuf)], ") ENGINE=%s", ArgusParser->MySQLDBEngine);
+      else
+         sprintf (&sbuf[strlen(sbuf)], ") TYPE=%s", ArgusParser->MySQLDBEngine);
       RaTableCreateString[cindex] = strdup(sbuf);
 
       cindex++;
@@ -1906,8 +1907,8 @@ ArgusCreateSQLSaveTable(char *table)
 #ifdef ARGUSDEBUG
                ArgusDebug (2, "generating table %s\n", str);
 #endif
-               if ((retn = mysql_real_query(&mysql, str, strlen(str))) != 0)
-                  ArgusLog(LOG_INFO, "mysql_real_query error %s", mysql_error(&mysql));
+               if ((retn = mysql_real_query(RaMySQL, str, strlen(str))) != 0)
+                  ArgusLog(LOG_INFO, "mysql_real_query error %s", mysql_error(RaMySQL));
 
                ArgusCreateTable = 1;
                RaSQLCurrentTable = strdup(table);
