@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,9 +27,9 @@
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/examples/ramysql/rasqltimeindex.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/examples/ramysql/rasqltimeindex.c#16 $
+ * $DateTime: 2014/06/05 13:44:29 $
+ * $Change: 2845 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -80,6 +80,10 @@ int ArgusSQLBulkBufferSize = 0;
 int ArgusSQLBulkBufferIndex = 0;
 char *ArgusSQLBulkLastTable = NULL;
 char *ArgusSQLBulkBuffer = NULL;
+char *ArgusSQLVersion = NULL;
+int MySQLVersionMajor = 0;
+int MySQLVersionMinor = 0;
+int MySQLVersionSub = 0;
 
 extern int ArgusSOptionRecord;
 int ArgusDeleteTable = 0;
@@ -128,7 +132,7 @@ void RaMySQLInit (void);
 
 
 MYSQL_ROW row;
-MYSQL mysql, *RaMySQL = NULL;
+MYSQL *RaMySQL = NULL;
 MYSQL_RES *mysqlRes;
 
 int RaParseCompleteInd = 0;
@@ -141,7 +145,7 @@ int RaCheckedTables = 0;
 
 
 void RaArgusInputComplete (struct ArgusInput *input) {
-   int cnt, retn, fileindex = 0, filestatus = -1;
+   int cnt, retn, fileindex, filestatus;
    char buf[MAXSTRLEN], *endptr = NULL;
    char sbuf[MAXSTRLEN], *sptr = NULL;
    char *pathptr, filename[MAXSTRLEN];
@@ -163,10 +167,10 @@ void RaArgusInputComplete (struct ArgusInput *input) {
 
          do {
             sprintf (buf, RaTableQueryString[6], sptr);
-            if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0) 
-               ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+            if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0) 
+               ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
             else {
-               if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+               if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
                   if ((retn = mysql_num_fields(mysqlRes)) == 1) {
                      while ((row = mysql_fetch_row(mysqlRes))) {
                         if (row[0] != NULL) {
@@ -183,16 +187,16 @@ void RaArgusInputComplete (struct ArgusInput *input) {
                   if (ArgusParser->vflag)
                      ArgusLog(LOG_ALERT, "Probe %s not found: installing", sptr);
                   sprintf (buf, RaTableQueryString[7], sptr);
-                  if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-                     ArgusLog(LOG_ERR, "mysql_real_query error %s: %s", buf, mysql_error(&mysql));
+                  if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+                     ArgusLog(LOG_ERR, "mysql_real_query error %s: %s", buf, mysql_error(RaMySQL));
                }
             }
          } while (!found && !error++);
       } while ((probe = (struct RaTimeProbesStruct *) probe->qhdr.nxt) != (void *) ArgusProbes->queue->start);
 
-      fileindex = -1, filestatus = -1;
       bzero (buf, sizeof(buf));
       bzero (filename, sizeof(filename));
+      fileindex = -1, filestatus = -1;
 
      if ((pathptr = realpath(input->filename, filename)) != NULL) {
          free (input->filename);
@@ -200,13 +204,11 @@ void RaArgusInputComplete (struct ArgusInput *input) {
      }
 
       sprintf (buf, RaTableQueryString[0], filename);
-      if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-         ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
-      else {
-         fileindex = -1;
-         filestatus = -1;
+      if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+         ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
-         if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+      else {
+         if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
             if ((retn = mysql_num_fields(mysqlRes)) == 2) {
                while ((row = mysql_fetch_row(mysqlRes))) {
                   if (row[0] != NULL) {
@@ -227,21 +229,21 @@ void RaArgusInputComplete (struct ArgusInput *input) {
       }
 
       if (filestatus == 1)
-         ArgusLog(LOG_ERR, "file %s is locked", ArgusInput->filename);
+         ArgusLog(LOG_ERR, "file %s index %d is locked", ArgusInput->filename, fileindex);
 
       if (filestatus == 2) {
          if (ArgusParser->vflag)
-            ArgusLog(LOG_INFO, "file %s already processed", ArgusInput->filename);
+            ArgusLog(LOG_INFO, "file %s index %d has already been processed", ArgusInput->filename, fileindex);
          goto RaArgusInputCompleteDone;
       }
 
       if (filestatus == 0) {
          sprintf (buf, RaTableQueryString[4], fileindex);
 
-         if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-            ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+         if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+            ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
-         if (mysql_affected_rows(&mysql) == 0)
+         if (mysql_affected_rows(RaMySQL) == 0)
             ArgusLog(LOG_ERR, "file %s is locked", ArgusInput->filename);
 
       } else {
@@ -252,16 +254,16 @@ void RaArgusInputComplete (struct ArgusInput *input) {
          sprintf (buf, RaTableQueryString[1], filename, size, 
                        mtime, MDFile(ArgusInput->filename), RaStartTime, RaEndTime);
 
-         if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0) 
-            ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+         if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0) 
+            ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
       }
 
       sprintf (buf, RaTableQueryString[2], filename);
 
-      if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-         ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+      if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+         ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
       else {
-         if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+         if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
             if ((retn = mysql_num_fields(mysqlRes)) == 2) {
                while ((row = mysql_fetch_row(mysqlRes))) {
                   if (row[0] != NULL) {
@@ -284,7 +286,7 @@ void RaArgusInputComplete (struct ArgusInput *input) {
             mysql_free_result(mysqlRes);
 
          } else
-            ArgusLog(LOG_ERR, "mysql_store_result error %s", mysql_error(&mysql));
+            ArgusLog(LOG_ERR, "mysql_store_result error %s", mysql_error(RaMySQL));
       }
 
       if ((probe = (struct RaTimeProbesStruct *) ArgusProbes->queue->start) == NULL) 
@@ -305,8 +307,8 @@ void RaArgusInputComplete (struct ArgusInput *input) {
                struct RaTimeHashTableHeader *hdr = (struct RaTimeHashTableHeader *) tstore->object;
 
                if ((strlen(buf) + 128) > MAXSTRLEN) {
-                  if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-                     ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+                  if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+                     ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
                   sprintf (buf, "%s", RaTableQueryString[3]);
                }
@@ -319,13 +321,13 @@ void RaArgusInputComplete (struct ArgusInput *input) {
             }
 
             if (buf[strlen(buf) - 1] == ')')
-               if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-                  ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+               if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+                  ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
             sprintf (buf, RaTableQueryString[5], fileindex);
 
-            if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-               ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+            if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+               ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
          }
 
          bzero (buf, sizeof(buf));
@@ -391,14 +393,14 @@ RaParseComplete (int sig)
       ArgusShutDown(sig);
 
       if (!(RaParseCompleteInd++)) {
-      printf (" Totalrecords %-8Ld  TotalManRecords %-8Ld  TotalFarRecords %-8Ld TotalPkts %-8Ld TotalBytes %-8Ld\n",
+      printf (" Totalrecords %-8lld  TotalManRecords %-8lld  TotalFarRecords %-8lld TotalPkts %-8lld TotalBytes %-8lld\n",
                        ArgusParser->ArgusTotalRecords,
                        ArgusParser->ArgusTotalMarRecords, ArgusParser->ArgusTotalFarRecords,
                        ArgusParser->ArgusTotalPkts, ArgusParser->ArgusTotalBytes);
       }
    }
    fflush(stdout);
-   mysql_close(&mysql);
+   mysql_close(RaMySQL);
 
    if (sig == SIGINT)
       exit(0);
@@ -533,10 +535,269 @@ void ArgusWindowClose(void) {
 #endif
 }
 
+/*
+void
+RaMySQLInit ()
+{
+   my_bool reconnectbuf = 1, *reconnect = &reconnectbuf;
+   char *sptr = NULL, *ptr;
+   char userbuf[1024], sbuf[1024], db[1024], *dbptr = NULL;
+   MYSQL_RES *mysqlRes;
+   int retn = 0;
+
+   bzero((char *)RaTableExistsNames,  sizeof(RaTableExistsNames));
+   bzero((char *)RaTableCreateNames,  sizeof(RaTableCreateNames));
+   bzero((char *)RaTableCreateString, sizeof(RaTableCreateString));
+   bzero((char *)RaTableDeleteString, sizeof(RaTableDeleteString));
+
+   if ((RaUser == NULL) && (ArgusParser->dbuserstr != NULL)) {
+      bzero(userbuf, sizeof(userbuf));
+      strncpy (userbuf, ArgusParser->dbuserstr, sizeof(userbuf));
+      if ((sptr = strchr (userbuf, ':')) != NULL) {
+         *sptr++ = '\0';
+         RaPass = strdup(sptr);
+      }
+      RaUser = strdup(userbuf);
+   }
+
+   if ((RaPass == NULL) && (ArgusParser->dbpassstr != NULL))
+      RaPass = ArgusParser->dbpassstr;
+
+   if (RaDatabase == NULL) {
+      if (ArgusParser->writeDbstr != NULL)
+         RaDatabase = strdup(ArgusParser->writeDbstr);
+
+      else if (ArgusParser->readDbstr != NULL)
+         RaDatabase = strdup(ArgusParser->readDbstr);
+
+      if (!(strncmp("mysql:", RaDatabase, 6))) {
+         char *tmp = RaDatabase;
+         RaDatabase = strdup(&RaDatabase[6]);
+         free(tmp);
+      }
+   }
+
+   if (RaDatabase == NULL) {
+      ArgusLog(LOG_ERR, "must specify database"); 
+
+   } else {
+      sprintf(db, "%s", RaDatabase);
+      dbptr = db;
+
+//    //[[username[:password]@]hostname[:port]]/database/tablename
+
+      if (!(strncmp ("//", dbptr, 2))) {
+         char *rhost = NULL, *ruser = NULL, *rpass = NULL;
+         if ((strncmp ("///", dbptr, 3))) {
+            dbptr = &dbptr[2];
+            rhost = dbptr;
+            if ((ptr = strchr (dbptr, '/')) != NULL) {
+               *ptr++ = '\0';
+               dbptr = ptr;
+
+               if ((ptr = strchr (rhost, '@')) != NULL) {
+                  ruser = rhost;
+                  *ptr++ = '\0';
+                  rhost = ptr;
+                  if ((ptr = strchr (ruser, ':')) != NULL) {
+                     *ptr++ = '\0';
+                     rpass = ptr;
+                  } else {
+                     rpass = NULL;
+                  }
+               }
+
+               if ((ptr = strchr (rhost, ':')) != NULL) {
+                  *ptr++ = '\0';
+                  RaPort = atoi(ptr);
+               }
+            } else
+               dbptr = NULL;
+
+         } else {
+            dbptr = &dbptr[3];
+         }
+
+         if (ruser != NULL) {
+            if (RaUser != NULL) free(RaUser);
+            RaUser = strdup(ruser);
+         }
+         if (rpass != NULL) {
+            if (RaPass != NULL) free(RaPass);
+            RaPass = strdup(rpass);
+         }
+         if (rhost != NULL) {
+            if (RaHost != NULL) free(RaHost);
+            RaHost = strdup(rhost);
+         }
+         free(RaDatabase);
+         RaDatabase = strdup(dbptr);
+      }
+   }
+ 
+   if ((ptr = strchr (RaDatabase, '/')) != NULL) {
+      *ptr++ = '\0';
+      RaTable = ptr;
+
+      if (ArgusParser->writeDbstr != NULL)
+         RaSQLSaveTable = strdup(RaTable);
+   }
+
+   if (!(ArgusParser->status & ARGUS_REAL_TIME_PROCESS))
+      ArgusLastTime = ArgusParser->ArgusRealTime;
+
+   if (RaMySQL == NULL)
+      if ((RaMySQL = (void *) ArgusCalloc(1, sizeof(*RaMySQL))) == NULL)
+         ArgusLog(LOG_ERR, "RaMySQLInit: ArgusCalloc error %s", strerror(errno));
+ 
+   if ((mysql_init(RaMySQL)) == NULL)
+      ArgusLog(LOG_ERR, "mysql_init error %s");
+
+   if (!mysql_thread_safe())
+      ArgusLog(LOG_INFO, "mysql not thread-safe");
+
+   mysql_options(RaMySQL, MYSQL_READ_DEFAULT_GROUP, ArgusParser->ArgusProgramName);
+   mysql_options(RaMySQL, MYSQL_OPT_RECONNECT, reconnect);
+
+   if ((mysql_real_connect(RaMySQL, RaHost, RaUser, RaPass, NULL, RaPort, NULL, 0)) == NULL)
+      ArgusLog(LOG_ERR, "mysql_connect error %s", mysql_error(RaMySQL));
+
+   bzero(sbuf, sizeof(sbuf));
+   sprintf (sbuf, "SHOW VARIABLES LIKE 'version'");
+
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
+
+   if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
+      if ((retn = mysql_num_fields(mysqlRes)) > 0) {
+         while ((row = mysql_fetch_row(mysqlRes))) {
+            int matches = 0;
+            unsigned long *lengths;
+            lengths = mysql_fetch_lengths(mysqlRes);
+            sprintf(sbuf, "%.*s", (int) lengths[1], row[1] ? row[1] : "NULL");
+
+           ArgusSQLVersion = strdup(sbuf);
+           if ((matches = sscanf(ArgusSQLVersion,"%d.%d.%d", &MySQLVersionMajor, &MySQLVersionMinor, &MySQLVersionSub)) > 0) {
+            }
+         }
+      }
+      mysql_free_result(mysqlRes);
+   }
+
+   bzero(sbuf, sizeof(sbuf));
+   sprintf (sbuf, "SHOW VARIABLES LIKE 'bulk_insert_buffer_size'");
+
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
+
+   if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
+      if ((retn = mysql_num_fields(mysqlRes)) > 0) {
+         while ((row = mysql_fetch_row(mysqlRes))) {
+            unsigned long *lengths;
+            lengths = mysql_fetch_lengths(mysqlRes);
+            sprintf(sbuf, "%.*s", (int) lengths[1], row[1] ? row[1] : "NULL");
+
+           ArgusSQLBulkBufferSize = (int)strtol(sbuf, (char **)NULL, 10);
+         }
+      }
+      mysql_free_result(mysqlRes);
+   }
+
+   bzero(sbuf, sizeof(sbuf));
+   sprintf (sbuf, "SHOW VARIABLES LIKE 'max_allowed_packet'");
+
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
+
+   if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
+      if ((retn = mysql_num_fields(mysqlRes)) > 0) {
+         while ((row = mysql_fetch_row(mysqlRes))) {
+            unsigned long *lengths;
+            lengths = mysql_fetch_lengths(mysqlRes);
+            sprintf(sbuf, "%.*s", (int) lengths[1], row[1] ? row[1] : "NULL");
+            
+           ArgusSQLMaxPacketSize = (int)strtol(sbuf, (char **)NULL, 10);
+         }
+      }
+      mysql_free_result(mysqlRes);
+   }
+
+   ArgusSQLBulkInsertSize = (ArgusSQLMaxPacketSize < ArgusSQLBulkBufferSize) ? ArgusSQLMaxPacketSize : ArgusSQLBulkBufferSize;
+
+   if ((ArgusSQLBulkBuffer = calloc(1, ArgusSQLBulkInsertSize)) == NULL)
+      ArgusLog(LOG_WARNING, "ArgusMySQLInit: cannot alloc bulk buffer size %d\n", ArgusSQLBulkInsertSize);
+
+   bzero(sbuf, sizeof(sbuf));
+   sprintf (sbuf, "CREATE DATABASE IF NOT EXISTS %s", RaDatabase);
+
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)  
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
+
+   sprintf (sbuf, "USE %s", RaDatabase);
+
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
+
+   if ((mysqlRes = mysql_list_tables(RaMySQL, NULL)) != NULL) {
+      char sbuf[MAXSTRLEN];
+
+      if ((retn = mysql_num_fields(mysqlRes)) > 0) {
+         int x, thisIndex = 0;
+
+         while ((row = mysql_fetch_row(mysqlRes))) {
+            unsigned long *lengths;
+            lengths = mysql_fetch_lengths(mysqlRes);
+            bzero(sbuf, sizeof(sbuf));
+            for (x = 0; x < retn; x++)
+               sprintf(&sbuf[strlen(sbuf)], "%.*s", (int) lengths[x], row[x] ? row[x] : "NULL");
+
+            RaTableExistsNames[thisIndex++] = strdup (sbuf);
+         }
+
+      } else {
+#ifdef ARGUSDEBUG
+         ArgusDebug (2, "mysql_num_fields() returned zero.\n");
+#endif
+      }
+      mysql_free_result(mysqlRes);
+   }
+
+   if (ArgusParser->writeDbstr != NULL) {
+      char *ptr;
+      sprintf (ArgusParser->RaDBString, "-w %s", ArgusParser->writeDbstr);
+      if ((ptr = strrchr(ArgusParser->writeDbstr, '/')) != NULL)
+         *ptr = '\0';
+
+   } else 
+   if (ArgusParser->readDbstr != NULL) {
+      char *ptr;
+      sprintf (ArgusParser->RaDBString, "-r %s", ArgusParser->readDbstr);
+      if ((ptr = strrchr(ArgusParser->readDbstr, '/')) != NULL)
+         *ptr = '\0';
+   } else  {
+      sprintf (ArgusParser->RaDBString, "db %s", RaDatabase);
+
+      if (RaHost)
+         sprintf (&ArgusParser->RaDBString[strlen(ArgusParser->RaDBString)], "@%s", RaHost);
+
+      sprintf (&ArgusParser->RaDBString[strlen(ArgusParser->RaDBString)], " user %s", RaUser);
+   }
+
+   if (ArgusParser->MySQLDBEngine == NULL)
+      ArgusParser->MySQLDBEngine = strdup("MyISAM");
+
+      RaCheckedTables = 1;
+
+#ifdef ARGUSDEBUG
+   ArgusDebug (1, "RaMySQLInit ()");
+#endif
+}
+*/
 
 void
 RaMySQLInit ()
 {
+   my_bool reconnectbuf = 1, *reconnect = &reconnectbuf;
    unsigned int RaTableFlags = 0;
    int retn = 0, x;
    char *sptr = NULL, *ptr;
@@ -574,9 +835,7 @@ RaMySQLInit ()
    if (RaDatabase == NULL)
       ArgusLog(LOG_ERR, "must specify database, use '-r mysql://user@host/db' or specify in .rarc");
 
-/*
-      //[[username[:password]@]hostname[:port]]/database/tablename
-*/
+//    //[[username[:password]@]hostname[:port]]/database/tablename
 
    if (!(strncmp ("//", RaDatabase, 2))) {
       if ((strncmp ("///", RaDatabase, 3))) {
@@ -621,26 +880,29 @@ RaMySQLInit ()
    if (!(ArgusParser->status & ARGUS_REAL_TIME_PROCESS))
       ArgusLastTime = ArgusParser->ArgusRealTime;
 
-   if ((mysql_init(&mysql)) == NULL)
+   if (RaMySQL == NULL)
+      if ((RaMySQL = (void *) ArgusCalloc(1, sizeof(*RaMySQL))) == NULL)
+         ArgusLog(LOG_ERR, "RaMySQLInit: ArgusCalloc error %s", strerror(errno));
+
+   if ((mysql_init(RaMySQL)) == NULL)
       ArgusLog(LOG_ERR, "mysql_init error %s");
 
    if (!mysql_thread_safe())
       ArgusLog(LOG_INFO, "mysql not thread-safe");
 
-   mysql_options(&mysql, MYSQL_READ_DEFAULT_GROUP, ArgusParser->ArgusProgramName);
+   mysql_options(RaMySQL, MYSQL_READ_DEFAULT_GROUP, ArgusParser->ArgusProgramName);
+   mysql_options(RaMySQL, MYSQL_OPT_RECONNECT, reconnect);
 
-   if ((mysql_real_connect(&mysql, RaHost, RaUser, RaPass, NULL, RaPort, NULL, 0)) == NULL)
-      ArgusLog(LOG_ERR, "mysql_connect error %s", mysql_error(&mysql));
-
-   RaMySQL = &mysql;
+   if ((mysql_real_connect(RaMySQL, RaHost, RaUser, RaPass, NULL, RaPort, NULL, 0)) == NULL)
+      ArgusLog(LOG_ERR, "mysql_connect error %s", mysql_error(RaMySQL));
 
    bzero(sbuf, sizeof(sbuf));
    sprintf (sbuf, "SHOW VARIABLES LIKE 'bulk_insert_buffer_size'");
 
-   if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0)
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
-   if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+   if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
       if ((retn = mysql_num_fields(mysqlRes)) > 0) {
          while ((row = mysql_fetch_row(mysqlRes))) {
             unsigned long *lengths;
@@ -656,10 +918,10 @@ RaMySQLInit ()
    bzero(sbuf, sizeof(sbuf));
    sprintf (sbuf, "SHOW VARIABLES LIKE 'max_allowed_packet'");
 
-   if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0)
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
-   if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+   if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
       if ((retn = mysql_num_fields(mysqlRes)) > 0) {
          while ((row = mysql_fetch_row(mysqlRes))) {
             unsigned long *lengths;
@@ -680,15 +942,15 @@ RaMySQLInit ()
    bzero(sbuf, sizeof(sbuf));
    sprintf (sbuf, "CREATE DATABASE IF NOT EXISTS %s", RaDatabase);
 
-   if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0)  
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)  
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
    sprintf (sbuf, "USE %s", RaDatabase);
 
-   if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0)
-      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+   if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0)
+      ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
 
-   if ((mysqlRes = mysql_list_tables(&mysql, NULL)) != NULL) {
+   if ((mysqlRes = mysql_list_tables(RaMySQL, NULL)) != NULL) {
       char sbuf[MAXSTRLEN];
 
       if ((retn = mysql_num_fields(mysqlRes)) > 0) {
@@ -734,7 +996,7 @@ RaMySQLInit ()
    }
 
    if (!RaCheckedTables) {
-      if ((mysqlRes = mysql_list_tables(&mysql, NULL)) != NULL) {
+      if ((mysqlRes = mysql_list_tables(RaMySQL, NULL)) != NULL) {
          if ((retn = mysql_num_fields(mysqlRes)) > 0) {
             char sbuf[MAXSTRLEN];
             int thisIndex = 0;
@@ -791,8 +1053,8 @@ RaMySQLInit ()
                   char sbuf[1024];
                   bzero (sbuf, 1024);
                   sprintf (sbuf, "%s", RaTableCreationString[i]);
-                  if ((retn = mysql_real_query(&mysql, sbuf, strlen(sbuf))) != 0) {
-                     ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+                  if ((retn = mysql_real_query(RaMySQL, sbuf, strlen(sbuf))) != 0) {
+                     ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
                   }
                }
             }
@@ -888,7 +1150,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
 
       if ((RaHost != NULL) && strcmp(RaHost, "localhost")) {
          if (ArgusParser->ais == NULL)
-            ArgusParser->ais = "/tmp/archive";
+            ArgusParser->ais = strdup("/tmp/archive");
          sprintf (RaLocalArchBuf, "%s/%s/%s", ArgusParser->ais, RaArchive, RaHost);
          RaLocalArchive = RaLocalArchBuf;
       } else {
@@ -957,10 +1219,10 @@ ArgusClientInit (struct ArgusParserStruct *parser)
             }
 
             sprintf (buf, RaTableQueryString[0], input->filename);
-            if ((retn = mysql_real_query(&mysql, buf, strlen(buf))) != 0)
-               ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(&mysql));
+            if ((retn = mysql_real_query(RaMySQL, buf, strlen(buf))) != 0)
+               ArgusLog(LOG_ERR, "mysql_real_query error %s", mysql_error(RaMySQL));
             else {
-               if ((mysqlRes = mysql_store_result(&mysql)) != NULL) {
+               if ((mysqlRes = mysql_store_result(RaMySQL)) != NULL) {
                   if ((retn = mysql_num_fields(mysqlRes)) == 2) {
                      while ((row = mysql_fetch_row(mysqlRes))) {
                         if (row[0] != NULL) {
@@ -990,7 +1252,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                   parser->ArgusInputFileList = input;
 
             } else {
-               ArgusLog(LOG_INFO, "file %s already processed", input->filename);
+               ArgusLog(LOG_INFO, "file %s index %d has already been processed", input->filename, fileindex);
                if (input->filename != NULL)
                   free (input->filename);
                ArgusFree(input);
