@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,9 +34,9 @@
  * written by Carter Bullard
  * QoSient, LLC
  *
- * $Id: //depot/argus-3.0.6/clients/examples/rapath/rapath.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/examples/rapath/rapath.c#13 $
+ * $DateTime: 2014/05/13 17:38:34 $
+ * $Change: 2824 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -68,9 +68,10 @@
 int RaInitialized = 0;
 int RaPrintThinkMapOutput = 0;
 int RaPrintSVGOutput = 0;
-int RaPrintASpath = 0;
-int RaPrintNodePath = 1;
+int RaPrintASNum = 0;
+int RaPrintNode = 0;
 int RaPrintAddr = 0;
+int RaPrintTreeNode = 0;
 int RaPrintDistance = 0;
 
 extern int RaHistoStart;
@@ -154,22 +155,31 @@ ArgusClientInit (struct ArgusParserStruct *parser)
                RaPrintSVGOutput++;
             if (!(strcasecmp (mode->mode, "addr"))) {
                RaPrintAddr = 1;
+               RaPrintNode = 0;
             }
+            if (!(strcasecmp (mode->mode, "tree")))
+               RaPrintTreeNode = 1;
+
             if (!(strcasecmp (mode->mode, "node"))) {
                RaPrintAddr = 0;
+               RaPrintNode = 1;
             }
             if (!(strcasecmp (mode->mode, "aspath"))) {
-               RaPrintASpath++;
-               RaPrintNodePath = 0;
+               RaPrintASNum++;
+            }
+            if (!(strcasecmp (mode->mode, "asnode"))) {
+               RaPrintASNum++;
+               RaPrintNode++;
             }
-            if (!(strcasecmp (mode->mode, "asnode")))
-               RaPrintNodePath = 0;
             if (!(strcasecmp (mode->mode, "dist")))
                RaPrintDistance = 1;
             mode = mode->nxt;
          }
       }
 
+      if (!(RaPrintAddr) && !(RaPrintNode) && !(RaPrintASNum))
+         RaPrintNode = 1;
+
       if (ArgusParser->RaPrintOptionStrings[0] == NULL) {
          parser->RaPrintOptionIndex = 0;
          parser->RaPrintOptionStrings[parser->RaPrintOptionIndex++] = strdup("srcid");
@@ -178,7 +188,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
          parser->RaPrintOptionStrings[parser->RaPrintOptionIndex++] = strdup("daddr");
          parser->RaPrintOptionStrings[parser->RaPrintOptionIndex++] = strdup("inode");
 
-         if (!RaPrintNodePath)
+         if (RaPrintASNum)
             parser->RaPrintOptionStrings[parser->RaPrintOptionIndex++] = strdup("ias:8");
 
          parser->RaPrintOptionStrings[parser->RaPrintOptionIndex++] = strdup("sttl");
@@ -195,14 +205,15 @@ ArgusClientInit (struct ArgusParserStruct *parser)
             ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewLabeler error");
 
          RaLabelParseResourceFile (parser, parser->ArgusLabeler, parser->ArgusFlowModelFile);
+         free(parser->ArgusFlowModelFile);
          parser->ArgusFlowModelFile = NULL;
       }
 
       if ((ArgusFlowAggregator = ArgusParseAggregator(parser, NULL, RaPathAggregationConfig)) == NULL)
-         ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
+         ArgusLog (LOG_ERR, "ArgusClientInit: ArgusParseAggregator error");
 
       if (parser->ArgusMaskList != NULL) {
-         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL)) == NULL)
+         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL, ARGUS_RECORD_AGGREGATOR)) == NULL)
             ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
 
 // have to see if we need to modify the model definition in our RaPathAggregation Config
@@ -219,7 +230,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
 
       } else {
          if ((parser->ArgusAggregator = ArgusParseAggregator(parser, NULL, RaMatrixAggregationConfig)) == NULL)
-            ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
+            ArgusLog (LOG_ERR, "ArgusClientInit: ArgusParseAggregator error");
       }
 
       if ((ArgusSorter = ArgusNewSorter(parser)) == NULL)
@@ -271,15 +282,19 @@ ArgusClientTimeout ()
 }
 
 
-struct RaPathTreeNode {
-   struct RaPathTreeNode *nxt;
+struct RaPathNode {
+   struct RaPathNode *nxt;
    struct ArgusQueueStruct *nodes;
    int as, ttl;
 };
 
+struct RaPathTree {
+};
 
-struct RaPathTreeNode *RaPathBuildTree (struct RaPathTreeNode *, struct ArgusQueueStruct *);
-void   RaPrintTree (struct RaPathTreeNode *);
+
+struct RaPathNode *RaPathBuildPath (struct ArgusQueueStruct *);
+void RaPathDeletePath (struct RaPathNode *);
+void RaPrintPath (struct RaPathNode *, char *, int);
 
 /*
    The idea is to build the path tree for a given queue.  The queue
@@ -291,17 +306,14 @@ void   RaPrintTree (struct RaPathTreeNode *);
 
 */
 
-char nodeChar = 'A';
-char RaTreeBuffer[MAXSTRLEN];
  
-void RaPathInsertTree (struct RaPathTreeNode *, struct RaPathTreeNode *);
-void RaPrintTreeNodes (struct RaPathTreeNode *, int);
+void RaPathInsertTree (struct RaPathTree *, struct RaPathNode *);
+void RaPrintPathNodes (struct RaPathNode *, int, char *buf, int);
 
-
-struct RaPathTreeNode *
-RaPathBuildTree (struct RaPathTreeNode *tree, struct ArgusQueueStruct *queue)
+struct RaPathNode *
+RaPathBuildPath (struct ArgusQueueStruct *queue)
 {
-   struct RaPathTreeNode *path = NULL, *node = NULL;
+   struct RaPathNode *path = NULL, *node = NULL;
    struct ArgusRecordStruct *argus;
    unsigned int pttl, tttl, as;
 
@@ -313,7 +325,7 @@ RaPathBuildTree (struct RaPathTreeNode *tree, struct ArgusQueueStruct *queue)
          tttl = attr->src.ttl;
 
          if (path == NULL) {
-            if ((path = (struct RaPathTreeNode *) ArgusCalloc (1, sizeof (*node))) == NULL)
+            if ((path = (struct RaPathNode *) ArgusCalloc (1, sizeof (*node))) == NULL)
                ArgusLog (LOG_ERR, "ArgusCalloc error %s", strerror(errno));
 
             node = path;
@@ -342,9 +354,9 @@ RaPathBuildTree (struct RaPathTreeNode *tree, struct ArgusQueueStruct *queue)
                      if (node->as != asn->inode_as)
                         node->as = -1;
                } else {
-                  struct RaPathTreeNode *prv = node;
+                  struct RaPathNode *prv = node;
 
-                  if ((node = (struct RaPathTreeNode *) ArgusCalloc (1, sizeof (*node))) == NULL)
+                  if ((node = (struct RaPathNode *) ArgusCalloc (1, sizeof (*node))) == NULL)
                      ArgusLog (LOG_ERR, "ArgusCalloc error %s", strerror(errno));
 
                   node->ttl = tttl;
@@ -365,31 +377,44 @@ RaPathBuildTree (struct RaPathTreeNode *tree, struct ArgusQueueStruct *queue)
       }
    }
  
-   if (tree == NULL)
-      tree = path;
-   else 
-      RaPathInsertTree (tree, path);
-
-   return (tree);
-}
-
-
-
-void
-RaPathInsertTree (struct RaPathTreeNode *tree, struct RaPathTreeNode *path)
-{
+   return (path);
 }
 
 void
-RaPrintTreeNodes (struct RaPathTreeNode *tree, int level)
+RaPathDeletePath (struct RaPathNode *path)
 {
-   struct RaPathTreeNode *path = tree;
+   if (path->nxt != NULL)
+      RaPathDeletePath(path->nxt);
+
+   if (path->nodes != NULL) {
+      ArgusDeleteQueue(path->nodes);
+   }
+   ArgusFree(path);
+
+   return;
+}
+
+
+void
+RaPathInsertTree (struct RaPathTree *tree, struct RaPathNode *path)
+{
+}
+
+// "as"       AS1 -> AS2 -> AS3
+// "node"     A -> B -> {C, D} -> F
+// "node"     [A -> B] -> [{C, D} -> F] -> [G -> H]
+// "asnode"   AS30496:[A -> B] -> AS6079:[C -> {D,E}] -> AS1257:[F] -> AS11164:[G -> H] -> AS5050:[I] -> AS9:[J -> {K,L}]
+
+void
+RaPrintPathNodes (struct RaPathNode *tree, int level, char *buf, int len)
+{
+   struct RaPathNode *path = tree;
    unsigned short as = 0;
    int status = 0, shop = 0;
 
    while (path != NULL) {
       struct ArgusQueueStruct *queue = path->nodes;
-      struct ArgusRecordStruct *argus;
+      struct ArgusRecordStruct *argus = NULL;
       int hopcount, multias = 0;
 
       if (status == 0) {
@@ -398,16 +423,16 @@ RaPrintTreeNodes (struct RaPathTreeNode *tree, int level)
                multias = 1;
             } else {
                as = path->as;
-               if (!RaPrintNodePath)
-                  printf ("AS%d", as);
-               if ((!RaPrintNodePath) && (!RaPrintASpath))
-                  printf (":");
-               if (!RaPrintASpath)
-                  printf ("[");
-               else {
-                  if (RaPrintDistance)
-                     printf (":%d", path->ttl);
-               }
+
+               if (RaPrintASNum) {
+                  sprintf (&buf[strlen(buf)], "AS%d", as);
+
+                  if (RaPrintNode || RaPrintAddr)
+                     sprintf (&buf[strlen(buf)], ":[");
+
+               } else 
+                  sprintf (&buf[strlen(buf)], "[");
+
                status++;
             }
          }
@@ -416,107 +441,120 @@ RaPrintTreeNodes (struct RaPathTreeNode *tree, int level)
       }
 
       if (queue->count > 1) {
-         if (!RaPrintASpath || (RaPrintASpath && !status)) {
-            printf ("{");
+         if (RaPrintNode || RaPrintAddr) {
+            sprintf (&buf[strlen(buf)], "{");
             while ((argus = (struct ArgusRecordStruct *) ArgusPopQueue(queue, ARGUS_NOLOCK)) != NULL) {
                if (multias) {
                   struct ArgusAsnStruct *asn = (void *)argus->dsrs[ARGUS_ASN_INDEX];
-                  if ((asn != NULL) && (asn->hdr.argus_dsrvl8.len > 2))
-                     if (!RaPrintNodePath)
-                        printf ("AS%d", asn->inode_as);
-                     if (!RaPrintASpath)
-                        printf (":[");
-               }
-               if (!RaPrintASpath) {
-                  if (RaPrintAddr) {
-                     char inodeStr[256], *inodePtr = inodeStr;
-                     ArgusPrintInode (ArgusParser, inodeStr, argus, 256);
-                     while (*inodePtr == ' ') inodePtr++;
-                     while (inodePtr[strlen(inodePtr) - 1] == ' ') inodePtr[strlen(inodePtr) - 1] = '\0';
-                     printf ("%s", inodePtr);
-                  } else
-                     printf ("%c", nodeChar);
+                  if ((asn != NULL) && (asn->hdr.argus_dsrvl8.len > 2)) {
+                     if (RaPrintASNum) {
+                        if (asn->inode_as > 0) {
+                          sprintf (&buf[strlen(buf)], "AS%d", asn->inode_as);
+                        } else {
+                           sprintf (&buf[strlen(buf)], "%c", argus->autoid);
+                        }
+                     } else
+                        sprintf (&buf[strlen(buf)], ":[");
+                  }
                }
 
+               if (RaPrintAddr) {
+                  char inodeStr[256], *inodePtr = inodeStr;
+                  ArgusPrintInode (ArgusParser, inodeStr, argus, 256);
+                  while (*inodePtr == ' ') inodePtr++;
+                  while (inodePtr[strlen(inodePtr) - 1] == ' ') inodePtr[strlen(inodePtr) - 1] = '\0';
+                  sprintf (&buf[strlen(buf)], "%s", inodePtr);
+               } else
+                  sprintf (&buf[strlen(buf)], "%c", argus->autoid);
+
                if (multias) {
-                  if (!RaPrintASpath)
-                     printf ("]");
+                  sprintf (&buf[strlen(buf)], "]");
                }
 
                if (queue->count > 0)
-                  printf (",");
-               nodeChar++;
+                  sprintf (&buf[strlen(buf)], ", ");
+
+               ArgusAddToQueue(ArgusModelerQueue, &argus->qhdr, ARGUS_LOCK);
             }
-            printf ("}");
-            if (RaPrintASpath)
-               if (RaPrintDistance)
-                  printf (":%d", path->ttl);
+
+            sprintf (&buf[strlen(buf)], "}");
+            if (RaPrintDistance) {
+               sprintf (&buf[strlen(buf)], ":%d", path->ttl);
+               shop = path->ttl;
+            }
+
+            while ((argus = (struct ArgusRecordStruct *) ArgusPopQueue(ArgusModelerQueue, ARGUS_LOCK)) != NULL)
+               ArgusAddToQueue(queue, &argus->qhdr, ARGUS_NOLOCK);
          }
 
       } else {
-         argus = (struct ArgusRecordStruct *) ArgusPopQueue(queue, ARGUS_NOLOCK);
-         if (!(RaPrintASpath && status)) {
-            if (RaPrintAddr) {
-               char inodeStr[256], *inodePtr = inodeStr;
-               ArgusPrintInode (ArgusParser, inodeStr, argus, 256);
-               while (*inodePtr == ' ') inodePtr++;
-               while (inodePtr[strlen(inodePtr) - 1] == ' ') inodePtr[strlen(inodePtr) - 1] = '\0';
-               printf ("%s", inodePtr);
-            } else
-               printf ("%c", nodeChar++);
+         argus = (struct ArgusRecordStruct *) queue->start;
+         
+         if (RaPrintAddr) {
+            char inodeStr[256], *inodePtr = inodeStr;
+            ArgusPrintInode (ArgusParser, inodeStr, argus, 256);
+            while (*inodePtr == ' ') inodePtr++;
+            while (inodePtr[strlen(inodePtr) - 1] == ' ') inodePtr[strlen(inodePtr) - 1] = '\0';
+            sprintf (&buf[strlen(buf)], "%s", inodePtr);
 
-            if (RaPrintASpath)
-               if (RaPrintDistance)
-                  printf (":%d", path->ttl);
+         } else 
+         if (RaPrintNode) {
+            sprintf (&buf[strlen(buf)], "%c", argus->autoid);
+         }
+
+         if (RaPrintDistance) {
+            sprintf (&buf[strlen(buf)], ":%d", path->ttl);
+            shop = path->ttl;
          }
       }
 
       if ((path = path->nxt) != NULL) {
          if (status) {
             if (path->as != as) {
-               if (!RaPrintASpath)
-                  printf ("]");
-               else {
-                  if (RaPrintDistance) {
-                     if (hopcount > 0) {
-                        printf ("-%d", shop + hopcount);
-                        hopcount = 0;
-                     }
+               if (RaPrintNode || RaPrintAddr)
+                  sprintf (&buf[strlen(buf)], "]");
+
+               if (RaPrintDistance) {
+                  if ((hopcount - shop) > 0) {
+                     sprintf (&buf[strlen(buf)], ":%d-%d", shop, hopcount);
+                     hopcount = 0;
                   }
-                  printf (" -> ");
                }
+
+               sprintf (&buf[strlen(buf)], " -> ");
                status--;
                as = 0;
-            } else
-               hopcount++;
 
-         } else 
-            if (RaPrintASpath)
-               printf (" -> ");
-         
-         if (!RaPrintASpath)
-            printf (" -> ");
-         
+            } else {
+               if (RaPrintNode || RaPrintAddr)
+                  sprintf (&buf[strlen(buf)], " -> ");
+               hopcount = path->ttl;
+            }
+
+         } else
+            if (RaPrintNode || RaPrintAddr)
+               sprintf (&buf[strlen(buf)], " -> ");
+
       } else {
          if (status) {
-            if (RaPrintASpath) {
-               if (RaPrintDistance) {
-                  if (hopcount > 0)
-                     printf ("-%d ", shop + hopcount);
-               }
-            } else
-               printf ("]");
+            if (RaPrintDistance) {
+               if ((hopcount - shop) > 0)
+                  sprintf (&buf[strlen(buf)], ":%d-%d", shop, hopcount);
+            }
+
+            if (RaPrintNode || RaPrintAddr)
+               sprintf (&buf[strlen(buf)], "]");
          }
-         printf ("\n");
       }
    }
 }
 
+
 void
-RaPrintTree (struct RaPathTreeNode *tree)
+RaPrintPath (struct RaPathNode *tree, char *buf, int len)
 {
-   bzero (RaTreeBuffer, MAXSTRLEN);
-   RaPrintTreeNodes(tree, 0);
+   bzero (buf, len);
+   RaPrintPathNodes(tree, 0, buf, len);
 }
 
 
@@ -524,6 +562,7 @@ void RaArgusInputComplete (struct ArgusInput *input) { return; }
 
 
 int RaParseCompleting = 0;
+struct RaPathTree *RaTreeNode = NULL;
 
 void
 RaParseComplete (int sig)
@@ -539,7 +578,7 @@ RaParseComplete (int sig)
  
          if (agg != NULL) {
             if (agg->queue->count) {
-               struct ArgusRecordStruct *argus, *tns, *cns;
+               struct ArgusRecordStruct *tns = NULL, *cns = NULL;
 
                if (!(ArgusSorter)) {
                   if ((ArgusSorter = ArgusNewSorter(ArgusParser)) == NULL)
@@ -560,28 +599,57 @@ RaParseComplete (int sig)
                }
 
                ArgusSortQueue (ArgusSorter, agg->queue);
-               argus = ArgusCopyRecordStruct((struct ArgusRecordStruct *) agg->queue->array[0]);
                if (nflag == 0)
                   ArgusParser->eNflag = agg->queue->count;
                else
                   ArgusParser->eNflag = nflag > agg->queue->count ? agg->queue->count : nflag;
 
-               for (i = 1; i < ArgusParser->eNflag; i++)
-                  ArgusMergeRecords (agg, argus, (struct ArgusRecordStruct *)agg->queue->array[i]);
-               ArgusParser->ns = argus;
-
                for (i = 0; i < ArgusParser->eNflag; i++) {
                   tns = (struct ArgusRecordStruct *) agg->queue->array[i];
 
                   if (tns->agg != NULL) {
                      int count = tns->agg->queue->count;
+                     char nodeChar = 'A';
+
                      ArgusSortQueue (ArgusSorter, tns->agg->queue);
 
-                     nodeChar = 'A';
-                     if (ArgusParser->Aflag) 
-                        RaPrintTree (RaPathBuildTree (NULL, tns->agg->queue));
+                     for (x = 0; x < count; x++) {
+                        cns = (struct ArgusRecordStruct *) tns->agg->queue->array[x];
+                        cns->autoid = nodeChar;
+                        nodeChar = ((nodeChar == 'Z') ? 'a' : nodeChar + 1);
+                     }
+
+                     if (ArgusParser->Aflag) {
+                        struct RaPathNode *path = RaPathBuildPath (tns->agg->queue);
+                        char RaTreeBuffer[MAXSTRLEN];
+
+                        if (path) {
+                           char srcId[32], srcAddr[32], dstAddr[32];
+
+                           if (RaPrintTreeNode)
+                              RaPathInsertTree(RaTreeNode, path);
+
+                           cns = (struct ArgusRecordStruct *) tns->agg->queue->array[0];
+
+                           bzero(srcId,   sizeof(srcId));
+                           bzero(srcAddr, sizeof(srcAddr));
+                           bzero(dstAddr, sizeof(dstAddr));
+
+                           ArgusPrintSourceID (ArgusParser,  srcId, cns, 0);
+                           ArgusPrintSrcAddr (ArgusParser, srcAddr, cns, 0);
+                           ArgusPrintDstAddr (ArgusParser, dstAddr, cns, 0);
+
+                           RaPrintPath (path, RaTreeBuffer, sizeof(RaTreeBuffer));
+                           
+                           srcId[strlen(srcId) - 1] = '\0';
+                           srcAddr[strlen(srcAddr) - 1] = '\0';
+                           dstAddr[strlen(dstAddr) - 1] = '\0';
+
+                           printf("%s(%s::%s) %s\n", srcId, srcAddr, dstAddr, RaTreeBuffer);
+                           RaPathDeletePath(path);
+                        }
+                     }
 
-                     nodeChar = 'A';
                      for (x = 0; x < count; x++) {
                         cns = (struct ArgusRecordStruct *) tns->agg->queue->array[x];
                         RaSendArgusRecord (cns);
@@ -594,6 +662,16 @@ RaParseComplete (int sig)
             }
          }
 
+         ArgusDeleteAggregator(ArgusParser, ArgusFlowAggregator);
+         ArgusDeleteHashTable(ArgusHashTable);
+         ArgusDeleteSorter(ArgusSorter);
+         ArgusDeleteQueue(ArgusModelerQueue);
+
+         if (RaMapAddrTable.array != NULL) {
+            ArgusFree (RaMapAddrTable.array);
+            RaMapAddrTable.array =  NULL;
+         }
+
 #ifdef ARGUSDEBUG
          ArgusDebug (2, "RaParseComplete(caught signal %d)\n", sig);
 #endif
@@ -646,22 +724,25 @@ usage ()
    extern char version[];
 
    fprintf (stdout, "Rapath Version %s\n", version);
-   fprintf (stdout, "usage: %s \n", ArgusParser->ArgusProgramName);
-   fprintf (stdout, "usage: %s [options]\n", ArgusParser->ArgusProgramName);
+   fprintf (stdout, "usage:  %s \n", ArgusParser->ArgusProgramName);
+   fprintf (stdout, "usage:  %s [-f ralabel.conf] [ra-options] [- filter-expression]\n\n", ArgusParser->ArgusProgramName);
 
-   fprintf (stdout, "options: -D <level>       specify debug level\n");
-   fprintf (stdout, "         -n               don't convert numbers to names.\n");
-   fprintf (stdout, "         -r <filelist>    read argus data <filelist>. '-' denotes stdin.\n");
-   fprintf (stdout, "         -S <host[:port]> specify remote argus <host> and optional port number.\n");
-   fprintf (stdout, "         -t <timerange>   specify <timerange> for reading records.\n");
-   fprintf (stdout, "                 format:  timeSpecification[-timeSpecification]\n");
-   fprintf (stdout, "                          timeSpecification: [mm/dd[/yy].]hh[:mm[:ss]]\n");
-   fprintf (stdout, "                                              mm/dd[/yy]\n");
-   fprintf (stdout, "                                              -%%d{yMhdms}\n");
-   fprintf (stdout, "         -T <secs>        attach to remote server for T seconds.\n");
-#ifdef ARGUS_SASL
-   fprintf (stdout, "         -U <user/auth>   specify <user/auth> authentication information.\n");
+
+   fprintf (stdout, "options:  -f <racluster.conf>  read label rules from <ralabel.conf>.\n");
+   fprintf (stdout, "          -A                   print path graph\n");
+#ifdef ARGUSDEBUG
+   fprintf (stdout, "          -D <level>           specify debug level\n");
 #endif
+   fprintf (stdout, "          -q                   quiet mode.  Supress printing records.\n");
+   fprintf (stdout, "          -m flow key fields   modify the flow model for path data.\n");
+   fprintf (stdout, "          -M <option>          specify a Mode of operation.\n");
+   fprintf (stdout, "             Available modes:      \n");
+   fprintf (stdout, "               as              print path with AS information\n");
+   fprintf (stdout, "               addr            print path as list of addresses\n");
+   fprintf (stdout, "               node            print path as list of nodes\n");
+   fprintf (stdout, "               dist            print path with hop counts\n");
+   fprintf (stdout, "               tree            print path tree\n");
+
    fflush (stdout);
 
    exit(1);
@@ -844,14 +925,18 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
       if ((cns = ArgusCopyRecordStruct(ns)) == NULL)
          ArgusLog (LOG_ERR, "RaProcessThisRecord: ArgusCopyRecordStruct error %s", strerror(errno));
 
+      ArgusLabelRecord(parser, cns);
+
       if ((tns = RaProcessAggregation(parser, agg, cns)) != NULL) {
          if (tns->agg == NULL) 
             if ((tns->agg = ArgusCopyAggregator(ArgusFlowAggregator)) == NULL)
-               ArgusLog (LOG_ERR, "RaProcessThisRecod: ArgusNewAggregator error");
+               ArgusLog (LOG_ERR, "RaProcessThisRecod: ArgusCopyAggregator error");
 
          if (tns != NULL)
-            RaProcessAggregation(parser, tns->agg, ns);
+            RaProcessAggregation(parser, tns->agg, cns);
       }
+
+      ArgusDeleteRecordStruct(parser, cns);
    }
 }
 
@@ -918,7 +1003,8 @@ RaSendArgusRecord(struct ArgusRecordStruct *argus)
             ArgusPrintRecord(ArgusParser, buf, argus, MAXSTRLEN);
             
             if (ArgusParser->Aflag && (!(RaPrintAddr))) {
-               if (fprintf (stdout, "  %c   %s\n", nodeChar++, buf) < 0)
+               int ret = fprintf (stdout, "  %c   %s\n", argus->autoid, buf);
+               if (ret < 0)
                   RaParseComplete(SIGQUIT);
             } else
                if (fprintf (stdout, "%s\n", buf) < 0)
