@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,14 +23,14 @@
  * rarpwatch.c  - IPv4 and IPv6 arpwatch, driven by argus flow data.
  *
  * written by Carter Bullard
- * QoSient, LLC 2003
+ * QoSient, LLC
  *
  */
 
 /* 
- * $Id: //depot/argus-3.0.6/clients/examples/rarpwatch/rarpwatch.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/examples/rarpwatch/rarpwatch.c#9 $
+ * $DateTime: 2014/06/09 14:18:59 $
+ * $Change: 2847 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -105,7 +105,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
             ArgusLog (LOG_ERR, "ArgusClientInit: ArgusParseAggregator error");
         
       } else 
-         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL)) == NULL)
+         if ((parser->ArgusAggregator = ArgusNewAggregator(parser, NULL, ARGUS_RECORD_AGGREGATOR)) == NULL)
             ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
 
       if (correct == 0) {
@@ -139,11 +139,11 @@ void
 RaArgusInputComplete (struct ArgusInput *input)
 {
    if (ArgusProcessFileIndependantly) {
-      ArgusParser->ArgusCurrentFile = input;
+      ArgusParser->ArgusCurrentInput = input;
       RaParseComplete (0);
 
       ArgusParser->RaInitialized = 0;
-      ArgusParser->ArgusCurrentFile = NULL;
+      ArgusParser->ArgusCurrentInput = NULL;
       ArgusClientInit(ArgusParser);
    }
 
@@ -157,7 +157,7 @@ RaParseComplete (int sig)
 {
    struct ArgusModeStruct *mode = NULL;
    int i = 0, x = 0, nflag = ArgusParser->eNflag;
-   struct ArgusInput *file = ArgusParser->ArgusCurrentFile;
+   struct ArgusInput *file = ArgusParser->ArgusCurrentInput;
    char buf[MAXSTRLEN];
    int label;
 
@@ -384,11 +384,11 @@ usage ()
 
 
 void RaProcessThisRecord (struct ArgusParserStruct *, struct ArgusRecordStruct *);
-int RaValidateArpFlowRecord(struct ArgusParserStruct *, struct ArgusRecordStruct *);
+int RaValidateArpFlowRecord (struct ArgusParserStruct *, struct ArgusRecordStruct *);
 
 
 int
-RaValidateArpFlowRecord(struct ArgusParserStruct *parser, struct ArgusRecordStruct *ns)
+RaValidateArpFlowRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct *ns)
 {
    int retn = 1;
 
@@ -481,7 +481,7 @@ RaProcessThisRecord (struct ArgusParserStruct *parser, struct ArgusRecordStruct
          fretn = ArgusFilterRecord (fcode, argus);
       }
 
-      if (agg->labelstr) {
+      if (agg->grepstr) {
          struct ArgusLabelStruct *label;
          if (((label = (void *)argus->dsrs[ARGUS_LABEL_INDEX]) != NULL)) {
             if (regexec(&agg->lpreg, label->l_un.label, 0, NULL, 0))
