@@ -1,6 +1,6 @@
 /*
  * Argus Software
- * Copyright (c) 2000-2012 QoSient, LLC
+ * Copyright (c) 2000-2014 QoSient, LLC
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,9 +26,9 @@
  * QoSient, LLC
  *
  * 
- * $Id: //depot/argus-3.0.6/clients/examples/raservices/rauserdata.c#1 $
- * $DateTime: 2012/04/17 12:22:02 $
- * $Change: 2368 $
+ * $Id: //depot/argus/clients/examples/raservices/rauserdata.c#10 $
+ * $DateTime: 2014/05/12 10:49:40 $
+ * $Change: 2818 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -54,13 +54,6 @@ int RaSignatureLength = 16;
 int RaSrcSig = 1;
 int RaDstSig = 1;
 
-/*
-#define ARGUS_EXACT_MATCH       0x00
-#define ARGUS_LONGEST_MATCH     0x01
-#define ARGUS_ANY_MATCH         0x02
-#define ARGUS_NODE_MATCH        0x04
-*/
-
 int RaTestUserData(struct RaBinStruct *, struct ArgusRecordStruct *, struct ArgusRecordStruct *, int);
 void ArgusMergeUserData(struct RaBinStruct *, struct ArgusRecordStruct *, struct ArgusRecordStruct *);
 
@@ -75,8 +68,8 @@ char *ArgusAggregationConfig[2] = {
 };
 
 
-int RaSrcTestThreshold  = 6;
-int RaDstTestThreshold  = 4;
+int RaSrcTestThreshold  = 10;
+int RaDstTestThreshold  = 10;
 int RaMinStartThreshold = 4;
 
 void
@@ -115,7 +108,7 @@ ArgusClientInit (struct ArgusParserStruct *parser)
       }
 
       if ((parser->ArgusAggregator = ArgusParseAggregator(parser, NULL, ArgusAggregationConfig)) == NULL)
-         ArgusLog (LOG_ERR, "ArgusClientInit: ArgusNewAggregator error");
+         ArgusLog (LOG_ERR, "ArgusClientInit: ArgusParseAggregator error");
 
       for (i = 0; i < MAX_PRINT_ALG_TYPES; i++) {
          if (parser->RaPrintAlgorithmList[i] != NULL) {
@@ -158,7 +151,7 @@ RaParseComplete (int sig)
                ArgusLog (LOG_ERR, "RaParseComplete: ArgusNewSorter error %s", strerror(errno));
      
          if ((mode = ArgusParser->ArgusMaskList) != NULL) {
-            int x, i;
+            int x = 0, i = 0;
             while (mode) {
                for (x = 0; x < MAX_SORT_ALG_TYPES; x++) { 
                   if (!strncmp (ArgusSortKeyWords[x], mode->mode, strlen(ArgusSortKeyWords[x]))) {
@@ -172,7 +165,7 @@ RaParseComplete (int sig)
          }
 
          if (ArgusParser->ArgusAggregator->queue && (ArgusParser->ArgusAggregator->queue->count)) {
-#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(HAVE_SOLARIS)
+#if defined(__OpenBSD__) || defined(__FreeBSD__) || defined(__APPLE_CC__) || defined(__APPLE__) || defined(ARGUS_SOLARIS)
             printf ("Total Records %lld SrcThreshold %d Dst Threshold %d ", ArgusParser->ArgusTotalRecords, RaSrcTestThreshold, RaDstTestThreshold);
 #else
             printf ("Total Records %Ld SrcThreshold %d Dst Threshold %d ", ArgusParser->ArgusTotalRecords, RaSrcTestThreshold, RaDstTestThreshold);
